package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.user.AnchorSignedRatio;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-07 15:41:23
 * @Description 主播公会签约表持久层
 */
@Mapper
@Repository
public interface AgentAnchorDAO {

    /**
     * 查询是否签约
     *
     * @param userId 用户id
     * @return 条数
     */
    int countByUserId(@Param("userId") Long userId);

    /**
     * 是否签约指定公会
     *
     * @param agentId 公会id
     * @param userId  用户id
     * @return
     */
    int countByAgentIdAndUserId(@Param("agentId") Long agentId, @Param("userId") Long userId);

    /**
     * 获取工会id
     *
     * @param userId 用户id
     * @return 工会id
     */
    Long getAgentId(@Param("userId") Long userId);

    /**
     * 查询主播数量
     *
     * @param agentId agentId
     * @return 数量
     */
    int getAnchorCount(@Param("agentId") Long agentId);

    /**
     * 查询签约工会提成比例
     *
     * @param userId
     * @param agentId
     * @return
     */
    AnchorSignedRatio selectSignedRatio(@Param("userId") Long userId, @Param("agentId") Long agentId);
}
