package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.AnchorDO;
import com.starbuds.server.common.pojo.media.LiveItemVO;
import com.starbuds.server.common.pojo.user.AnchorInfoVO;
import com.starbuds.server.common.pojo.user.AnchorPkVO;
import com.starbuds.server.common.pojo.user.AnchorSingleChatInfoVO;
import com.starbuds.server.common.pojo.user.SingleChatUserVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2019-12-21 15:03:22
 * @Description 主播持久层
 */
@Mapper
@Repository
public interface AnchorDAO {
    /**
     * 新增
     */
    int insert(AnchorDO entity);

    /**
     * 根据主键更新
     */
    int updateById(AnchorDO entity);

    /**
     * 根据主键查询
     */
    AnchorDO selectById(@Param("userId") Long userId);

    /**
     * 查询直播状态
     */
    Integer findLiveStatus(@Param("userId") Long userId);

    /**
     * 查询推流状态
     */
    int findStreamStatus(@Param("userId") Long userId);


    /**
     * 查询直播封面
     *
     * @param userId
     * @return
     */
    String findLiveCover(@Param("userId") Long userId);

    /**
     * 根据 id 查询直播项
     *
     * @param userIds
     * @return
     */
    List<LiveItemVO> selectLiveItemsByIds(List<Long> userIds);

    /**
     * 查询主播详情
     *
     * @param userId
     * @return
     */
    AnchorInfoVO selectInfoById(@Param("userId") Long userId);

    /**
     * 查询粉丝团名称
     *
     * @param userId
     * @return
     */
    String selectFansNameById(@Param("userId") Long userId);

    /**
     * 更新粉丝团名称
     *
     * @param userId
     * @param fansName
     * @return
     */
    int updateFansName(@Param("userId") Long userId, @Param("fansName") String fansName);

    /**
     * 根据粉丝团名称统计
     *
     * @param fansName
     * @return
     */
    int countByFansName(@Param("fansName") String fansName);

    /**
     * 更新agentId
     *
     * @param userId  用户id
     * @param agentId 公会id
     * @return 条数
     */
    int updateAgentId(@Param("userId") Long userId, @Param("agentId") Long agentId);

    /**
     * 获取agentId
     *
     * @param userId 用户id
     * @return agentId
     */
    Long getAgentId(@Param("userId") Long userId);

    /**
     * 获取用户直播状态
     *
     * @param userId
     * @return
     */
    Integer getLiveStatus(@Param("userId") Long userId);

    /**
     * 查询主播认证时间
     *
     * @param userId
     * @return
     */
    Long selectAuthTime(@Param("userId") Long userId);

    /**
     * 查询单聊信息
     *
     * @param userId
     * @return
     */
    AnchorSingleChatInfoVO selectSingleChatInfo(@Param("userId") Long userId);

    /**
     * 获取单聊状态
     *
     * @param userId
     * @return
     */
    Integer selectSingleChatEnable(@Param("userId") Long userId);

    /**
     * 获取单聊主播用户
     *
     * @param userIds
     */
    List<SingleChatUserVO> selectSingleChatUserByIds(@Param("userIds") List<Long> userIds);

    /**
     * 获取pk主播列表
     *
     * @param userIds
     * @param userId
     * @param agentId
     * @param liveStatus
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<AnchorPkVO> selectAnchorPkList(@Param("userIds") List<Long> userIds,
                                        @Param("userId") Long userId,
                                        @Param("agentId") Long agentId,
                                        @Param("liveStatus") Integer liveStatus,
                                        @Param("pageCursor") int pageCursor,
                                        @Param("pageSize") int pageSize);

    /**
     * 获取pk主播列表大小
     *
     * @param userIds
     * @param userId
     * @param agentId
     * @param liveStatus
     * @return
     */
    int countAnchorPkList(@Param("userIds") List<Long> userIds,
                          @Param("userId") Long userId,
                          @Param("agentId") Long agentId,
                          @Param("liveStatus") Integer liveStatus);

    /**
     * 获取主播列表
     *
     * @param userIds
     * @return
     */
    List<AnchorPkVO> selectAnchorList(@Param("userIds") List<Long> userIds);

    /**
     * 查询是否允许售卖商品
     *
     * @param userId
     * @return
     */
    Integer selectSellEnable(@Param("userId") Long userId);

    /**
     * 随机获取一位在线主播
     *
     * @return
     */
    List<Long> selectRandomAnchorUserId();
}
