package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.pojo.user.ChannelDTO;
import com.starbuds.server.common.pojo.user.ChannelRatio;
import com.starbuds.server.service.user.pojo.ThirdPartyNoticeApi;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * ChannelDAO
 *
 * @author Lay
 * @date 2019/12/6
 */
@Mapper
@Repository
public interface ChannelDAO {
    /**
     * 根据 id 查询短信配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectSmsConfig(@Param("channelId") Long channelId);

    /**
     * 根据 id 查询一键登录配置
     *
     * @param channelId
     * @param clientType
     * @return
     */
    ChannelDO selectOneKeyLoginConfig(@Param("channelId") Long channelId, @Param("clientType") Integer clientType);

    /**
     * 获取微信oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectWxAppOAuthConfig(@Param("channelId") Long channelId);

    /**
     * 获取qq oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectQqConfig(@Param("channelId") Long channelId);

    /**
     * 获取微博oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectWeiboConfig(@Param("channelId") Long channelId);

    /**
     * 获取微博oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectHuaweiConfig(@Param("channelId") Long channelId);

    /**
     * 获取facebook oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectFacebookConfig(@Param("channelId") Long channelId);

    /**
     * 获取google oauth配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectGoogleConfig(@Param("channelId") Long channelId);

    /**
     * 获取支付方式配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectPayWayConfig(@Param("channelId") Long channelId);

    /**
     * 查询支付配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectPayConfig(@Param("channelId") Long channelId);

    /**
     * 获取配置
     *
     * @param channelId
     * @return
     */
    ChannelDO selectAppInitConfig(@Param("channelId") Long channelId);

    /**
     * 获取渠道名称
     *
     * @param channelId
     * @return
     */
    String selectChannelName(@Param("channelId") Long channelId);

    /**
     * 获取置顶推荐渠道 id
     *
     * @param channelId
     * @return
     */
    Long selectTopSuggestChannelId(@Param("channelId") Long channelId);

    /**
     * 查询单聊推荐渠道
     *
     * @param channelId
     * @return
     */
    Long selectSingleChatSuggestChannelId(@Param("channelId") Long channelId);

    /**
     * 查询 App 版本跟随渠道
     *
     * @param channelId
     * @return
     */
    Long selectAppVersionChannelId(@Param("channelId") Long channelId);

    /**
     * 查询 Banner 跟随渠道
     *
     * @param channelId
     * @return
     */
    Long selectBannerChannelId(@Param("channelId") Long channelId);

    /**
     * 获取所有渠道id
     *
     * @return
     */
    List<Long> selectAllChannelId();

    /**
     * 获取市场包状态
     *
     * @param channelId
     * @return
     */
    Integer selectMarketStatus(@Param("channelId") Long channelId);

    /**
     * 获取渠道分成比例
     *
     * @param channelIds
     * @return
     */
    List<ChannelRatio> selectRatioList(@Param("channelIds") List<Long> channelIds);

    /**
     * 获取百度akey
     *
     * @param channelId
     * @return
     */
    String selectBaiduAKey(@Param("channelId") Long channelId);

    /**
     * 获取渠道单聊分类
     *
     * @param channelId
     * @return
     */
    String selectSingleChatCategories(@Param("channelId") Long channelId);

    /**
     * 获取海报图跟随渠道id
     *
     * @param channelId
     * @return
     */
    Long selectPosterImageChannelId(@Param("channelId") Long channelId);

    /**
     * 获取是否必填邀请码
     *
     * @param channelId
     * @return
     */
    Integer selectInviteCodeEnable(@Param("channelId") Long channelId);

    /**
     * 是否开启人机校验
     *
     * @param channelId
     * @return
     */
    Integer selectMachineValidateEnable(@Param("channelId") Long channelId);

    /**
     * 查询(第三方)App手机登陆服务地址
     *
     * @param channelId
     * @return
     */
    String selectApiAuthByPhone(@Param("channelId") Long channelId);

    /**
     * 查询第三方用户通知类API信息列表
     *
     * @return
     */
    List<ThirdPartyNoticeApi> selectThirdPartyNoticeApiList();

    /**
     * 查询第三方用户通知API
     *
     * @param channelId
     * @return
     */
    ThirdPartyNoticeApi selectByChannelId(@Param("channelId") Long channelId);

    /**
     * 查询渠道邀请配置分组
     *
     * @param channelId
     * @return
     */
    Integer selectInviteConfigGroupId(@Param("channelId") Long channelId);

    /**
     * 获取渠道
     *
     * @param channelIds
     * @return
     */
    List<ChannelDTO> selectChannelsByChannelIds(@Param("channelIds") Set<Long> channelIds);

    /**
     * 获取语聊配置id
     *
     * @param channelId
     * @return
     */
    Long selectRtcConfigId(@Param("channelId") Long channelId);

    /**
     * 获取个播配置id
     *
     * @param channelId
     * @return
     */
    Long selectSingleRtcConfigId(@Param("channelId") Long channelId);

    /**
     * 获取打招呼状态  0关闭  1直播  2语聊 3打开直播和语聊
     *
     * @param channelId
     * @return
     */
    Integer selectGreetEnable(@Param("channelId") Long channelId);

    /**
     * 获取客服id
     *
     * @param channelId
     * @return
     */
    Long selectCustomerUserId(@Param("channelId") Long channelId);

    /**
     * 获取渠道
     *
     * @param channelIds
     * @return
     */
    @MapKey("channelId")
    Map<Long, ChannelDTO> selectChannelMapByChannelIds(@Param("channelIds") Set<Long> channelIds);
}
