package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import com.starbuds.server.common.pojo.user.FamilyAuditVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-07 15:41:23
 * @Description 家族审核表持久层
 */
@Mapper
@Repository
public interface FamilyAuditDAO {

    /**
     * 新增
     */
    int insert(FamilyAuditDO entity);

    /**
     * 查询是否存在未审核的申请
     *
     * @param familyId    家族id
     * @param userId      用户id
     * @param applyType   申请类型
     * @param auditStatus 审核状态
     * @return 条数
     */
    int countByFamilyId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus);

    /**
     * 获取家族签约记录
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @return
     */
    FamilyAuditDO selectFamilyAudit(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus);

    /**
     * 查询审核状态
     *
     * @param userId   用户id
     * @param familyId 家族id
     * @return 审核状态
     */
    Integer selectAuditStatus(@Param("userId") Long userId, @Param("familyId") Long familyId);

    /**
     * 获取用户最近一次申请信息
     *
     * @param userId    用户id
     * @param familyId  家族id
     * @param applyType 申请类型
     * @return 申请信息
     */
    FamilyAuditVO selectLastApplyInfo(@Param("userId") Long userId, @Param("familyId") Long familyId, @Param("applyType") Integer applyType);

    /**
     * 获取家族id
     *
     * @param userId
     * @param applyType
     * @return
     */
    Long selectFamilyId(@Param("userId") Long userId, @Param("applyType") Integer applyType);

    /**
     * 获取家族id（忽略状态）
     *
     * @param userId
     * @return
     */
    Long selectFamilyIdIgnoreStatus(@Param("userId") Long userId);

    /**
     * 判断是否有符合条件的数量
     *
     * @param userId
     * @param auditStatus
     * @return
     */
    int countByUserIdAndStatus(@Param("userId") Long userId, @Param("auditStatus") Integer auditStatus);

    /**
     * 获取最近一次离开家族的时间
     *
     * @param userId
     * @param familyId
     * @param applyType
     * @param auditStatus
     * @return
     */
    Long selectLastLeaveTime(@Param("userId") Long userId, @Param("familyId") Long familyId, @Param("applyType") Integer applyType, @Param("auditStatus") Integer auditStatus);

    /**
     * 更新状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditTime
     * @param auditMsg
     * @param adminId
     * @return
     */
    int updateStatus(@Param("auditId") Long auditId, @Param("auditStatus") Integer auditStatus, @Param("auditMsg") String auditMsg, @Param("auditTime") Long auditTime, @Param("adminId") Long adminId);

    /**
     * 批量插入
     *
     * @param list
     * @return
     */
    int batchInsert(@Param("list") List<FamilyAuditDO> list);
}
