package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.FamilyDO;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.user.FamilyDTO;
import com.starbuds.server.common.pojo.user.FamilyVO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-07 15:41:23
 * @Description 家族持久层
 */
@Mapper
@Repository
public interface FamilyDAO {

    /**
     * 是否存在
     *
     * @param familyId familyId
     * @return 是否存在
     */
    int countById(@Param("familyId") Long familyId);

    /**
     * 获取家族列表
     *
     * @param content 搜索的内容
     * @return 家族列表
     */
    List<FamilyDTO> selectFamilyPageList(@Param("content") String content,
                                         @Param("pageCursor") int pageCursor,
                                         @Param("pageSize") int pageSize);

    /**
     * 查询家族列表条数
     *
     * @param content 搜索的内容
     * @return 条数
     */
    int countFamilyPageList(@Param("content") String content);

    /**
     * 获取家族详情
     *
     * @param familyId 家族id
     * @return 家族详情
     */
    FamilyVO selectFamilyDetail(@Param("familyId") Long familyId);

    /**
     * 获取家族名
     *
     * @param familyId
     * @return
     */
    String selectFamilyName(@Param("familyId") Long familyId);

    /**
     * 获取家族
     *
     * @param familyId
     * @return
     */
    FamilyDO selectById(@Param("familyId") Long familyId);

    /**
     * 获取家族
     * @param familyId
     * @return
     */

    /**
     * 获取familyMap
     *
     * @param familyIds
     * @return
     */
    @MapKey("familyId")
    Map<Long, FamilyDO> selectFamilyMap(@Param("familyIds") Set<Long> familyIds);

    /**
     * 判断是否存在
     *
     * @param president
     * @return
     */
    int countByPresident(@Param("president") Long president);
}
