package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.FamilySkillAuditDO;
import com.starbuds.server.common.pojo.user.SkillUserQO;
import com.starbuds.server.common.pojo.user.SkillsSubmit;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/15 15:16
 * @Description 家族用户技能审核持久层
 */
@Mapper
@Repository
public interface FamilySkillAuditDAO {

    /**
     * 插入或更新
     *
     * @param skillAuditDO
     * @return
     */
    int insert(FamilySkillAuditDO skillAuditDO);

    /**
     * 获取认证中的技能
     *
     * @param familyId
     * @param userId
     * @param lastLeaveTime
     * @return
     */
    List<SkillsSubmit> selectSkills(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("lastLeaveTime") Long lastLeaveTime);

    /**
     * 获取最近一次记录
     *
     * @param userId
     * @param familyId
     * @param skillId
     * @param skillTagId
     * @param auditStatus
     */
    FamilySkillAuditDO selectLast(@Param("userId") Long userId, @Param("familyId") Long familyId, @Param("skillId") Long skillId, @Param("skillTagId") Long skillTagId, @Param("auditStatus") Integer auditStatus);

    /**
     * 根据用户查询是否提交
     *
     * @param userId
     * @param familyId
     * @return
     */
    int countByUserId(@Param("userId") Long userId, @Param("familyId") Long familyId);

    /**
     * 判断技能认证是否存在
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param auditStatus
     * @return
     */
    int countBySkillId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("skillId") Long skillId, @Param("auditStatus") Integer auditStatus);

    /**
     * 获取最近一次集合
     *
     * @param qoList
     * @return
     */
    List<FamilySkillAuditDO> selectLastList(@Param("qoList") List<SkillUserQO> qoList);
}
