package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.data.FamilySkillVO;
import com.starbuds.server.common.pojo.user.SkillAuth;
import com.starbuds.server.common.pojo.user.SkillUser;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/10/15 15:16
 * @Description 家族用户技能持久层
 */
@Mapper
@Repository
public interface FamilySkillDAO {

    /**
     * 获取已经认证的技能
     *
     * @param familyId
     * @param userId
     * @return
     */
    List<SkillAuth> selectSkillAuth(@Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 获取已经认证的技能
     *
     * @param familyId
     * @param userId
     * @return
     */
    List<SkillAuth> selectSkillAuthIgnoreVoiceMatch(@Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 判断技能是否认证
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    int countBySkillId(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("skillId") Long skillId);

    /**
     * 更新技能
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param orderPrice
     * @param orderTime
     * @param orderStatus
     * @return
     */
    int updateSkill(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("skillId") Long skillId, @Param("orderPrice") Long orderPrice, @Param("orderTime") String orderTime, @Param("orderStatus") Integer orderStatus);

    /**
     * 根据id查询
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    FamilySkillVO selectVOById(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("skillId") Long skillId);

    /**
     * 获取技能用户列表
     *
     * @param pageCursor
     * @param pageSize
     * @param skillId
     * @return
     */
    List<SkillUser> selectSkillUserList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("skillId") Long skillId);

    /**
     * 获取推荐技能用户列表
     *
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<SkillUser> selectRecommendSkillUserList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 获取技能用户列表大小
     *
     * @param skillId
     * @return
     */
    int countSkillUsers(@Param("skillId") Long skillId);

    /**
     * 获取技能用户列表大小
     *
     * @param skillId
     * @return
     */
    int countSkillUsersSimple(@Param("skillId") Long skillId);

    /**
     * 获取可用技能用户列表大小
     *
     * @param skillId
     * @return
     */
    int countAvailableSkillUsersSimple(@Param("skillId") Long skillId);

    /**
     * 获取可用并开启接单技能用户列表大小
     *
     * @param skillId
     * @return
     */
    List<SkillUser> selectSkillUserListSimple(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("skillId") Long skillId);

    /**
     * 获取可用技能用户列表大小
     *
     * @param skillId
     * @return
     */
    List<SkillUser> selectAvailableSkillUserList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("skillId") Long skillId);

    /**
     * 获取用户技能详情
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    SkillUser selectSkillUserById(@Param("familyId") Long familyId, @Param("userId") Long userId, @Param("skillId") Long skillId);
}
