package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.common.KeyValueMapDO;
import com.starbuds.server.common.pojo.db.FamilyUserDO;
import com.starbuds.server.common.pojo.user.UserSignedRatio;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-07 15:41:23
 * @Description 用户签约家族表持久层
 */
@Mapper
@Repository
public interface FamilyUserDAO {

    /**
     * 查询是否签约
     *
     * @param userId 用户id
     * @return 条数
     */
    int countByUserId(@Param("userId") Long userId);

    /**
     * 是否签约指定家族
     *
     * @param familyId 家族id
     * @param userId   用户id
     * @return
     */
    int countById(@Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 获取家族id
     *
     * @param userId 用户id
     * @return
     */
    Long selectFamilyId(@Param("userId") Long userId);

    /**
     * 查询主播数量
     *
     * @param familyId familyId
     * @return 数量
     */
    int selectUserCount(@Param("familyId") Long familyId);

    /**
     * 查询签约家族提成比例
     *
     * @param familyId
     * @param userId
     * @return
     */
    UserSignedRatio selectSignedRatio(@Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 获取家族
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, FamilyUserDO> selectFamilyIds(@Param("userIds") Set<Long> userIds);

    /**
     * 增加接通次数
     *
     * @param userId
     * @param familyId
     * @return
     */
    int incrementConnectTimes(@Param("userId") Long userId, @Param("familyId") Long familyId);

    /**
     * 增加匹配次数
     *
     * @param userId
     * @param familyId
     * @return
     */
    int incrementMatchTimes(@Param("userId") Long userId, @Param("familyId") Long familyId);

    /**
     * 获取用户技能认证数据
     *
     * @param userIds
     * @return
     */
    List<KeyValueMapDO> getUserSkillList(@Param("userIds") Set<Long> userIds);
}
