package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.FollowRelationDO;
import com.starbuds.server.common.pojo.user.FansUser;
import com.starbuds.server.common.pojo.user.FollowUser;
import com.starbuds.server.common.pojo.user.RelationDTO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2019-12-20 15:52:54
 * @Description 关注关系持久层
 */
@Mapper
@Repository
public interface FollowRelationDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(FollowRelationDO entity);

    /**
     * 根据主键更新
     *
     * @param entity
     * @return
     */
    int updateById(FollowRelationDO entity);

    /**
     * 根据主键查询
     *
     * @param relationId
     * @return
     */
    FollowRelationDO selectById(@Param("relationId") Long relationId);

    /**
     * 根据用户id删除
     *
     * @param userId
     * @param targetId
     * @return
     */
    Integer deleteByUserId(@Param("userId") Long userId, @Param("targetId") Long targetId);


    /**
     * 新增或更新
     *
     * @param entity
     * @return
     */
    int insertOrUpdate(FollowRelationDO entity);

    /**
     * 根据 id 查询关系
     *
     * @param relationIdA
     * @param relationIdB
     * @return
     */
    FollowRelationDO selectByIds(@Param("relationIdA") String relationIdA, @Param("relationIdB") String relationIdB);

    /**
     * 更新关系
     *
     * @param relationId
     * @param direction
     * @param status
     * @param whereDirection
     * @param whereStatus
     * @return
     */
    int updateByDirection(@Param("relationId") String relationId, @Param("direction") Integer direction, @Param("status") Integer status, @Param("whereDirection") Integer whereDirection, @Param("whereStatus") Integer whereStatus);

    /**
     * 条件插入
     *
     * @param followRelation
     * @param relationIdB
     * @return
     */
    int insertWhereNotExists(@Param("bean") FollowRelationDO followRelation, @Param("relationIdB") String relationIdB);

    /**
     * 根据 用户 id 查询关系
     *
     * @param userId
     * @param targetUserIds
     * @return
     */
    List<FollowRelationDO> selectAllByUserId(@Param("userId") Long userId, @Param("targetUserIds") List<Long> targetUserIds);

    /**
     * 获取用户id
     *
     * @param userId
     * @return
     */
    List<RelationDTO> selectFollows(@Param("userId") Long userId);

    /**
     * 获取用户
     *
     * @param userId
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<FollowUser> selectFollowsUser(@Param("userId") Long userId, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);


    Set<Long> selectFollowsUserIds(@Param("userId") Long userId);

    /**
     * 获取用户id
     *
     * @param userId
     * @return
     */
    List<RelationDTO> selectFans(@Param("userId") Long userId);

    /**
     * 获取用户
     *
     * @param userId
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<FansUser> selectFansUser(@Param("userId") Long userId, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 是否关注
     *
     * @param userId
     * @param targetId
     * @return
     */
    int isFollower(@Param("userId") Long userId, @Param("targetId") Long targetId);

    /**
     * 获取粉丝用户id
     *
     * @param userId
     * @return
     */
    List<Long> selectFansUserIds(@Param("userId") Long userId);

    /**
     * 统计粉丝数量
     *
     * @param userId
     * @return
     */
    int countFans(@Param("userId") Long userId);

    /**
     * 统计关注数
     *
     * @param userId
     * @return
     */
    int countFollow(@Param("userId") Long userId);

    /**
     * 获取历史记录
     *
     * @param endTime
     * @param cursor
     * @param pageSize
     * @return
     */
    List<FollowRelationDO> getFollowHis(@Param("endTime") Long endTime, @Param("cursor") int cursor, @Param("pageSize") int pageSize);
}
