package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.user.*;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * tb_user
 *
 * @author Lay
 * @date 2019/12/4
 */
@Mapper
@Repository
public interface UserDAO {
    /**
     * 创建用户
     *
     * @param userDO
     * @return
     */
    int insert(UserDO userDO);

    /**
     * 根据用户号查询 id
     *
     * @param userNo
     * @return
     */
    Long selectIdByNo(@Param("userNo") Long userNo);

    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insertSelective(UserDO entity);

    /**
     * 根据主键更新
     *
     * @param entity 用户
     * @return 更新条数
     */
    int updateById(UserDO entity);

    /**
     * 更新昵称
     *
     * @param userId
     * @param userName
     * @return
     */
    int updateName(@Param("userId") Long userId, @Param("userName") String userName);

    /**
     * 根据主键查询
     *
     * @param userId 用户id
     * @return 用户
     */
    UserDO selectById(@Param("userId") Long userId);

    int setUserIsFirstRechargedClick(@Param("userId") Long userId);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<UserDO> selectPageList(QueryWrapper qw);

    /**
     * 用户是否存在
     *
     * @param userId 用户id
     * @return 条数
     */
    int countById(@Param("userId") Long userId);

    /**
     * 根据id查询
     *
     * @param userIds 用户id集合
     * @return 用户
     */
    List<UserDO> findByIds(@Param("userIds") List<Long> userIds,
                           @Param("pageCursor") int pageCursor,
                           @Param("pageSize") int pageSize);

    /**
     * 查询条数
     *
     * @param userIds 用户id集合
     * @return 条数
     */
    int countByIds(@Param("userIds") List<Long> userIds);

    /**
     * 查询是否主播
     *
     * @param userId 用户id
     * @return 是否主播
     */
    Integer isAnchor(@Param("userId") Long userId);

    /**
     * 查询是否在线
     *
     * @param userId
     * @return
     */
    Integer selectIsOnlineById(@Param("userId") Long userId);

    /**
     * 获取在线用户数
     *
     * @return
     */
    Long selectOnlineCount();

    /**
     * 查询是否在线
     *
     * @param userIds
     * @return 用户id:在线状态
     */
    @MapKey("userId")
    Map<Long, UserDO> selectIsOnlineByIds(@Param("userIds") Set<Long> userIds);

    /**
     * 更新在线状态
     *
     * @param userId
     * @param isOnline
     * @return
     */
    int updateIsOnline(@Param("userId") Long userId,
                       @Param("isOnline") Integer isOnline,
                       @Param("userUpdateTime") Long userUpdateTime);

    /**
     * 查询财富等级
     *
     * @param userId 用户id
     * @return 财富等级
     */
    Integer selectWealthLevel(@Param("userId") Long userId);

    /**
     * 查询魅力等级
     *
     * @param userId 用户id
     * @return 魅力等级
     */
    Integer selectCharmLevel(@Param("userId") Long userId);

    /**
     * 查询星耀等级
     *
     * @param userId
     * @return
     */
    Integer selectStarLevel(@Param("userId") Long userId);

    /**
     * 更新财富等级
     *
     * @param userId      用户id
     * @param wealthLevel 财富等级
     * @return 更新条数
     */
    int updateWealthLevel(@Param("userId") Long userId,
                          @Param("wealthLevel") int wealthLevel);

    /**
     * 更新魅力等级
     *
     * @param userId     用户id
     * @param charmLevel 魅力等级
     * @return 更新条数
     */
    int updateCharmLevel(@Param("userId") Long userId,
                         @Param("charmLevel") int charmLevel);

    /**
     * 更新星耀等级
     *
     * @param userId
     * @param starLevel
     * @return
     */
    int updateStarLevel(@Param("userId") Long userId, @Param("starLevel") int starLevel);

    /**
     * 更新靓号
     *
     * @param userId 用户id
     * @param niceNo 靓号
     * @return 更新条数
     */
    int updateNiceNo(@Param("userId") Long userId, @Param("niceNo") Long niceNo);

    /**
     * 获取用户卡片信息
     *
     * @param userId
     * @return
     */
    UserCardVO selectCardById(@Param("userId") Long userId);

    /**
     * 获取用户卡片信息只返回用户信息
     *
     * @param userId
     * @return
     */
    UserCardVO selectCardOnlyReturnUserInfo(@Param("userId") Long userId);

    /**
     * 获取用户
     *
     * @param userIds
     */
    @MapKey("userId")
    Map<Long, UserCardVO> selectCardMapByIds(@Param("userIds") Set<Long> userIds);

    /**
     * 更新用户状态
     *
     * @param userId 用户id
     * @param status 状态
     * @return 条数
     */
    int updateStatus(@Param("userId") Long userId, @Param("status") Integer status);

    /**
     * 获取用户状态
     *
     * @param userId 用户id
     * @return 状态
     */
    Integer getStatus(@Param("userId") Long userId);

    /**
     * 查询用户基本信息
     *
     * @param userId 用户id
     * @return 基本信息
     */
    UserBasicInfo selectBasicInfo(Long userId);

    /**
     * 获取用户
     *
     * @param params
     * @return
     */
    UserBasicInfo selectByUserNoOrNiceNo(@Param("params") Long params);

    /**
     * 查询用户id
     *
     * @param pageCursor 光标
     * @param pageSize   分页大小
     * @return 用户id
     */
    List<Long> getUserIds(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 根据 id 查询用户信息
     *
     * @param userIds
     * @return
     */
    List<UserCardVO> selectNameAndAvatarByIds(@Param("userIds") Collection<Long> userIds);

    /**
     * 根据 id 查询头像昵称
     *
     * @param userId
     * @return
     */
    UserCardVO selectNameAndAvatarById(Long userId);

    /**
     * 根据 id 查询用户信息
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, PeopleNearbyVO> selectPeopleNearbyMapByIds(@Param("userIds") Collection<Long> userIds);

    /**
     * 根据昵称统计
     *
     * @param userName
     * @param userId
     * @return
     */
    int countByName(@Param("userName") String userName, @Param("userId") Long userId);

    /**
     * 查询昵称是否存在
     *
     * @param userName
     * @return
     */
    Integer selectExistUserName(@Param("userName") String userName);

    /**
     * 查询修改昵称次数
     *
     * @param userId
     * @return
     */
    Integer selectRenameTimesById(@Param("userId") Long userId);


    /**
     * 查询用户名
     *
     * @param userId
     * @return
     */
    String selectNameById(@Param("userId") Long userId);

    /**
     * 更新位置
     *
     * @param userId
     * @param userLatitude
     * @param userLongitude
     * @param areaId
     * @param locationCity  定位城市
     * @return
     */
    int updatePosition(@Param("userId") Long userId,
                       @Param("userLatitude") Double userLatitude,
                       @Param("userLongitude") Double userLongitude,
                       @Param("areaId") Integer areaId,
                       @Param("locationCity") String locationCity);


    /**
     * 获取用户手机号
     *
     * @param userId
     * @return
     */
    String selectUserPhone(@Param("userId") Long userId);

    /**
     * 统计是否充值
     *
     * @param userId
     * @return
     */
    int countHasRechargedById(@Param("userId") Long userId);

    /**
     * 查询用户靓号
     *
     * @param userId
     * @return
     */
    Long selectNiceNo(@Param("userId") Long userId);

    /**
     * 查询用户星芽号
     *
     * @param userId
     * @return
     */
    Long selectUserNo(@Param("userId") Long userId);

    /**
     * 查询用户 id
     *
     * @param userNo
     * @return
     */
    Long selectUserId(@Param("userNo") Long userNo);

    /**
     * 是否充值过
     *
     * @param userId
     * @return
     */
    Integer selectHasRecharged(@Param("userId") Long userId);

    /**
     * 更新手机号
     *
     * @param userId
     * @param userPhone
     * @return
     */
    int updatePhone(@Param("userId") Long userId, @Param("userPhone") String userPhone);

    /**
     * 获取星座
     *
     * @param userId
     * @return
     */
    Integer selectUserConstellation(@Param("userId") Long userId);

    /**
     * 获取国家id
     *
     * @param userId
     * @return
     */
    Integer selectCountryId(@Param("userId") Long userId);

    /**
     * 更新用户区域 id
     *
     * @param userId
     * @param areaId
     * @return
     */
    int updateAreaId(@Param("userId") Long userId, @Param("areaId") Integer areaId);

    /**
     * 获取渠道id
     *
     * @param userId
     * @return
     */
    Long selectChannelId(@Param("userId") Long userId);

    /**
     * 更新用户头像
     *
     * @param userId
     * @param userAvatar
     * @return
     */
    int updateUserAvatar(@Param("userId") Long userId, @Param("userAvatar") String userAvatar);

    /**
     * 获取用户
     *
     * @param userIds
     * @return
     */
    List<UserDTO> selectUsers(@Param("userIds") List<Long> userIds);

    /**
     * 根据用户id列表查询总数（判断是否用户id都存在）
     *
     * @param userIds
     * @return
     */
    int selectCountByUserIds(@Param("userIds") Set<Long> userIds);

    /**
     * 获取注册时间
     *
     * @param userId
     * @return
     */
    Long selectCreateTime(@Param("userId") Long userId);

    /**
     * 获取用户Map
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, UserDO> selectUserMapByIds(@Param("userIds") Collection<Long> userIds);

    /**
     * 判断是否封禁
     *
     * @param userId
     * @return
     */
    Integer selectUserStatus(@Param("userId") Long userId);

    /**
     * 获取语音聊天状态
     *
     * @param userId
     * @return
     */
    Integer selectVoiceChatStatus(@Param("userId") Long userId);

    /**
     * 更新语音聊天状态
     *
     * @param userId
     * @param voiceChatStatus
     * @return
     */
    int updateVoiceChatStatus(@Param("userId") Long userId, @Param("voiceChatStatus") Integer voiceChatStatus);

    /**
     * 更新状态
     *
     * @param userIds
     * @param userStatus
     */
    int updateUserStatus(@Param("userIds") List<Long> userIds, @Param("userStatus") Integer userStatus);

    /**
     * 获取基本信息
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, UserBasicInfo> selectBasicInfoMap(@Param("userIds") Set<Long> userIds);

    /**
     * 获取用户简单信息
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, UserSimpleVO> selectSimpleInfoMap(@Param("userIds") Collection<Long> userIds);

    /**
     * 获取用户在线状态信息
     *
     * @param userIds
     * @return
     */
    @MapKey("userId")
    Map<Long, UserOnline> selectUserOnlineMap(@Param("userIds") Collection<Long> userIds);

    /**
     * 获取用户简单信息
     *
     * @return
     */
    Integer countOnlineUser();

    /**
     * 获取用户简单信息
     *
     * @return
     */
    List<Long> selectPageListOnlineUser(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);
}
