package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.UserFollowRelationDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 关注关系 Mapper
 */
@Mapper
@Repository
public interface UserFollowRelationDAO {

    /**
     * 插入数据
     */
    int insert(UserFollowRelationDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<UserFollowRelationDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(UserFollowRelationDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    UserFollowRelationDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    UserFollowRelationDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<UserFollowRelationDO> selectPageList(QueryWrapper qw);

    /**
     * 获取用户第一条数据
     *
     * @return
     */
    UserFollowRelationDO getFirstUserFollow();

    /**
     * 获取用户和操作对象的关系数据
     *
     * @param userId
     * @param targetId
     * @return
     */
    List<UserFollowRelationDO> getUserRelation(@Param("userId") Long userId, @Param("targetId") Long targetId);

    /**
     * 更新是否是朋友关系
     *
     * @param userId
     * @param targetId
     * @param isFriend
     * @param now
     */
    void updateIsFriend(@Param("userId") Long userId, @Param("targetId") Long targetId, @Param("isFriend") int isFriend, @Param("now") Long now);

    /**
     * 删除好友
     *
     * @param userId
     * @param targetId
     * @param now
     */
    void delFriend(@Param("userId") Long userId, @Param("targetId") Long targetId, @Param("now") Long now);

    /**
     * 删除关系
     *
     * @param userId
     * @param targetId
     * @param followType
     * @param now
     */
    void delRelation(@Param("userId") Long userId, @Param("targetId") Long targetId, @Param("followType") int followType, @Param("now") Long now);
}