package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.UserInterviewHisDO;
import com.starbuds.server.common.pojo.user.IntervieweeVO;
import com.starbuds.server.common.pojo.user.IntervieweeVOV2;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/20
 */
@Mapper
@Repository
public interface UserInterviewHisDAO {
    /**
     * 创建受访记录
     *
     * @param userInterviewHisDO
     * @return
     */
    int insertOrUpdate(UserInterviewHisDO userInterviewHisDO);

    /**
     * 更新用户未读访客为已读
     *
     * @param userId
     * @return
     */
    int updateReadByUser(Long userId);

    /**
     * 获取用户未读访客
     *
     * @param userId
     * @return
     */
    Long selectUnReadByUser(Long userId);

    /**
     * 查询记录
     *
     * @param pageCursor
     * @param pageSize
     * @param userId
     * @return
     */
    List<IntervieweeVO> selectAll(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("userId") Long userId);

    /**
     * 查询记录
     *
     * @param pageCursor
     * @param pageSize
     * @param userId
     * @return
     */
    List<IntervieweeVOV2> selectAllV2(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("userId") Long userId, @Param("dateInt") Integer dateInt);

    /**
     * 统计
     *
     * @param userId
     * @return
     */
    Integer selectAllV2Count(@Param("userId") Long userId, @Param("dateInt") Integer dateInt);

    /**
     * 最近看过的用户id列表
     *
     * @param visitorUserId
     * @param limit
     * @return
     */
    List<Long> selectRecentlySeenUserIds(@Param("visitorUserId") Long visitorUserId, @Param("limit") int limit);

    /**
     * 统计
     *
     * @param userId
     * @return
     */
    int count(@Param("userId") Long userId);

    /**
     * 统计记录
     *
     * @param userId
     * @return
     */
    Integer selectInterviewCount(@Param("userId") Long userId, @Param("visitorUserId") Long visitorUserId, @Param("dateInt") Integer dateInt);

    /**
     * 统计谁看过我记录
     *
     * @return
     */
    Long selectInterviewCountByUser(@Param("userId") Long userId, @Param("dateInt") Integer dateInt);

    /**
     * 统计谁看过我用户数
     *
     * @return
     */
    Long selectIntervieweeTotalByUser(@Param("userId") Long userId);

}
