package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.UserIntimacyRelationDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Author lujiao
 * @DateTime 2021/7/22 15:50
 * @Description
 */
@Mapper
@Repository
public interface UserIntimacyRelationDAO {

    /**
     * 插入数据
     */
    int insert(UserIntimacyRelationDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<UserIntimacyRelationDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(UserIntimacyRelationDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    UserIntimacyRelationDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    UserIntimacyRelationDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<UserIntimacyRelationDO> selectPageList(QueryWrapper qw);

    /**
     * 获取用户亲密关系
     *
     * @param userId
     * @param targetId
     * @return
     */
    UserIntimacyRelationDO getUserIntimacy(@Param("userId") Long userId, @Param("targetId") Long targetId);

    /**
     * 增长用户亲密度
     *
     * @param userIntimacyRelationId
     * @param increaseValue
     * @param updateTime
     * @return
     */
    int increaseUserIntimacyVal(@Param("userIntimacyRelationId") Long userIntimacyRelationId,
                                @Param("increaseValue") BigDecimal increaseValue,
                                @Param("updateTime") Long updateTime);

    /**
     * 亲密度列表
     *
     * @param targetUserId
     * @return
     */
    List<UserIntimacyRelationDO> getUserIntimacyRelation(Long targetUserId);

}
