package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.UserScoreDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-18 16:23:32
 * @Description 用户积分表持久层
 */
@Mapper
@Repository
public interface UserScoreDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(UserScoreDO entity);

    /**
     * 新增或更新
     *
     * @param userScoreDO
     * @return
     */
    int insertOrUpdate(UserScoreDO userScoreDO);

    /**
     * 更新
     *
     * @param userScoreDO
     * @return
     */
    int update(UserScoreDO userScoreDO);

    /**
     * CAS 更新积分
     *
     * @param scoreId
     * @param scoreValue
     * @param whereVersion
     * @return
     */
    int updateCas(@Param("scoreId") String scoreId, @Param("scoreValue") Long scoreValue, @Param("whereVersion") Long whereVersion);

    /**
     * 根据主键查询
     *
     * @param scoreId
     * @return
     */
    UserScoreDO selectById(@Param("scoreId") String scoreId);

    /**
     * 更新积分值
     *
     * @param scoreId
     * @param scoreValue
     * @param whereVersion
     * @param updateTime
     * @return
     */
    int updateValue(@Param("scoreId") String scoreId, @Param("scoreValue") Long scoreValue, @Param("whereVersion") Long whereVersion, @Param("updateTime") long updateTime);

    /**
     * 根据 id 查询积分
     *
     * @param scoreId
     * @return
     */
    Long selectValueById(@Param("scoreId") String scoreId);

    /**
     * 获取值
     *
     * @param userIds
     * @param scoreType
     * @return
     */
    Long selectSumValue(@Param("userIds") List<Long> userIds, @Param("scoreType") Integer scoreType);

    /**
     * 根据 ID 统计
     *
     * @param scoreId
     * @return
     */
    int countById(@Param("scoreId") String scoreId);

    /**
     * 获取用户积分等级排名
     *
     * @param scoreType
     * @return
     */
    List<UserScoreDO> selectActivitiesMedalPersonRankList(@Param("scoreType") Integer scoreType);
}
