package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.db.UserSecurityDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Map;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/3
 */
@Mapper
@Repository
public interface UserSecurityDAO {
    /**
     * 创建记录
     *
     * @param userSecurityDO
     * @return
     */
    int insert(UserSecurityDO userSecurityDO);

    /**
     * 查询是否市场审核人员
     *
     * @param userPhone
     * @param areaCode
     * @return
     */
    Integer selectIsMarketAuditorByPhone(@Param("userPhone") String userPhone, @Param("areaCode") String areaCode);

    /**
     * 根据手机号统计
     *
     * @param userPhone
     * @param areaCode
     * @return
     */
    int countByPhone(@Param("userPhone") String userPhone, @Param("areaCode") String areaCode);

    /**
     * 根据手机号查询用户 id
     *
     * @param userPhone
     * @param areaCode
     * @return
     */
    Long selectIdByPhone(@Param("userPhone") String userPhone, @Param("areaCode") String areaCode);

    /**
     * 更新 IM 令牌
     *
     * @param userId
     * @param imToken
     * @return
     */
    int updateImToken(@Param("userId") Long userId, @Param("imToken") String imToken);

    /**
     * 绑定手机号
     *
     * @param userId
     * @param userPhone
     * @param areaCode
     * @return
     */
    int updatePhone(@Param("userId") Long userId, @Param("userPhone") String userPhone, @Param("areaCode") String areaCode);

    /**
     * 根据id查询手机号
     *
     * @param userId
     * @return
     */
    String selectPhoneById(@Param("userId") Long userId);

    /**
     * 获取区号
     *
     * @param userId
     * @return
     */
    String selectAreaCode(@Param("userId") Long userId);

    /**
     * 获取security
     *
     * @param userId
     * @return
     */
    UserSecurityDO selectById(@Param("userId") Long userId);

    /**
     * 查询用户密码
     *
     * @param userId
     * @return
     */
    String selectUserPass(@Param("userId") Long userId);

    /**
     * 更新密码
     *
     * @param userId
     * @param userPass
     * @return
     */
    int updateUserPass(@Param("userId") Long userId, @Param("userPass") String userPass);

    /**
     * 根据手机号码获得用户id
     *
     * @param phoneList 用户手机列表
     * @param areaCode  区号
     * @return key:手机号; value:用户id
     */
    @MapKey("userPhone")
    Map<String, UserDO> selectPhoneUserIdMap(@Param("phoneList") Set<String> phoneList, @Param("areaCode") String areaCode);
}
