package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.UserWatchRelationDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 用户守护表 Mapper
 *
 * @author lizhi
 */
@Mapper
@Repository
public interface UserWatchRelationDAO {

    /**
     * 插入数据
     */
    int insert(UserWatchRelationDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<UserWatchRelationDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(UserWatchRelationDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    UserWatchRelationDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    UserWatchRelationDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<UserWatchRelationDO> selectPageList(QueryWrapper qw);

    /**
     * 获取未过期的守护
     *
     * @param userId
     * @param targetUserId
     * @param watchId
     * @param now
     * @return
     */
    UserWatchRelationDO getExistWatch(@Param("userId") Long userId, @Param("targetUserId") Long targetUserId, @Param("watchId") Long watchId, @Param("now") Long now);

    /**
     * 延长过期时间
     *
     * @param userWatchRelationId
     * @param increaseTime
     * @param updateTime
     */
    void updateExpireTime(@Param("userWatchRelationId") Long userWatchRelationId, @Param("increaseTime") long increaseTime,
                          @Param("increaseWatchValue") Integer increaseWatchValue, @Param("updateTime") long updateTime);

    /**
     * 获取即将到期的时间
     *
     * @return
     */
    List<UserWatchRelationDO> getExpiredRelation(@Param("startTime") Long startTime, @Param("endTime") Long endTime);

    /**
     * 守护对象
     *
     * @param targetUserId
     * @return
     */
    List<UserWatchRelationDO> getUserWatchRelation(@Param("targetUserId") Long targetUserId, @Param("now") Long now, @Param("pageCursor") Integer pageCursor, @Param("pageSize") Integer pageSize);


    /**
     * 守护对象总和
     *
     * @param targetUserId
     * @param now
     * @return
     */
    Integer getUserWatchRelationCount(@Param("targetUserId") Long targetUserId, @Param("now") Long now);

    /**
     * 获取用户守护值最高的那个
     *
     * @param userId
     * @param targetUserId
     * @param now
     * @return
     */
    UserWatchRelationDO getUserTopWatchRelation(@Param("userId") Long userId, @Param("targetUserId") Long targetUserId, @Param("now") long now);

    /**
     * 获取已经开通的列表
     *
     * @param userId
     * @param targetUserId
     * @param now
     * @return
     */
    List<UserWatchRelationDO> getExistWatchList(@Param("userId") Long userId, @Param("targetUserId") Long targetUserId, @Param("now") Long now);

    /**
     * 获取历史列表
     *
     * @param scoreLimit
     * @param cursor
     * @param pageSize
     * @return
     */
    List<UserWatchRelationDO> getHisList(@Param("scoreLimit") Long scoreLimit, @Param("cursor") Integer cursor, @Param("pageSize") Integer pageSize);
}
