package com.starbuds.server.service.user.dao;

import com.starbuds.server.common.pojo.db.WatchConfigDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 守护配置表 Mapper
 */
@Mapper
@Repository
public interface WatchConfigDAO {

    /**
     * 插入数据
     */
    int insert(WatchConfigDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<WatchConfigDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(WatchConfigDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    WatchConfigDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    WatchConfigDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<WatchConfigDO> selectPageList(QueryWrapper qw);

    /**
     * 获取配置列表
     *
     * @param qw
     * @return
     */
    List<WatchConfigDO> getWatchConfigList(QueryWrapper qw);

    /**
     * 软删除
     *
     * @param watchConfigId
     * @return
     */
    Integer delWatchConfig(@Param("watchConfigId") Integer watchConfigId);

    /**
     * 通过ID获取
     *
     * @param watchConfigId
     * @return
     */
    WatchConfigDO getWatchConfigById(@Param("watchConfigId") Long watchConfigId);

    /**
     * 通过ID获取列表
     *
     * @param watchIds
     * @return
     */
    List<WatchConfigDO> getWatchConfigListByIds(@Param("watchIds") Set<Long> watchIds);

    /**
     * 获取可用的配置列表
     *
     * @return
     */
    List<WatchConfigDO> getEnableWatchConfigList();

    /**
     * 通过是否只允许拍卖显示
     *
     * @param isAuctionOnly
     * @return
     */
    List<WatchConfigDO> getEnableWatchConfigListByAuction(@Param("isAuctionOnly") Integer isAuctionOnly);
}