package com.starbuds.server.service.user.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.SignType;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.user.AdminService;
import com.starbuds.server.service.user.dao.AdminDAO;
import com.starbuds.server.service.user.pojo.admin.AdminSecurityDO;
import com.starbuds.server.service.user.pojo.admin.ManageTokenParam;
import com.starbuds.server.service.user.provider.AdminTokenProvider;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;

/**
 * @author aoe
 * @date 2022/1/10
 */
@Service(version = "1.0.0")
public class AdminServiceImpl implements AdminService {

    @Resource
    private AdminDAO adminDAO;

    @Reference(version = "1.0.0")
    RedisService redisService;

    public String getAdminPkg(String phone) {
        String token = getAdminToken(phone);
        if (token == null) {
            return null;
        }
        return getAdminPkgByToken(token);
    }

    private String getAdminToken(String phone) {
        AdminSecurityDO admin = adminDAO.selectByAccount(phone);
        if (admin == null) {
            return null;
        }
        Long adminId = admin.getAdminId();
        String token = getTokenByCache(adminId);
        if (StringUtils.isBlank(token)) {
            token = RandomUtil.randomString(64);
            // 令牌缓存
            redisService.hashPut(RedisUserKeys.ADMIN_TOKEN, String.valueOf(adminId), token);
        }

        // 加密令牌
        ManageTokenParam tokenParam = new ManageTokenParam();
        tokenParam.setAdminId(adminId);
        tokenParam.setRoleId(admin.getRoleId());
        tokenParam.setToken(token);
        tokenParam.setTimestamp(System.currentTimeMillis());

        return AdminTokenProvider.encryptToken(tokenParam);
    }

    private String getTokenByCache(Long adminId) {
        return "" + redisService.hashGet(RedisUserKeys.ADMIN_TOKEN, "" + adminId);
    }

    private String getAdminPkgByToken(String token) {
        // Admin head pkg
        HeaderPackage headerPackage = new HeaderPackage();
        headerPackage.setClientType(ClientType.Web);
        headerPackage.setAppType(AppType.Admin);
        headerPackage.setPackageName("com.starbuds.manage");
        headerPackage.setTimestamp(System.currentTimeMillis());
        headerPackage.setDeviceId("deviceId");
        headerPackage.setToken(token);
        headerPackage.setAppVersion("1.0.0");
        headerPackage.setSignType(SignType.MD5);
        String toSign = headerPackage.toQueryString();
        headerPackage.setSign(SecureUtil.md5(toSign + "pq" + SignType.MD5));
        String headerJson = JSON.toJSONString(headerPackage);
        return Base64.encode(headerJson);
    }
}
