package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.media.StreamStatusEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.message.push.PushActionType;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.user.AnchorOperationEnum;
import com.starbuds.server.common.constant.user.AnchorPkSearchType;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.media.LiveItemVO;
import com.starbuds.server.common.pojo.message.im.live.LiveUserBlackMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveUserForbiddenMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveUserMasterMsg;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.task.PushService;
import com.starbuds.server.common.service.user.AgentService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.LiveUserProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/21 14:45
 * @Description 主播服务
 */
@Slf4j
@Service(version = "1.0.0")
public class AnchorServiceImpl implements AnchorService {

    @Autowired
    AnchorAuditDAO anchorAuditDAO;

    @Autowired
    AnchorDAO anchorDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    AgentService agentService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Resource
    LiveUserProvider liveUserProvider;

    @Resource
    UserDAO userDAO;

    @Resource
    AnchorMasterDAO anchorMasterDAO;

    @Resource
    AnchorMasterHisDAO anchorMasterHisDAO;

    @Resource
    AnchorForbiddenHisDAO anchorForbiddenHisDAO;

    @Resource
    AnchorFansDAO anchorFansDAO;

    @Resource
    UserLoginHisDAO userLoginHisDAO;

    @Resource
    AnchorDisableHisDAO anchorDisableHisDAO;

    @Resource
    AnchorBlackDAO anchorBlackDAO;

    @Resource
    AnchorBlackHisDAO anchorBlackHisDAO;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Resource
    FollowRelationDAO followRelationDAO;

    @Reference(version = "1.0.0")
    PushService pushService;

    /**
     * 发起认证申请
     *
     * @param userId          用户id
     * @param idCardHoldImg   手持身份证照
     * @param idCardFaceImg   身份证正面照
     * @param idCardNationImg 身份证反面照
     * @return 是否成功
     */
    @Override
    public boolean addApply(Long userId, String idCardHoldImg, String idCardFaceImg, String idCardNationImg) {
        AnchorAuditDO auditDO = new AnchorAuditDO();
        auditDO.setAuditId(identityService.getId());
        auditDO.setUserId(userId);
        auditDO.setIdCardHoldImg(idCardHoldImg);
        auditDO.setIdCardFaceImg(idCardFaceImg);
        auditDO.setIdCardNationImg(idCardNationImg);
        auditDO.setCreateTime(System.currentTimeMillis());
        auditDO.setStatus(AuditStatusEnum.WAIT.getCode());
        anchorAuditDAO.insert(auditDO);
        return true;
    }

    /**
     * 获取最近一次认证记录
     *
     * @param userId 用户id
     * @return 认证记录
     */
    @Override
    public AnchorAuditVO getLastApplyRecord(Long userId) {
        AnchorAuditDO record = anchorAuditDAO.getLastApplyRecord(userId);
        if (record != null) {
            AnchorAuditVO auditVO = new AnchorAuditVO();
            BeanUtils.copyProperties(record, auditVO);
            return auditVO;
        }
        return null;
    }

    /**
     * 获取最近一次认证记录
     *
     * @param userIds 用户id
     * @return 认证记录
     */
    @Override
    public Map<Long, AnchorAuditVO> getLastApplyRecordMapByUserIds(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        return anchorAuditDAO.selectLastApplyRecordMapByUserIds(userIds);
    }

    /**
     * 获取最近一次认证状态
     *
     * @param userId 用户id
     * @return 认证状态
     */
    @Override
    public Integer getLastApplyStatus(Long userId) {
        return anchorAuditDAO.getLastApplyStatus(userId);
    }

    /**
     * 获取直播封面
     *
     * @param userId
     * @return 直播封面
     */
    @Override
    public String getLiveCover(Long userId) {
        return anchorDAO.findLiveCover(userId);
    }

    /**
     * 编辑主播信息
     *
     * @param anchorDO
     * @return
     */
    @Override
    public boolean editAnchor(AnchorDO anchorDO) {
        return anchorDAO.updateById(anchorDO) == BoolIntVal.True;
    }

    /**
     * 判断是否主播
     *
     * @param userId 用户id
     * @return 是否主播
     */
    @Override
    public boolean isAnchor(Long userId) {
        return Optional.ofNullable(userDAO.isAnchor(userId)).map(t -> BoolIntVal.True == t).orElse(false);
    }

    /**
     * 查询直播状态
     *
     * @param userId 用户id
     * @return 是否主播
     */
    @Override
    public LiveStatusEnum getLiveStatus(Long userId) {
        return LiveStatusEnum.toEnum(anchorDAO.findLiveStatus(userId));
    }

    /**
     * 查询推流状态
     *
     * @param userId 用户id
     * @return 推流状态
     */
    @Override
    public StreamStatusEnum getStreamStatus(Long userId) {
        return StreamStatusEnum.toEnum(anchorDAO.findStreamStatus(userId));
    }

    /**
     * 主播认证记录是否存在
     *
     * @param auditId
     * @param auditStatus
     * @return
     */
    @Override
    public boolean isAnchorAuditExists(Long auditId, AuditStatusEnum auditStatus) {
        return anchorAuditDAO.countByIdAndStatus(auditId, Optional.ofNullable(auditStatus).map(t -> t.getCode()).orElse(null)) > 0;
    }

    /**
     * 设置主播认证状态
     *
     * @param auditId
     * @param adminId
     * @param auditStatus
     * @param auditMsg
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean setAnchorAuditStatus(Long auditId, Long adminId, AuditStatusEnum auditStatus, String auditMsg) {
        try {
            AnchorAuditDO auditDO = anchorAuditDAO.selectById(auditId);

            long current = System.currentTimeMillis();
            auditDO.setStatus(auditStatus.getCode());
            auditDO.setAuditMsg(auditMsg);
            auditDO.setAuditTime(current);
            auditDO.setAuditId(auditDO.getAuditId());
            auditDO.setAdminId(adminId);
            anchorAuditDAO.updateById(auditDO);

            // 消息通知
            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(identityService.getId());
            taskDO.setTaskName("系统消息-主播认证");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(System.currentTimeMillis());
            taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
            taskDO.setTargetValue(auditDO.getUserId().toString());
            taskDO.setAdminId(0L);
            taskDO.setRemark("系统自动推送");

            PushMsgParam pushParam = new PushMsgParam();

            if (AuditStatusEnum.PASS == auditStatus) {
                // 直播间聊天室 id
                String chatRoomId = String.format("LCR%d", auditDO.getUserId());

                AnchorDO anchor = new AnchorDO();
                anchor.setUserId(auditDO.getUserId());
                anchor.setAnchorAuthTime(current);
                anchor.setAuthAdminId(adminId);
                anchor.setLiveStatus(LiveStatusEnum.Closed.getValue());
                anchor.setStreamStatus(StreamStatusEnum.Closed.getValue());
                anchor.setLiveChatRoomId(chatRoomId);
                anchor.setSingleChatEnable(0);
                anchor.setSingleChatStatus(0);
                anchor.setBattleEnable(BoolIntVal.True);
                anchor.setBattling(BoolIntVal.False);
                anchor.setBattleTimes(0);
                anchor.setBattleWinTimes(0);
                anchor.setBattleLossTimes(0);
                anchor.setBattleDrawTimes(0);
                anchorDAO.insert(anchor);

                UserDO user = new UserDO();
                user.setUserId(auditDO.getUserId());
                user.setIsAnchor(BoolIntVal.True);
                user.setUserUpdateTime(current);
                userDAO.updateById(user);

                taskDO.setPushTitle("认证成功");
                String pushBody = String.format("%s恭喜您，认证主播成功，快去开播吧~", userDAO.selectNameById(auditDO.getUserId()));
                taskDO.setPushBody(pushBody);

                pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
                pushParam.setModuleType(ModuleTypeEnum.ANCHOR.getDesc());
                pushParam.setModuleId(auditDO.getUserId().toString());
                taskDO.setExtParameters(JSON.toJSONString(pushParam));

                // 数据统计
                // 查询用户最后登录渠道
                Long channelId = 0L;
                String headers = userLoginHisDAO.selectLastHeaders(auditDO.getUserId());
                if (StringUtils.isNotBlank(headers)) {
                    HeaderPackage headerPackage = JSON.parseObject(headers, HeaderPackage.class);
                    channelId = headerPackage.getChannelId();
                }
                statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.AUTH_ANCHORS, BigDecimal.ONE);

            } else {
                taskDO.setPushTitle("认证失败");
                if (StringUtils.isNotEmpty(auditMsg) && auditMsg.length() > 20) {
                    auditMsg = auditMsg.substring(0, 20);
                }
                String pushBody = String.format("主播认证失败！%s", auditMsg);
                taskDO.setPushBody(pushBody);

                pushParam.setActionType(PushActionType.NO_ACTION.getValue());
                taskDO.setExtParameters(JSON.toJSONString(pushParam));
            }

            pushService.addPushTask(taskDO);

            return true;
        } catch (Exception e) {
            log.error("主播认证处理失败", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 根据 id 查询信息
     *
     * @param userIds
     * @return
     */
    @Override
    public List<LiveItemVO> getLiveItemsByIds(List<Long> userIds) {
        return anchorDAO.selectLiveItemsByIds(userIds);
    }

    /**
     * 获取主播详情
     *
     * @param userId
     * @return
     */
    @Override
    public AnchorInfoVO getInfo(Long userId) {
        return anchorDAO.selectInfoById(userId);
    }

    /**
     * 查询粉丝团名称
     *
     * @param userId
     * @return
     */
    @Override
    public String getFansName(Long userId) {
        return anchorDAO.selectFansNameById(userId);
    }

    /**
     * 判断粉丝团名称是否已存在
     *
     * @param fansName
     * @return
     */
    @Override
    public boolean isFansNameExists(String fansName) {
        return anchorDAO.countByFansName(fansName) > 0;
    }

    /**
     * 设置粉丝团名称
     *
     * @param userId
     * @param fansName
     * @return
     */
    @Override
    public boolean setFansName(Long userId, String fansName) {
        try {
            anchorDAO.updateFansName(userId, fansName);
            return true;
        } catch (Exception e) {
            log.error("粉丝团名称设置失败", e);
        }
        return false;
    }

    /**
     * 设置/取消房管
     *
     * @param anchorUserId
     * @param masterUserId
     * @param action
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean setMaster(Long anchorUserId, Long masterUserId, Integer action) {
        try {
            Long curTime = System.currentTimeMillis();

            // 设置房管
            AnchorMasterDO anchorMasterDO = new AnchorMasterDO();
            anchorMasterDO.setAnchorUserId(anchorUserId);
            anchorMasterDO.setMasterUserId(masterUserId);
            anchorMasterDO.setStatus(BoolIntVal.True == action ? CommonStatus.Normal : CommonStatus.Deleted);
            anchorMasterDO.setCreateTime(curTime);
            anchorMasterDO.setUpdateTime(curTime);
            anchorMasterDAO.insertOrUpdate(anchorMasterDO);

            // 写入记录
            AnchorMasterHisDO anchorMasterHisDO = new AnchorMasterHisDO();
            anchorMasterHisDO.setAnchorUserId(anchorUserId);
            anchorMasterHisDO.setMasterUserId(masterUserId);
            anchorMasterHisDO.setHisAction(action);
            anchorMasterHisDO.setHisCreateTime(curTime);
            anchorMasterHisDAO.insert(anchorMasterHisDO);

            // 更新用户直播间缓存
            userService.refreshLiveUserProfile(masterUserId, anchorUserId);

            // 发送消息
            LiveUserMasterMsg liveUserMasterMsg = new LiveUserMasterMsg();
            liveUserMasterMsg.setAction(action);
            imService.sendLiveRoomMsg(ImTargetType.GROUP, masterUserId, anchorUserId, liveUserMasterMsg);

            return true;
        } catch (Exception e) {
            log.error("房管设置失败", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;
    }

    /**
     * 判断是否房管
     *
     * @param anchorUserId
     * @param masterUserId
     * @return
     */
    @Override
    public boolean isMaster(Long anchorUserId, Long masterUserId) {
        return anchorMasterDAO.countById(anchorUserId, masterUserId) > 0;
    }

    /**
     * 获取房管列表
     *
     * @param anchorUserId
     * @return
     */
    @Override
    public List<AnchorMasterVO> getMasterList(Long anchorUserId) {
        return anchorMasterDAO.selectAllByAnchorId(anchorUserId);
    }

    /**
     * 获取房管数量
     *
     * @param anchorUserId
     * @return
     */
    @Override
    public int getMasterCount(Long anchorUserId) {
        return anchorMasterDAO.countByAnchorId(anchorUserId);
    }

    /**
     * 是否禁播
     *
     * @param userId 用户id
     * @return 是否禁播
     */
    @Override
    public boolean isDisable(Long userId) {
        Long enableTime = Optional.ofNullable(getEnableTime(userId)).orElse(0L);
        //永久禁播
        if (enableTime < 0) {
            return true;
        }
        return System.currentTimeMillis() < enableTime;
    }

    /**
     * 获取解禁时间点
     *
     * @param userId 用户id
     * @return 时间点
     */
    @Override
    public Long getEnableTime(Long userId) {
        AnchorDisableHisDO anchorDisableHisDO = anchorDisableHisDAO.selectLast(userId);
        if (anchorDisableHisDO != null) {
            return anchorDisableHisDO.getEnableTime();
        }
        return 0L;
    }

    /**
     * 查询最新封禁记录
     *
     * @param userId
     * @return
     */
    @Override
    public AnchorDisableHisDO getLatestDisableHis(Long userId) {
        return anchorDisableHisDAO.selectLast(userId);
    }

    /**
     * 判断用户是否被禁言
     *
     * @param anchorUserId
     * @param userId
     * @return
     */
    @Override
    public boolean isUserForbidden(Long anchorUserId, Long userId) {
        return StringUtils.isNotBlank(redisService.get(RedisCacheKeys.getLiveForbiddenKey(anchorUserId, userId)));
    }

    /**
     * 禁言用户
     *
     * @param anchorUserId
     * @param targetUserId
     * @param masterUserId
     * @param remarks
     * @param duration
     * @return
     */
    @Override
    public boolean setUserForbiddenWords(Long anchorUserId, Long targetUserId, Long masterUserId, String remarks, long duration) {
        Long curTime = System.currentTimeMillis();

        try {
            String redisKey = RedisCacheKeys.getLiveForbiddenKey(anchorUserId, targetUserId);

            String forbidden = redisService.get(redisKey);

            // 写入记录
            AnchorForbiddenHisDO anchorForbiddenHisDO = new AnchorForbiddenHisDO();
            anchorForbiddenHisDO.setAnchorUserId(anchorUserId);
            anchorForbiddenHisDO.setTargetUserId(targetUserId);
            anchorForbiddenHisDO.setMasterUserId(masterUserId);
            anchorForbiddenHisDO.setHisRemarks(remarks);
            anchorForbiddenHisDO.setHisDuration(duration);
            anchorForbiddenHisDO.setHisCreateTime(curTime);
            anchorForbiddenHisDAO.insert(anchorForbiddenHisDO);

            // redis 禁言
            if (duration == 0) {
                // 解禁
                redisService.delKey(redisKey);
            } else {
                if (duration < 0) {
                    // 永久封禁
                    redisService.set(redisKey, String.format("%d:%d", curTime, duration));
                } else {
                    redisService.set(redisKey, String.format("%d:%d", curTime, duration), duration, TimeUnit.MILLISECONDS);
                }
            }

            if (StringUtils.isBlank(forbidden)) {
                // 之前未被禁言，直播间通报禁言消息
                LiveUserForbiddenMsg liveUserForbiddenMsg = new LiveUserForbiddenMsg();
                liveUserForbiddenMsg.setDuration(duration);
                imService.sendLiveRoomMsg(ImTargetType.GROUP, targetUserId, anchorUserId, liveUserForbiddenMsg);
            }

            return true;
        } catch (Exception e) {
            log.error("用户禁言失败", e);
        }

        return false;
    }

    /**
     * 获取禁言用户
     *
     * @param anchorUserId
     * @return
     */
    @Override
    public List<ForbiddenWordsUser> getForbiddenUsers(Long anchorUserId) {
        List<String> keys = redisService.getKeys(RedisCacheKeys.getLiveForbiddenKeyPattern(anchorUserId));
        if (CollectionUtil.isNotEmpty(keys)) {
            List<Long> userIds = new ArrayList<>();
            List<ForbiddenWordsUser> list = new ArrayList<>();
            for (String key : keys) {
                Long userId = Convert.toLong(key.replace(RedisCacheKeys.LIVE_FORBIDDEN, "").replace(String.format("%d_", anchorUserId), ""));
                if (userId == null) {
                    continue;
                }

                String keyVal = redisService.get(key);
                if (StringUtils.isBlank(keyVal)) {
                    continue;
                }
                Long disableTime;
                Long duration;
                if (keyVal.contains(":")) {
                    String[] keyVals = keyVal.split(":");
                    disableTime = Convert.toLong(keyVals[0]);
                    duration = Convert.toLong(keyVals[1]);
                } else {
                    disableTime = Convert.toLong(keyVal);
                    duration = -1L;
                }
                userIds.add(userId);
                ForbiddenWordsUser forbiddenWordsUser = new ForbiddenWordsUser();
                forbiddenWordsUser.setUserId(userId);
                forbiddenWordsUser.setDisableTime(disableTime);
                forbiddenWordsUser.setDuration(duration);
                list.add(forbiddenWordsUser);
            }

            // 用户数据准备
            List<UserCardVO> userList = userDAO.selectNameAndAvatarByIds(userIds);
            if (CollectionUtil.isNotEmpty(userList)) {
                Map<Long, UserCardVO> userCardMap = userList.stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
                for (ForbiddenWordsUser forbiddenWordsUser : list) {
                    UserCardVO userCard = userCardMap.get(forbiddenWordsUser.getUserId());
                    forbiddenWordsUser.setUserName(userCard.getUserName());
                    forbiddenWordsUser.setUserAvatar(userCard.getUserAvatar());
                    forbiddenWordsUser.setWealthLevel(userCard.getWealthLevel());
                }
                list.sort(Comparator.comparingLong(ForbiddenWordsUser::getDisableTime).reversed());
                return list;
            }
        }
        return Collections.emptyList();
    }

    /**
     * 设置公会id
     *
     * @param userId  用户id
     * @param agentId 公会id
     * @return 是否成功
     */
    @Override
    public boolean setAgentId(Long userId, Long agentId) {
        return anchorDAO.updateAgentId(userId, agentId) == 1;
    }

    /**
     * 查询公会id
     *
     * @param userId 用户id
     * @return 公会id
     */
    @Override
    public Long getAgentId(Long userId) {
        return anchorDAO.getAgentId(userId);
    }

    /**
     * 是否签约
     *
     * @param userId 用户id
     * @return 是否签约
     */
    @Override
    public boolean isSign(Long userId) {
        return Optional.ofNullable(anchorDAO.getAgentId(userId)).orElse(0L) > 0;
    }

    /**
     * 判断是否粉丝团成员
     *
     * @param anchorUserId
     * @param fansUserId
     * @return
     */
    @Override
    public boolean isFans(Long anchorUserId, Long fansUserId) {
        return anchorFansDAO.countById(anchorUserId, fansUserId) > 0;
    }

    /**
     * 加入粉丝团
     *
     * @param anchorUserId
     * @param fansUserId
     * @return
     */
    @Override
    public Long addFans(Long anchorUserId, Long fansUserId) {
        Long balance = payService.payForFans(anchorUserId, fansUserId);
        if (balance != null && balance > 0) {
            Long curTime = System.currentTimeMillis();
            AnchorFansDO anchorFansDO = new AnchorFansDO();
            anchorFansDO.setAnchorUserId(anchorUserId);
            anchorFansDO.setFansUserId(fansUserId);
            anchorFansDO.setStatus(CommonStatus.Normal);
            anchorFansDO.setJoinTime(curTime);
            anchorFansDO.setUpdateTime(curTime);
            anchorFansDO.setFansLevel(1);
            anchorFansDAO.insertOrUpdate(anchorFansDO);

            // 更新直播间数据
            liveUserProvider.refreshLiveUserProfile(fansUserId, anchorUserId);
        }
        return balance;
    }

    /**
     * 获取粉丝团列表
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    @Override
    public PageList<FansTeamMemberVO> getFansList(int pageIndex, int pageSize, Long userId) {
        PageList<FansTeamMemberVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(anchorFansDAO.selectMemberList(pageList.getPageCursor(), pageSize, userId));
        pageList.setTotal(anchorFansDAO.countMembers(userId));
        return pageList;
    }

    /**
     * 获取主播信息
     *
     * @param userId 用户id
     * @return 主播信息
     */
    @Override
    public AnchorInfo getAnchorInfo(Long userId) {
        AnchorInfo anchorInfo = new AnchorInfo();
        anchorInfo.setIsSign(isSign(userId) ? BoolIntVal.True : BoolIntVal.False);
        anchorInfo.setAgentId(anchorDAO.getAgentId(userId));
        anchorInfo.setApplyStatus(Optional.ofNullable(agentService.getAuditStatus(userId)).orElse(-1));
        anchorInfo.setFansName(getFansName(userId));
        return anchorInfo;
    }

    /**
     * 获取主播认证时间
     *
     * @param userId
     * @return
     */
    @Override
    public Long getAnchorAuthTime(Long userId) {
        return anchorDAO.selectAuthTime(userId);
    }

    /**
     * 拉黑用户
     *
     * @param anchorUserId
     * @param userId
     * @param remarks
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addBlack(Long anchorUserId, Long userId, String remarks) {
        try {
            // 写入拉黑
            Long curTime = System.currentTimeMillis();
            AnchorBlackDO anchorBlack = new AnchorBlackDO();
            anchorBlack.setAnchorUserId(anchorUserId);
            anchorBlack.setUserId(userId);
            anchorBlack.setStatus(CommonStatus.Normal);
            anchorBlack.setBlackRemarks(remarks);
            anchorBlack.setUpdateTime(curTime);
            anchorBlackDAO.insertOrUpdate(anchorBlack);

            // 写入拉黑记录
            AnchorBlackHisDO anchorBlackHis = new AnchorBlackHisDO();
            anchorBlackHis.setHisId(identityService.getId());
            anchorBlackHis.setAnchorUserId(anchorUserId);
            anchorBlackHis.setUserId(userId);
            anchorBlackHis.setHisRemarks(remarks);
            anchorBlackHis.setHisCreateTime(curTime);
            anchorBlackHisDAO.insert(anchorBlackHis);

            // 发送拉黑消息
            LiveUserBlackMsg msg = new LiveUserBlackMsg();
            imService.sendLiveRoomMsg(ImTargetType.GROUP, userId, anchorUserId, msg);

            return true;
        } catch (Exception e) {
            log.error("拉黑用户失败", e);
        }
        return false;
    }

    /**
     * 解禁用户
     *
     * @param anchorUserId
     * @param userId
     * @return
     */
    @Override
    public boolean removeBlack(Long anchorUserId, Long userId) {
        try {
            AnchorBlackDO anchorBlack = new AnchorBlackDO();
            anchorBlack.setAnchorUserId(anchorUserId);
            anchorBlack.setUserId(userId);
            anchorBlack.setStatus(CommonStatus.Deleted);
            anchorBlack.setBlackRemarks("解禁");
            anchorBlack.setUpdateTime(System.currentTimeMillis());
            anchorBlackDAO.insertOrUpdate(anchorBlack);
            return true;
        } catch (Exception e) {
            log.error("解禁黑名单失败", e);
        }
        return false;
    }

    /**
     * 获取黑名单列表
     *
     * @param pageIndex
     * @param pageSize
     * @param anchorUserId
     * @return
     */
    @Override
    public PageList<AnchorBlackVO> getBlackList(int pageIndex, int pageSize, Long anchorUserId) {
        PageList<AnchorBlackVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(anchorBlackDAO.selectAll(pageList.getPageCursor(), pageSize, anchorUserId));
        pageList.setTotal(anchorBlackDAO.count(anchorUserId));
        return pageList;
    }

    /**
     * 是否被拉黑
     *
     * @param anchorUserId
     * @param userId
     * @return
     */
    @Override
    public boolean isBlack(Long anchorUserId, Long userId) {
        return anchorBlackDAO.countById(anchorUserId, userId) > 0;
    }

    /**
     * 禁播主播
     *
     * @param userId        用户id
     * @param adminId       管理员id
     * @param enableTime    解禁时间
     * @param disableReason 禁播理由
     * @param reportId      报告id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean disableAnchor(Long userId, Long adminId, Long enableTime, String disableReason, Long reportId) {
        long currentTime = System.currentTimeMillis();

        // 禁播记录
        AnchorDisableHisDO hisDO = new AnchorDisableHisDO();
        hisDO.setType(AnchorOperationEnum.DISABLE.getCode());
        hisDO.setUserId(userId);
        hisDO.setAdminId(adminId);
        hisDO.setDisableTime(currentTime);
        hisDO.setEnableTime(enableTime);
        hisDO.setRemark(disableReason);
        hisDO.setReportId(reportId);
        anchorDisableHisDAO.insert(hisDO);

        //禁播流
        if (!liveService.disableStream(userId, enableTime)) {
            throw new RuntimeException("禁播失败");
        }

        return true;
    }

    /**
     * 获取主播单聊信息
     *
     * @param userId
     * @return
     */
    @Override
    public AnchorSingleChatInfoVO getSingleChatInfo(Long userId) {
        return anchorDAO.selectSingleChatInfo(userId);
    }

    /**
     * 获取单聊权限
     *
     * @param userId
     * @return
     */
    @Override
    public Integer getSingleChatEnable(Long userId) {
        return anchorDAO.selectSingleChatEnable(userId);
    }

    @Override
    public PageList<AnchorPkVO> getAnchorPkList(AnchorPkSearchType type, Long userId, int pageIndex, int pageSize) {
        PageList<AnchorPkVO> page = new PageList<>(pageIndex, pageSize);
        List<Long> userIds = new ArrayList<>();
        Long agentId = null;
        switch (type) {
            case ALL:
                break;
            case AGENT:
                agentId = anchorDAO.getAgentId(userId);
                if (agentId == null) {
                    page.setList(new ArrayList<>());
                    page.setTotal(0);
                    return page;
                }
                break;
            case FOLLOW:
//                userIds = socialRelationService.getFollowUserIds(userId);
                List<RelationDTO> relationList = followRelationDAO.selectFollows(userId);
                userIds = relationList.stream().map(RelationDTO::getUserId).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(userIds)) {
                    page.setList(new ArrayList<>());
                    page.setTotal(0);
                    return page;
                }
                break;
            default:
                break;
        }

        List<AnchorPkVO> list = anchorDAO.selectAnchorPkList(userIds, userId, agentId, LiveStatusEnum.Living.getValue(), page.getPageCursor(), pageSize);
        page.setList(list);
        page.setTotal(anchorDAO.countAnchorPkList(userIds, userId, agentId, LiveStatusEnum.Living.getValue()));
        return page;
    }

    @Override
    public PageList<AnchorPkVO> getAnchorList(String content, int pageIndex, int pageSize) {
        PageList<AnchorPkVO> page = new PageList<>(pageIndex, pageSize);
        PageList<Long> userPage = userSearchService.getAnchorList(content, pageIndex, pageSize);
        List<Long> userIds = userPage.getList();
        if (CollectionUtils.isEmpty(userIds)) {
            page.setList(new ArrayList<>());
            page.setTotal(0);
            return page;
        }
        page.setList(anchorDAO.selectAnchorList(userIds));
        page.setTotal(userPage.getTotal());
        return page;
    }

    @Override
    public int getFansMemberCount(Long userId) {
        return anchorFansDAO.countMembers(userId);
    }

    /**
     * 判断是否允许售卖商品
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isSellEnabled(Long userId) {
        return Optional.ofNullable(anchorDAO.selectSellEnable(userId)).map(t -> BoolIntVal.True == t).orElse(false);
    }
}
