package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.finance.PayWayEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.user.OAuthTypeEnum;
import com.starbuds.server.common.constant.user.OneKeyLoginEnum;
import com.starbuds.server.common.pojo.data.OauthConfig;
import com.starbuds.server.common.pojo.data.OneKeyLoginAgentConfig;
import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.pojo.db.LiveCategoryDO;
import com.starbuds.server.common.pojo.db.RtcRoomCategoryDO;
import com.starbuds.server.common.pojo.message.push.AliyunPushConfig;
import com.starbuds.server.common.pojo.rtc.RtcRoomCategory;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.service.user.dao.ChannelDAO;
import com.starbuds.server.service.user.dao.LiveCategoryDAO;
import com.starbuds.server.service.user.dao.RtcRoomCategoryDAO;
import com.starbuds.server.service.user.dao.SingleChatCategoryDAO;
import com.starbuds.server.service.user.provider.OauthConfigProvider;
import com.starbuds.server.service.user.provider.OneKeyLoginConfigProvider;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 渠道服务实现
 *
 * @author Lay
 * @date 2019/12/6
 */
@Slf4j
@Service(version = "1.0.0")
public class ChannelServiceImpl implements ChannelService {

    @Resource
    ChannelDAO channelDAO;

    @Resource
    LiveCategoryDAO liveCategoryDAO;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Resource
    SingleChatCategoryDAO singleChatCategoryDAO;

    @Reference(version = "1.0.0")
    OauthConfigService oauthConfigService;

    @Reference(version = "1.0.0")
    OneKeyLoginConfigService oneKeyLoginConfigService;

    @Resource
    OneKeyLoginConfigProvider oneKeyLoginConfigProvider;

    @Resource
    OauthConfigProvider oauthConfigProvider;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    AliyunPushConfigService aliyunPushConfigService;


    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    /**
     * 获取渠道短信配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelSmsConfig getSmsConfig(Long channelId) {
        // 查询渠道配置
        ChannelDO channelDO = channelDAO.selectSmsConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectSmsConfig(0L);
        }

        if (channelDO != null) {
            ChannelSmsConfig channelSmsConfig = new ChannelSmsConfig();
            channelSmsConfig.setSmsAgentId(channelDO.getSmsAgentId());
            channelSmsConfig.setSmsSignId(channelDO.getSmsSignId());
            return channelSmsConfig;
        }

        return null;
    }

    /**
     * 获取一键登录配置
     *
     * @param channelId  渠道id
     * @param clientType 客户端类型
     * @return
     */
    @Override
    public ChannelOneKeyLoginConfig getOneKeyLoginConfig(Long channelId, Integer clientType) {
        return oneKeyLoginConfigProvider.getOneKeyLoginConfig(channelId, clientType);
    }

    /**
     * 获取微信app配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelAppConfig getWxAppConfig(Long channelId) {
        return oauthConfigProvider.getWxAppConfig(channelId);
    }

    /**
     * 获取qq配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelAppConfig getQqConfig(Long channelId) {
        return oauthConfigProvider.getQqConfig(channelId);
    }

    /**
     * 获取weibo配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelAppConfig getWeiboConfig(Long channelId) {
        return oauthConfigProvider.getWeiboConfig(channelId);
    }

    /**
     * 获取支付方式配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelPayWayConfig getPayWayConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectPayWayConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectPayWayConfig(0L);
        }
        if (channelDO != null) {
            String payWay = channelDO.getPayWay();

            ChannelPayWayConfig payWayConfig = new ChannelPayWayConfig();
            payWayConfig.setPayWay(payWay);
            payWayConfig.setRechargeRatio(channelDO.getRechargeRatio());

            if (StringUtils.isNotBlank(payWay)) {
                String[] payWays = payWay.split(",");
                if (payWays.length > 0) {
                    payWayConfig.setPayWaySize(payWays.length);
                    payWayConfig.setPayWays(Arrays.stream(payWays).map(t -> Convert.toInt(t)).collect(Collectors.toList()));
                }
            }

            return payWayConfig;
        }
        return null;
    }

    /**
     * 获取支付配置
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelPayConfig getPayConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectPayConfig(channelId);
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectPayConfig(0L);
        }
        if (channelDO != null) {
            ChannelPayConfig payConfig = new ChannelPayConfig();
            payConfig.setAlipayConfigId(channelDO.getAlipayConfigId());
            payConfig.setWxPayConfigId(channelDO.getWxPayConfigId());
            payConfig.setHuaweiPayConfigId(channelDO.getHuaweiPayConfigId());

            return payConfig;
        }
        return null;
    }

    /**
     * 获取渠道初始化配置
     *
     * @param channelId
     * @param clientType
     * @param appVersion
     * @param packageName
     * @return
     */
    @Override
    public ChannelInitConfig getChannelInitConfig(Long channelId, Integer clientType, String appVersion, String packageName) {
        ChannelDO channelDO = channelDAO.selectAppInitConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectAppInitConfig(0L);
        }
        if (channelDO == null) {
            log.info("该渠道暂无配置,channelId:{}", channelId);
            return null;
        }
        // 支付方式
        List<PayWayInitConfig> payWayConfigs = new ArrayList<>();
        String payWay = channelDO.getPayWay();
        if (!StringUtil.isNullOrEmpty(payWay)) {
            String[] payWays = payWay.split(",");
            for (String item : payWays) {
                PayWayInitConfig payWayConfig = new PayWayInitConfig();
                payWayConfig.setType(Integer.valueOf(item));
                payWayConfig.setPayName(PayWayEnum.toEnum(Integer.valueOf(item)).getName());
                payWayConfigs.add(payWayConfig);
            }
        }
        // oauth配置
        List<Long> configIds = new ArrayList<>();
        configIds.add(channelDO.getWxAppOAuthConfigId());
        configIds.add(channelDO.getQqOAuthConfigId());
        configIds.add(channelDO.getWbOAuthConfigId());
        List<OauthInitConfig> oauthConfigs = new ArrayList<>();
        for (int i = 0; i < configIds.size(); i++) {
            OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(configIds.get(i));
            if (oauthConfig != null) {
                OauthInitConfig oauthInit = new OauthInitConfig();
                oauthInit.setType(oauthConfig.getAuthType());
                oauthInit.setOauthName(OAuthTypeEnum.getOauthType(oauthConfig.getAuthType()).getName());
                oauthInit.setAppId(oauthConfig.getAppId());
                oauthConfigs.add(oauthInit);
            }
        }
        // 一键登录
        Long agentId = channelDO.getOneKeyLoginAgentId();
        OneKeyLoginInitConfig loginInitConfig = null;
        ClientType type = ClientType.toEnum(clientType);
        if (agentId != null) {
            OneKeyLoginAgentConfig agentConfig = oneKeyLoginConfigService.getOneKeyLoginAgentConfig(agentId);
            if (agentConfig != null) {
                loginInitConfig = new OneKeyLoginInitConfig();
                loginInitConfig.setAgentName(agentConfig.getAgentName());
                loginInitConfig.setType(OneKeyLoginEnum.toEnum(agentConfig.getAgentName()).getCode());
                String accessKey = "";
                switch (type) {
                    case Android:
                        accessKey = agentConfig.getAndroidAccessKey();
                        break;
                    case iOS:
                        accessKey = agentConfig.getIosAccessKey();
                        break;
                    default:
                        log.error("暂不支持该客户端一键登录");
                        break;
                }
                loginInitConfig.setAppId(accessKey);
            }
        }

        // 直播分类
        List<LiveCategory> liveCategories = new ArrayList<>();
        String categories = channelDO.getLiveCategories();
        if (!StringUtil.isNullOrEmpty(categories)) {
            long[] ids = Arrays.stream(categories.split(",")).mapToLong(Long::valueOf).toArray();
            List<LiveCategoryDO> doList = liveCategoryDAO.findByIds(ids);
            if (!CollectionUtils.isEmpty(doList)) {
                for (LiveCategoryDO item : doList) {
                    liveCategories.add(new LiveCategory(item.getCategoryId(), item.getCategoryName(), item.getSellEnable()));
                }
            }
        }

        // 语聊分类
        List<RtcRoomCategory> rtcRoomCategories = new ArrayList<>();
        String roomCategories = channelDO.getRtcRoomCategories();
        if (!StringUtil.isNullOrEmpty(roomCategories)) {
            long[] ids = Arrays.stream(roomCategories.split(",")).mapToLong(Long::valueOf).toArray();
            List<RtcRoomCategoryDO> doList = rtcRoomCategoryDAO.findByIds(ids, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
            if (!CollectionUtils.isEmpty(doList)) {
                for (RtcRoomCategoryDO item : doList) {
                    rtcRoomCategories.add(new RtcRoomCategory(item.getCategoryId(), item.getCategoryName(), item.getLayout(), item.getMakeFriend(), item.getBackgroundColor(), item.getBackgroundImage(), item.getBattleEnable(), RtcBusinessTypeEnum.RTC_NORMAL.getValue()));
                }
            }
        }

        // 个播分类
        if (isDev()) {
            log.info("获取出事配置：channlId:{}", channelId);
            log.info("获取出事配置：channlDO:{}", channelDO);
        }
        List<RtcRoomCategory> singleRtcRoomCategories = new ArrayList<>();
        String singleRoomCategories = channelDO.getSingleRtcRoomCategories();
        if (!StringUtil.isNullOrEmpty(singleRoomCategories)) {
            long[] ids = Arrays.stream(singleRoomCategories.split(",")).mapToLong(Long::valueOf).toArray();
            List<RtcRoomCategoryDO> doList = rtcRoomCategoryDAO.findByIds(ids, RtcBusinessTypeEnum.RTC_SINGLE.getValue());
            if (!CollectionUtils.isEmpty(doList)) {
                for (RtcRoomCategoryDO item : doList) {
                    singleRtcRoomCategories.add(new RtcRoomCategory(item.getCategoryId(), item.getCategoryName(), item.getLayout(), item.getMakeFriend(), item.getBackgroundColor(), item.getBackgroundImage(), item.getBattleEnable(), RtcBusinessTypeEnum.RTC_SINGLE.getValue()));
                }
            }
        }

        // 单聊分类
        List<SingleChatCategory> singleChatCategoryList;
        String singleChatCategories = channelDO.getSingleChatCategories();
        if (StringUtils.isNotBlank(singleChatCategories)) {
            long[] ids = Arrays.stream(singleChatCategories.split(",")).mapToLong(Long::valueOf).toArray();
            singleChatCategoryList = singleChatCategoryDAO.selectByIds(ids);
        } else {
            singleChatCategoryList = Collections.emptyList();
        }

        // 渠道初始化配置
        ChannelInitConfig initConfig = new ChannelInitConfig();
        initConfig.setPayWays(payWayConfigs);
        initConfig.setOauths(oauthConfigs);
        initConfig.setOneKeyLogin(loginInitConfig);

        // 阿里云推送配置
        AliyunPushConfig aliyunPushConfig = aliyunPushConfigService.getAliyunPushConfig(packageName);
        if (aliyunPushConfig == null) {
            log.info("包名{}无推送配置", packageName);
        } else {
            Long pushAppKey = null;
            String pushAppSecret = "";
            switch (type) {
                case Android:
                    pushAppKey = aliyunPushConfig.getAndroidAppKey();
                    pushAppSecret = aliyunPushConfig.getAndroidAppSecret();
                    break;
                case iOS:
                    pushAppKey = aliyunPushConfig.getIosAppKey();
                    pushAppSecret = aliyunPushConfig.getIosAppSecret();
                    break;
                default:
                    log.error("暂不支持该客户端");
                    break;
            }
            initConfig.setPushAppKey(pushAppKey);
            initConfig.setPushAppSecret(pushAppSecret);
        }

        //数美
        JSONObject sm = JSONObject.parseObject(releaseConfigService.getReleaseConfig("shumei"));
        ShumeiConfig shumeiConfig = new ShumeiConfig();
        shumeiConfig.setOrganization(sm.getString("organization"));
        shumeiConfig.setAccessKey(sm.getString("accessKey"));
        shumeiConfig.setAndroidPublicKey(sm.getString("androidPublicKey"));
        shumeiConfig.setIosPublicKey(sm.getString("iosPublicKey"));
        shumeiConfig.setAppId(sm.getString("appId"));

        //即构
//        JSONObject zego = JSONObject.parseObject(releaseConfigService.getReleaseConfig("zego"));
//        ZegoConfig zegoConfig = new ZegoConfig();
//        zegoConfig.setAppId(zego.getLong("appId").toString());
//        zegoConfig.setAppSign(zego.getString("appSign"));

        initConfig.setCoinName(channelDO.getCoinName());
        initConfig.setLiveCategories(liveCategories);
        initConfig.setRtcRoomCategories(rtcRoomCategories);
        initConfig.setSingleRtcRoomCategories(singleRtcRoomCategories);
        initConfig.setSingleChatCategories(singleChatCategoryList);
        initConfig.setLiveTips(channelDO.getLiveTips());
        initConfig.setImGroupId(channelDO.getImGroupId());
        initConfig.setLiveMsgServer(channelDO.getLiveMsgServer());
        initConfig.setUrlNoble(channelDO.getUrlNoble());
        initConfig.setUrlNiceNo(channelDO.getUrlNiceNo());
        initConfig.setUrlLevel(channelDO.getUrlLevel());
        initConfig.setUrlVip(channelDO.getUrlVip());
        initConfig.setUrlVirtualCoin(channelDO.getUrlVirtualCoin());
        initConfig.setUrlCoinShop(channelDO.getUrlCoinShop());
        initConfig.setUrlRechargeRewards(channelDO.getUrlRechargeRewards());
        initConfig.setUrlChatTimeWallet(channelDO.getUrlChatTimeWallet());
        initConfig.setMarketStatus(Optional.ofNullable(dataService.getMarketStatus(appVersion, channelDO.getChannelId())).orElse(0));
        initConfig.setRankAddress(channelDO.getRankAddress());
        initConfig.setLiveFps(channelDO.getLiveFps());
        initConfig.setLiveKbps(channelDO.getLiveKbps());
        initConfig.setConnectMicroFps(channelDO.getConnectMicroFps());
        initConfig.setConnectMicroKbps(channelDO.getConnectMicroKbps());
        initConfig.setInviteCodeEnable(channelDO.getInviteCodeEnable());
        initConfig.setCustomerUserId(Optional.ofNullable(channelDO.getCustomerUserId()).orElse(channelDAO.selectCustomerUserId(0L)));
        initConfig.setWxAppService(channelDO.getWxAppService());
        initConfig.setWxSalesService(channelDO.getWxSalesService());
        initConfig.setShumei(shumeiConfig);
//        initConfig.setZego(zegoConfig);

        return initConfig;
    }

    /**
     * 判断直播类目是否存在
     *
     * @param channelId
     * @param categoryId
     * @return
     */
    @Override
    public boolean isLiveCategoryExists(Long channelId, Long categoryId) {
        ChannelDO channelDO = channelDAO.selectAppInitConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectAppInitConfig(0L);
        }
        if (StringUtils.isNotBlank(channelDO.getLiveCategories())) {
            Set<String> categorySet = Arrays.stream(channelDO.getLiveCategories().split(",")).collect(Collectors.toSet());
            return categorySet.contains(String.valueOf(categoryId));
        }
        return false;
    }

    /**
     * 获取直播消息服务器地址
     *
     * @param channelId
     * @return
     */
    @Override
    public String getLiveMsgServer(Long channelId) {
        ChannelDO channelDO = channelDAO.selectAppInitConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectAppInitConfig(0L);
        }
        return channelDO.getLiveMsgServer();
    }

    /**
     * 获取渠道名称
     *
     * @param channelId
     * @return
     */
    @Override
    public String getChannelName(Long channelId) {
        return channelDAO.selectChannelName(channelId);
    }

    /**
     * 获取渠道名称
     *
     * @param channelIds
     * @return
     */
    @Override
    public Map<Long, String> getChannelNameMap(Set<Long> channelIds) {
        if (CollectionUtil.isEmpty(channelIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, ChannelDTO> channelMap = channelDAO.selectChannelMapByChannelIds(channelIds);
        if (CollectionUtil.isEmpty(channelMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, String> resultMap = new HashMap<>();
        for (Long channelId : channelIds) {
            resultMap.put(channelId, channelMap.get(channelId) == null ? "" : channelMap.get(channelId).getChannelName());
        }
        return resultMap;
    }

    /**
     * 获取指定推荐渠道 id
     *
     * @param channelId
     * @return
     */
    @Override
    public Long getTopSuggestChannelId(Long channelId) {
        return Optional.ofNullable(channelDAO.selectTopSuggestChannelId(Optional.ofNullable(channelId).orElse(0L))).orElse(0L);
    }

    /**
     * 单聊推荐跟随渠道
     *
     * @param channelId
     * @return
     */
    @Override
    public Long getSingleChatSuggestChannelId(Long channelId) {
        return Optional.ofNullable(channelDAO.selectSingleChatSuggestChannelId(Optional.ofNullable(channelId).orElse(0L))).orElse(0L);
    }

    /**
     * 获取版本跟随渠道 id
     *
     * @param channelId
     * @return
     */
    @Override
    public Long getAppVersionChannelId(Long channelId) {
        return Optional.ofNullable(channelDAO.selectAppVersionChannelId(Optional.ofNullable(channelId).orElse(0L))).orElse(0L);
    }

    /**
     * 获取 Banner 跟随渠道 id
     *
     * @param channelId
     * @return
     */
    @Override
    public Long getBannerChannelId(Long channelId) {
        return Optional.ofNullable(channelDAO.selectBannerChannelId(Optional.ofNullable(channelId).orElse(0L))).orElse(0L);
    }

    /**
     * 获取所有渠道id
     *
     * @return
     */
    @Override
    public List<Long> getAllChannelId() {
        return channelDAO.selectAllChannelId();
    }

    @Override
    public Map<Long, ChannelRatio> getRation(List<Long> channelIds) {
        Map<Long, ChannelRatio> map = new HashMap<>();
        List<ChannelRatio> channelRatios = channelDAO.selectRatioList(channelIds);
        if (!CollectionUtil.isEmpty(channelRatios)) {
            Map<Long, List<ChannelRatio>> strMap = channelRatios.stream().collect(Collectors.groupingBy(ChannelRatio::getChannelId));
            for (Map.Entry<Long, List<ChannelRatio>> entry : strMap.entrySet()) {
                map.put(entry.getKey(), entry.getValue().get(0));
            }
        }
        return map;
    }

    @Override
    public String getSingleChatCategories(Long channelId) {
        // 单聊推荐跟随渠道
        Long singleChannelId = channelDAO.selectSingleChatSuggestChannelId(channelId);
        if (singleChannelId != null) {
            channelId = singleChannelId;
        }
        return channelDAO.selectSingleChatCategories(channelId);
    }

    @Override
    public Long getPosterImageChannelId(Long channelId) {
        return channelDAO.selectPosterImageChannelId(channelId);
    }

    @Override
    public Integer getInviteCodeEnable(Long channelId) {
        return channelDAO.selectInviteCodeEnable(channelId);
    }

    @Override
    public boolean isMachineValidateEnable(Long channelId) {
        return Optional.ofNullable(channelDAO.selectMachineValidateEnable(channelId)).orElse(BoolIntVal.False) == BoolIntVal.True;
    }

    /**
     * 获取渠道邀请配置
     *
     * @param channelId
     * @return
     */
    @Override
    public Integer getInviteConfigGroupId(Long channelId) {
        return channelDAO.selectInviteConfigGroupId(channelId);
    }

    @Override
    public List<ChannelDTO> getChannelsByChannelIds(Set<Long> channelIds) {
        if (CollectionUtils.isEmpty(channelIds)) {
            return new ArrayList<>();
        }
        return channelDAO.selectChannelsByChannelIds(channelIds);
    }
}
