package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.common.KeyValueMapDO;
import com.starbuds.server.common.pojo.data.FamilySkillVO;
import com.starbuds.server.common.pojo.db.FamilyAuditDO;
import com.starbuds.server.common.pojo.db.FamilyDO;
import com.starbuds.server.common.pojo.db.FamilySkillAuditDO;
import com.starbuds.server.common.pojo.db.FamilyUserDO;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.mq.QueueUserExitFamilyMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/7 16:29
 * @Description 家族业务实现
 */
@Slf4j
@Service(version = "1.0.0")
public class FamilyServiceImpl implements FamilyService {

    @Resource
    FamilyDAO familyDAO;

    @Resource
    FamilyAuditDAO familyAuditDAO;

    @Resource
    FamilyUserDAO familyUserDAO;

    @Resource
    FamilySkillAuditDAO familySkillAuditDAO;

    @Resource
    FamilySkillDAO familySkillDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 家族是否存在
     *
     * @param familyId 家族id
     * @return 是否存在
     */
    @Override
    public boolean isFamilyExist(Long familyId) {
        return familyDAO.countById(familyId) > 0;
    }

    /**
     * 获取家族列表
     *
     * @param content   内容
     * @param pageIndex 当前页码
     * @param pageSize  分页大小
     * @return 分页对象
     */
    @Override
    public PageList<FamilyDTO> getFamilyList(String content, int pageIndex, int pageSize) {
        PageList<FamilyDTO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familyDAO.selectFamilyPageList(content, page.getPageCursor(), pageSize));
        page.setTotal(familyDAO.countFamilyPageList(content));
        return page;
    }

    /**
     * 获取家族详情
     *
     * @param familyId 家族id
     * @param userId
     * @return 家族详情
     */
    @Override
    public FamilyVO getFamilyDetail(Long familyId, Long userId) {
        FamilyVO vo = familyDAO.selectFamilyDetail(familyId);
        if (vo != null) {
            vo.setUserCount(familyUserDAO.selectUserCount(familyId));
            FamilyAuditVO lastApplyInfo;
            if (isFamilyPresident(userId)) {
                lastApplyInfo = new FamilyAuditVO();
                lastApplyInfo.setFamilyId(familyId);
                lastApplyInfo.setUserId(userId);
                lastApplyInfo.setApplyType(ApplyTypeEnum.JOIN.getCode());
                lastApplyInfo.setAuditStatus(AuditStatusEnum.PASS.getCode());
            } else {
                lastApplyInfo = familyAuditDAO.selectLastApplyInfo(userId, familyId, null);
                ;
                if (lastApplyInfo != null && lastApplyInfo.getAuditStatus() == AuditStatusEnum.PASS.getCode() && lastApplyInfo.getApplyType() == ApplyTypeEnum.LEAVE.getCode()) {
                    lastApplyInfo = null;
                } else {

                }
            }

            vo.setLastApplyInfo(lastApplyInfo);
        }
        return vo;
    }

    /**
     * 获取家族
     *
     * @param familyId 家族id
     * @return 家族详情
     */
    @Override
    public FamilyDO getFamily(Long familyId) {
        return familyDAO.selectById(familyId);
    }

    /**
     * 添加家族审核 （申请签约，申请解约）
     *
     * @param auditDO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addFamilyAudit(FamilyAuditDO auditDO) {
        return familyAuditDAO.insert(auditDO) > 0;
    }

    /**
     * 批量添加家族审核
     *
     * @param auditDOS
     * @return
     */
    @Override
    public boolean addFamilyAudits(List<FamilyAuditDO> auditDOS) {
        return familyAuditDAO.batchInsert(auditDOS) > 0;
    }

    /**
     * 申请技能认证
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param skillTagId
     * @param skillIntro
     * @param images
     * @param voice
     * @param duration
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addSkillAuth(Long familyId, Long userId, Long skillId, Long skillTagId, String skillIntro, String images, String voice, Long duration) {
        FamilySkillAuditDO skillAuditDO = new FamilySkillAuditDO();
        skillAuditDO.setAuditId(identityService.getId());
        skillAuditDO.setUserId(userId);
        skillAuditDO.setFamilyId(familyId);
        skillAuditDO.setSkillId(skillId);
        skillAuditDO.setSkillTagId(skillTagId);
        skillAuditDO.setSkillIntro(skillIntro);
        skillAuditDO.setImages(images);
        skillAuditDO.setVoice(voice);
        skillAuditDO.setDuration(duration);
        skillAuditDO.setCreateTime(System.currentTimeMillis());
        skillAuditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
        if (familySkillAuditDAO.insert(skillAuditDO) > 0) {
            try {
                // 系统消息
                SystemMsg systemMsg = new SystemMsg();
                systemMsg.setContentType(ContentType.TEXT);
                systemMsg.setTitle("技能认证提交成功");
                String body = String.format("你的【%s】技能认证已成功提交，请耐心等待家族审核！", skillService.getSkillName(skillId));
                systemMsg.setBody(body);
                systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
                systemMsg.setModuleType(ModuleTypeEnum.SKILL_AUTH.getDesc());
                systemMsg.setModuleId(Convert.toStr(1)); // 认证中
                imService.sendCustomImMsg(null, userId, systemMsg, null);
            } catch (Exception e) {
                log.error("出现错误", e);
            }
            return true;
        }
        return false;
    }

    /**
     * 是否签约指定家族
     *
     * @param familyId 家族id
     * @param userId   用户id
     * @return 是否签约
     */
    @Override
    public boolean isFamilyUserExist(Long familyId, Long userId) {
        return familyUserDAO.countById(familyId, userId) > 0;
    }

    /**
     * 查询审核状态
     *
     * @param userId   用户id
     * @param familyId 家族id
     * @return 签约状态
     */
    @Override
    public Integer getAuditStatus(Long userId, Long familyId) {
        if (isFamilyPresident(userId)) {
            return AuditStatusEnum.PASS.getCode();
        }
        return familyAuditDAO.selectAuditStatus(userId, familyId);
    }

    /**
     * 获取最近一次申请信息
     *
     * @param userId
     * @param familyId
     * @param applyType
     * @return
     */
    @Override
    public FamilyAuditVO getLastApplyInfo(Long userId, Long familyId, Integer applyType) {
        if (isFamilyPresident(userId)) {
            FamilyAuditVO lastApplyInfo = new FamilyAuditVO();
            lastApplyInfo.setFamilyId(familyId);
            lastApplyInfo.setUserId(userId);
            lastApplyInfo.setApplyType(ApplyTypeEnum.JOIN.getCode());
            lastApplyInfo.setAuditStatus(AuditStatusEnum.PASS.getCode());
            return lastApplyInfo;
        }
        return familyAuditDAO.selectLastApplyInfo(userId, familyId, applyType);
    }

    /**
     * 获取最近一次离开时间
     *
     * @param userId
     * @param familyId
     * @return
     */
    @Override
    public Long getLastLeaveTime(Long userId, Long familyId) {
        return familyAuditDAO.selectLastLeaveTime(userId, familyId, ApplyTypeEnum.LEAVE.getCode(), AuditStatusEnum.PASS.getCode());
    }

    /**
     * 获取家族id
     *
     * @param userId
     * @return
     */
    @Override
    public Long getFamilyId(Long userId) {
        return familyUserDAO.selectFamilyId(userId);
    }

    /**
     * 获取家族id
     *
     * @param userIds
     */
    @Override
    public Map<Long, Long> getFamilyIds(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, FamilyUserDO> familyMap = familyUserDAO.selectFamilyIds(userIds);
        if (CollectionUtil.isEmpty(familyMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Long> result = new HashMap<>();
        for (Map.Entry<Long, FamilyUserDO> entry : familyMap.entrySet()) {
            result.put(entry.getKey(), Optional.ofNullable(entry.getValue()).map(FamilyUserDO::getFamilyId).orElse(null));
        }
        return result;
    }

    /**
     * 获取家族
     *
     * @param familyIds
     * @return
     */
    @Override
    public Map<Long, FamilyDO> getFamilyMap(Set<Long> familyIds) {
        if (CollectionUtil.isEmpty(familyIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, FamilyDO> familyMap = familyDAO.selectFamilyMap(familyIds);
        if (CollectionUtil.isEmpty(familyMap)) {
            return Collections.EMPTY_MAP;
        }
        return familyMap;
    }


    /**
     * 获取家族id(忽略状态)
     *
     * @param userId
     * @return
     */
    @Override
    public Long getFamilyIdIgnoreStatus(Long userId) {
        Long familyId = familyUserDAO.selectFamilyId(userId);
        if (familyId == null) {
            familyId = familyAuditDAO.selectFamilyIdIgnoreStatus(userId);
        }
        return familyId;
    }

    /**
     * 获取已经认证的技能
     *
     * @param userId
     * @return
     */
    @Override
    public List<SkillAuth> getSkillsAuth(Long userId) {
        return familySkillDAO.selectSkillAuth(getFamilyId(userId), userId);
    }

    /**
     * 获取已经认证的技能 不包括语音速配
     *
     * @param userId
     * @return
     */
    @Override
    public List<SkillAuth> getSkillsAuthIgnoreVoiceMatch(Long userId) {
        return familySkillDAO.selectSkillAuthIgnoreVoiceMatch(getFamilyId(userId), userId);
    }

    /**
     * 获取认证中的技能
     *
     * @param userId
     * @return
     */
    @Override
    public List<SkillsSubmit> getSkillsSubmit(Long userId) {
        Long familyIdIgnoreStatus = getFamilyIdIgnoreStatus(userId);
        FamilyAuditVO lastApplyInfo = getLastApplyInfo(userId, familyIdIgnoreStatus, null);
        if (lastApplyInfo != null) {
            if (lastApplyInfo.getApplyType() == ApplyTypeEnum.LEAVE.getCode() && lastApplyInfo.getAuditStatus() == AuditStatusEnum.PASS.getCode()) {
                return new ArrayList<>();
            }
        }

        Long lastLeaveTime = getLastLeaveTime(userId, familyIdIgnoreStatus);

        return familySkillAuditDAO.selectSkills(familyIdIgnoreStatus, userId, lastLeaveTime);
    }

    /**
     * 判断用户技能是否认证
     *
     * @param skillId
     * @param userId
     * @return
     */
    @Override
    public boolean isSkillAuth(Long skillId, Long userId) {
        Long familyId = getFamilyId(userId);
        if (familyId == null) {
            return false;
        }
        return familySkillDAO.countBySkillId(familyId, userId, skillId) > 0;
    }

    /**
     * 技能设置
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param orderPrice
     * @param orderTime
     * @param orderStatus
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setSkill(Long familyId, Long userId, Long skillId, Long orderPrice, String orderTime, Integer orderStatus) {
        return familySkillDAO.updateSkill(familyId, userId, skillId, orderPrice, orderTime, orderStatus) > 0;
    }

    /**
     * 获取技能
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @return
     */
    @Override
    public FamilySkillVO getSkill(Long familyId, Long userId, Long skillId) {
        return familySkillDAO.selectVOById(familyId, userId, skillId);
    }

    /**
     * 获取技能用户列表
     *
     * @param pageIndex
     * @param pageSize
     * @param skillId
     * @return
     */
    @Override
    public PageList<SkillUser> getSkillUserList(int pageIndex, int pageSize, Long skillId) {
        PageList<SkillUser> page = new PageList<>(pageIndex, pageSize);
        List<SkillUser> userList = familySkillDAO.selectSkillUserList(page.getPageCursor(), pageSize, skillId);
        if (CollectionUtil.isNotEmpty(userList)) {
            Set<Long> userIdSet = userList.stream().map(SkillUser::getUserId).collect(Collectors.toSet());
            // 用户所在房间
            Map<Long, Long> userRoomIdMap = rtcUserService.getUserRoomIdMap(userIdSet);
            Set<Long> roomIds = userRoomIdMap.values().stream().collect(Collectors.toSet());
            Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(roomIds);
            // 用户查询条件
            List<SkillUserQO> qoList = new ArrayList<>();
            for (SkillUser skillUser : userList) {
                SkillUserQO userQO = new SkillUserQO();
                userQO.setFamilyId(skillUser.getFamilyId());
                userQO.setUserId(skillUser.getUserId());
                userQO.setSkillId(skillId);
                userQO.setSkillTagId(skillUser.getSkillTagId());
                userQO.setAuditStatus(AuditStatusEnum.PASS.getCode());
                qoList.add(userQO);
            }
            List<FamilySkillAuditDO> auditDOList = familySkillAuditDAO.selectLastList(qoList);
            for (SkillUser skillUser : userList) {
                Long userId = skillUser.getUserId();
                Long familyId = skillUser.getFamilyId();
                Long skillTagId = skillUser.getSkillTagId();
                Optional<FamilySkillAuditDO> first = auditDOList.stream()
                        .filter(e -> userId.equals(e.getUserId()) && familyId.equals(e.getFamilyId()) && skillId.equals(skillUser.getSkillId()) && skillTagId.equals(e.getSkillTagId()))
                        .findFirst();
                skillUser.setVoice(first.map(FamilySkillAuditDO::getVoice).orElse(""));
                skillUser.setSkillIntro(first.map(FamilySkillAuditDO::getSkillIntro).orElse(""));
                skillUser.setDuration(first.map(FamilySkillAuditDO::getDuration).orElse(null));
                skillUser.setImages(first.map(FamilySkillAuditDO::getImages).orElse(""));
                Long roomId = userRoomIdMap.get(skillUser.getUserId());
                skillUser.setRoomId(roomId);
                skillUser.setRoomName(roomId == null ? null : Optional.ofNullable(roomMap.get(roomId)).map(RtcRoom::getRoomName).orElse(""));
                skillUser.setRoomCover(roomId == null ? null : Optional.ofNullable(roomMap.get(roomId)).map(RtcRoom::getRoomCover).orElse(""));
            }
        }
        page.setList(userList);
        page.setTotal(familySkillDAO.countSkillUsers(skillId));
        return page;
    }

    /**
     * 获取技能用户数量
     */
    @Override
    public Integer countSkillUsersSimple(Long skillId) {
        return Optional.ofNullable(familySkillDAO.countSkillUsersSimple(skillId)).orElse(0);
    }

    /**
     * 获取技能用户列表
     *
     * @param pageIndex
     * @param pageSize
     * @param skillId
     * @return
     */
    @Override
    public List<SkillUser> getSkillUserListSimple(int pageIndex, int pageSize, Long skillId) {
        return familySkillDAO.selectSkillUserListSimple((pageIndex - 1) * pageSize, pageSize, skillId);
    }

    /**
     * 获取技能用户数量
     */
    @Override
    public Integer countAvailableSkillUsersSimple(Long skillId) {
        return Optional.ofNullable(familySkillDAO.countAvailableSkillUsersSimple(skillId)).orElse(0);
    }

    /**
     * 获取技能用户列表
     *
     * @param pageIndex
     * @param pageSize
     * @param skillId
     * @return
     */
    @Override
    public List<SkillUser> getAvailableSkillUserList(int pageIndex, int pageSize, Long skillId) {
        return familySkillDAO.selectAvailableSkillUserList((pageIndex - 1) * pageSize, pageSize, skillId);
    }


    /**
     * 获取推荐技能用户列表
     *
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<SkillUser> getRecommendSkillUserList(int pageIndex, int pageSize) {
        PageList<SkillUser> page = new PageList<>(pageIndex, pageSize);
        List<SkillUser> userList = familySkillDAO.selectRecommendSkillUserList(page.getPageCursor(), pageSize);

        if (CollectionUtil.isNotEmpty(userList)) {
            Set<Long> userIdSet = userList.stream().map(SkillUser::getUserId).collect(Collectors.toSet());
            // 用户所在房间
            Map<Long, Long> userRoomIdMap = rtcUserService.getUserRoomIdMap(userIdSet);
            Set<Long> roomIds = userRoomIdMap.values().stream().collect(Collectors.toSet());
            Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(roomIds);
            // 用户查询条件
            List<SkillUserQO> qoList = new ArrayList<>();
            for (SkillUser skillUser : userList) {
                SkillUserQO userQO = new SkillUserQO();
                userQO.setFamilyId(skillUser.getFamilyId());
                userQO.setUserId(skillUser.getUserId());
                userQO.setSkillId(skillUser.getSkillId());
                userQO.setSkillTagId(skillUser.getSkillTagId());
                userQO.setAuditStatus(AuditStatusEnum.PASS.getCode());
                qoList.add(userQO);
            }
            List<FamilySkillAuditDO> auditDOList = familySkillAuditDAO.selectLastList(qoList);
            for (SkillUser skillUser : userList) {
                Long userId = skillUser.getUserId();
                Long familyId = skillUser.getFamilyId();
                Long skillId = skillUser.getSkillId();
                Long skillTagId = skillUser.getSkillTagId();

                Optional<FamilySkillAuditDO> first = auditDOList.stream()
                        .filter(e -> userId.equals(e.getUserId()) && familyId.equals(e.getFamilyId()) && skillId.equals(skillUser.getSkillId()) && skillTagId.equals(e.getSkillTagId()))
                        .findFirst();
                skillUser.setVoice(first.map(FamilySkillAuditDO::getVoice).orElse(""));
                skillUser.setSkillIntro(first.map(FamilySkillAuditDO::getSkillIntro).orElse(""));
                skillUser.setDuration(first.map(FamilySkillAuditDO::getDuration).orElse(null));
                skillUser.setImages(first.map(FamilySkillAuditDO::getImages).orElse(""));

                Long roomId = userRoomIdMap.get(skillUser.getUserId());
                skillUser.setRoomId(roomId);
                skillUser.setRoomName(roomId == null ? null : Optional.ofNullable(roomMap.get(roomId)).map(RtcRoom::getRoomName).orElse(""));
                skillUser.setRoomCover(roomId == null ? null : Optional.ofNullable(roomMap.get(roomId)).map(RtcRoom::getRoomCover).orElse(""));
            }
        }
        page.setList(userList);
        page.setTotal(familySkillDAO.countSkillUsers(null));
        return page;
    }

    /**
     * 获取家族名称
     *
     * @param familyId
     * @return
     */
    @Override
    public String getFamilyName(Long familyId) {
        return familyDAO.selectFamilyName(familyId);
    }

    /**
     * 判断家族认证是否存在
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @return
     */
    @Override
    public boolean isFamilyAuditExist(Long familyId, Long userId, Integer applyType, Integer auditStatus) {
        return familyAuditDAO.countByFamilyId(familyId, userId, applyType, auditStatus) > 0;
    }

    /**
     * 获取家族签约记录
     *
     * @param familyId
     * @param userId
     * @param applyType
     * @param auditStatus
     * @return
     */
    @Override
    public FamilyAuditDO getFamilyAudit(Long familyId, Long userId, Integer applyType, Integer auditStatus) {
        return familyAuditDAO.selectFamilyAudit(familyId, userId, applyType, auditStatus);
    }

    /**
     * 判断技能认证是否存在
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param auditStatus
     * @return
     */
    @Override
    public boolean isSkillAuditExist(Long familyId, Long userId, Long skillId, Integer auditStatus) {
        return familySkillAuditDAO.countBySkillId(familyId, userId, skillId, auditStatus) > 0;
    }

    /**
     * 是否提交过技能认证
     *
     * @param userId
     * @param familyId
     * @return
     */
    @Override
    public boolean isSkillSubmit(Long userId, Long familyId) {
        return familySkillAuditDAO.countByUserId(userId, familyId) > 0;
    }

    /**
     * 获取用户技能详情
     *
     * @param familyId
     * @param userId
     * @param skillId
     * @param curUserId
     * @return
     */
    @Override
    public SkillUser getSkillUserDetail(Long familyId, Long userId, Long skillId, Long curUserId) {
        SkillUser skillUser = familySkillDAO.selectSkillUserById(familyId, userId, skillId);
        if (skillUser != null) {
            if (curUserId != null) {
                skillUser.setFollowStatus(socialRelationService.isFollower(curUserId, userId) ? BoolIntVal.True : BoolIntVal.False);
            } else {
                skillUser.setFollowStatus(BoolIntVal.False);
            }
            FamilySkillAuditDO auditDO = familySkillAuditDAO.selectLast(userId, familyId, skillId, skillUser.getSkillTagId(), AuditStatusEnum.PASS.getCode());
            skillUser.setVoice(Optional.ofNullable(auditDO).map(FamilySkillAuditDO::getVoice).orElse(""));
            skillUser.setSkillIntro(Optional.ofNullable(auditDO).map(FamilySkillAuditDO::getSkillIntro).orElse(""));
            skillUser.setDuration(Optional.ofNullable(auditDO).map(FamilySkillAuditDO::getDuration).orElse(null));
            skillUser.setImages(Optional.ofNullable(auditDO).map(FamilySkillAuditDO::getImages).orElse(""));
            Long roomId = rtcUserService.getUserStayRoomId(userId);
            roomId = roomId != null && roomId > 0 ? roomId : null;
            skillUser.setRoomId(roomId);
            RtcRoom rtcRoom = null;
            if (roomId != null) {
                rtcRoom = rtcRoomService.getRoom(roomId);
            }
            skillUser.setRoomName(roomId == null ? null : Optional.ofNullable(rtcRoom).map(RtcRoom::getRoomName).orElse(""));
            skillUser.setRoomCover(roomId == null ? null : Optional.ofNullable(rtcRoom).map(RtcRoom::getRoomCover).orElse(""));
        }
        return skillUser;
    }

    /**
     * 是否家族长
     *
     * @param userId
     */
    @Override
    public boolean isFamilyPresident(Long userId) {
        return familyDAO.countByPresident(userId) > 0;
    }

    /**
     * 增加接通次数
     *
     * @param matchUserId
     */
    @Override
    public boolean incrementConnectTimes(Long matchUserId) {
        return familyUserDAO.incrementConnectTimes(matchUserId, getFamilyId(matchUserId)) > 0;
    }

    /**
     * 增加匹配次数
     *
     * @param matchUserId
     */
    @Override
    public boolean incrementMatchTimes(Long matchUserId) {
        return familyUserDAO.incrementMatchTimes(matchUserId, getFamilyId(matchUserId)) > 0;
    }

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param auditTime
     * @param adminId
     * @return
     */
    @Override
    public boolean setFamilyAuditStatus(Long auditId, Integer auditStatus, String auditMsg, Long auditTime, Long adminId) {
        return familyAuditDAO.updateStatus(auditId, auditStatus, auditMsg, auditTime, adminId) > 0;
    }

    /**
     * 是否有技能
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, Integer> getUserSkillMap(Set<Long> userIds) {
        if (CollectionUtil.isNotEmpty(userIds)) {
            List<KeyValueMapDO> resList = familyUserDAO.getUserSkillList(userIds);
            if (CollectionUtil.isNotEmpty(resList)) {
                Map<Long, Integer> res = new HashMap<>();
                resList.forEach(i -> {
                    if (i.getMValue() > 0) {
                        res.put(i.getMKey(), 1);
                    } else {
                        res.put(i.getMKey(), 0);
                    }
                });
                return res;
            }
        }
        return new HashMap<>();
    }

    /**
     * 发送用户退出家族消息
     *
     * @return
     */
    @Override
    public boolean sendUserExitFamilyMsg(QueueUserExitFamilyMsg msg) {
        return rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FAMILY_USER_EXIT, msg.serialize());
    }

}
