package com.starbuds.server.service.user.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.ForbiddenWayTypeEnum;
import com.starbuds.server.common.constant.user.LogoutStatusEnum;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.mq.ActivityLoginUserMsg;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.pojo.user.UserLoginHisDTO;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.SecurityService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.redis.RedisUtil;
import com.starbuds.server.service.user.rocket.RocketProducer;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 安全服务实现
 *
 * @author Lay
 * @date 2019/11/27
 */
@Service(version = "1.0.0")
@Slf4j
public class SecurityServiceImpl implements SecurityService {

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    UserTokenDAO userTokenDAO;

    @Resource
    UserSecurityDAO userSecurityDAO;

    @Resource
    UserForbiddenDAO userForbiddenDAO;

    @Resource
    UserLoginHisDAO userLoginHisDAO;

    @Resource
    UserOauthDAO userOauthDAO;

    @Resource
    UserDAO userDAO;

    @Resource
    DeviceDAO deviceDAO;

    @Resource
    UserLogoutDAO userLogoutDAO;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    RocketProducer rocketProducer;

    static final String AES_KEY = "vbrde2efr7w9gcrbdiov3dyf5l3c5r0r";

    /**
     * 创建令牌
     *
     * @param clientType
     * @param appType
     * @param userId
     * @param deviceId
     * @param expireTime
     * @param hisId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String createToken(ClientType clientType, AppType appType, Long userId, String deviceId, long expireTime, Long hisId, String rdid) {
        String field = String.format("%d:%d:%d", userId, clientType.getValue(), appType.getValue());
        String value = String.format("%s:%d", deviceId, expireTime);

        try {
            String toEncrypt = field + "_" + value;
            AES aes = new AES(AES_KEY.getBytes());
            String token = aes.encryptBase64(toEncrypt);

            ClientType opClientType = null;
            if (ClientType.iOS == clientType) {
                opClientType = ClientType.Android;
            } else if (ClientType.Android == clientType) {
                opClientType = ClientType.iOS;
            }

            if (opClientType != null) {
                // 移除对端
                String opFiled = String.format("%d:%d:%d", userId, opClientType.getValue(), appType.getValue());
                // 移除令牌
                redisUtil.hDelete(RedisUserKeys.USER_TOKEN, opFiled);
            }

            // 写入数据库
            UserTokenDO userTokenDO = new UserTokenDO();
            userTokenDO.setHisId(hisId);
            userTokenDO.setClientType(clientType.getValue());
            userTokenDO.setAppType(appType.getValue());
            userTokenDO.setUserId(userId);
            userTokenDO.setDeviceId(deviceId);
            userTokenDO.setCreateTime(System.currentTimeMillis());
            userTokenDO.setExpireTime(expireTime);
            userTokenDO.setThirdDeviceId(rdid);
            userTokenDAO.insert(userTokenDO);

            // 写入缓存
            redisUtil.hPut(RedisUserKeys.USER_TOKEN, field, value);

            return token;
        } catch (Exception e) {
            log.error("令牌创建失败", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return null;
    }

    /**
     * 如果不存在生成令牌
     *
     * @param clientType
     * @param appType
     * @param userId
     * @param deviceId
     * @param expireTime
     * @param hisId
     * @return
     */
    @Override
    public String createTokenIfNotExist(ClientType clientType, AppType appType, Long userId, String deviceId, long expireTime, Long hisId) {
        String field = String.format("%d:%d:%d", userId, clientType.getValue(), appType.getValue());
        Object object = redisUtil.hGet(RedisUserKeys.USER_TOKEN, field);
        if (object != null) {
            String toEncrypt = field + "_" + object.toString();
            AES aes = new AES(AES_KEY.getBytes());
            String token = aes.encryptBase64(toEncrypt);
            return token;
        }
        return createToken(clientType, appType, userId, deviceId, expireTime, hisId, "");
    }

    /**
     * 移除令牌
     *
     * @param userId
     * @return
     */
    @Override
    public boolean removeToken(Long userId) {
        try {
            List<String> keys = new ArrayList<>();
            // 可能出现的key
            for (ClientType obj : ClientType.values()) {
                keys.add(String.format("%d:%d:%d", userId, obj.getValue(), AppType.App.getValue()));
            }
            if (CollectionUtils.isNotEmpty(keys)) {
                redisUtil.hDelete(RedisUserKeys.USER_TOKEN, keys.toArray(new String[0]));
            }
            return true;
        } catch (Exception e) {
            log.error("令牌移除失败");
        }
        return false;
    }

    /**
     * 获取令牌
     *
     * @param token
     * @return
     */
    @Override
    public TokenBO getToken(String token) {
        try {
            if (StringUtil.isNullOrEmpty(token)) {
                return null;
            }
            AES aes = new AES(AES_KEY.getBytes());
            String tokenDecrypted = aes.decryptStr(token);
            if (StringUtils.isNotBlank(tokenDecrypted)) {
                String[] fieldAndValue = tokenDecrypted.split("_");
                String field = fieldAndValue[0];
                String value = fieldAndValue[1];

                // 判断缓存中是否存在
                Object cachedValue = redisUtil.hGet(RedisUserKeys.USER_TOKEN, field);
                if (cachedValue != null && value.equals(cachedValue.toString())) {
                    String[] fieldItems = field.split(":");
                    String[] valueItems = value.split(":");
                    TokenBO tokenBO = new TokenBO();
                    tokenBO.setUserId(Convert.toLong(fieldItems[0]));
                    tokenBO.setClientType(ClientType.toEnum(Convert.toInt(fieldItems[1])));
                    tokenBO.setAppType(AppType.toEnum(Convert.toInt(fieldItems[2])));
                    tokenBO.setDeviceId(valueItems[0]);
                    tokenBO.setExpireTime(Convert.toLong(valueItems[1]));

                    return tokenBO;
                }
            }
        } catch (Exception e) {
            log.error("令牌校验失败", e);
        }
        return null;
    }

    /**
     * 是否市场审核人员手机号
     *
     * @param userPhone
     * @return
     */
    @Override
    public boolean isMarketAuditorPhone(String userPhone, String areaCode) {
        Integer isMarketAuditor = Optional.ofNullable(userSecurityDAO.selectIsMarketAuditorByPhone(userPhone, areaCode)).orElse(BoolIntVal.False);
        return BoolIntVal.True == isMarketAuditor;
    }

    /**
     * 判断手机号是否存在
     *
     * @param userPhone
     * @param areaCode
     * @return
     */
    @Override
    public boolean isPhoneExists(String userPhone, String areaCode) {
        return userSecurityDAO.countByPhone(userPhone, areaCode) > 0;
    }

    /**
     * 根据手机号获取用户 id
     *
     * @param userPhone
     * @param areaCode
     * @return
     */
    @Override
    public Long getUserIdByPhone(String userPhone, String areaCode) {
        return userSecurityDAO.selectIdByPhone(userPhone, areaCode);
    }

    /**
     * 创建登录记录
     *
     * @param userLoginHisDTO
     * @return
     */
    @Override
    public Long addLoginHisAndSendMsg(UserLoginHisDTO userLoginHisDTO) {
        UserLoginHisDO userLoginHisDO = new UserLoginHisDO();
        userLoginHisDO.setUserId(userLoginHisDTO.getUserId());
        userLoginHisDO.setHisLoginType(userLoginHisDTO.getHisLoginType().getValue());
        userLoginHisDO.setHisLoginParams(userLoginHisDTO.getHisLoginParams());
        userLoginHisDO.setHisLoginResult(userLoginHisDTO.getHisLoginResult());
        userLoginHisDO.setHisHeaders(userLoginHisDTO.getHisHeaders());
        userLoginHisDO.setHisIp(userLoginHisDTO.getHisIp());
        userLoginHisDO.setHisCreateTime(System.currentTimeMillis());

        try {
            userLoginHisDAO.insert(userLoginHisDO);
            ActivityLoginUserMsg msg = new ActivityLoginUserMsg();
            msg.setUserId(userLoginHisDTO.getUserId());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ACTIVITY_LOGING, msg.serialize());
            return userLoginHisDO.getHisId();
        } catch (Exception e) {
            log.error("登录记录创建失败", e);
        }
        return null;
    }

    /**
     * 创建 IM 令牌
     *
     * @param userId
     * @param expireSeconds
     * @return
     */
    @Override
    public String createImToken(Long userId, long expireSeconds) {
        if (expireSeconds == 0L) {
            // 设置一年
            expireSeconds = 365 * 24 * 3600L;
        }

        UserCardVO userCard = userDAO.selectNameAndAvatarById(userId);
        ImUserProfile imUserProfile = new ImUserProfile();
        imUserProfile.setUserId(userId);
        imUserProfile.setUserName(userCard.getUserName());
        imUserProfile.setUserAvatar(userCard.getUserAvatar());
        String imToken = imService.getToken(imUserProfile);

        if (StringUtils.isNotBlank(imToken)) {
            // 持久化
            userSecurityDAO.updateImToken(userId, imToken);
            return imToken;
        }

        return null;
    }

    /**
     * 根据 UnionId 获取用户 id
     *
     * @param unionId
     * @param channelId
     * @param type
     * @return
     */
    @Override
    public Long getUserIdByUnionId(String unionId, Long channelId, Integer type) {
        return userOauthDAO.selectUserIdByUnionId(unionId, channelId, type);
    }

    /**
     * 判断用户是否已绑定第三方账号
     *
     * @param userId
     * @param channelId
     * @param type
     * @return
     */
    @Override
    public boolean isUserOauthBind(Long userId, Long channelId, Integer type) {
        return userOauthDAO.countById(userId, channelId, type) > 0;
    }

    /**
     * 保存oauth信息
     *
     * @param userId
     * @param channelId
     * @param type
     * @param unionId
     * @param openId
     * @return
     */
    @Override
    public boolean saveOauthInfo(Long userId, Long channelId, Integer type, String unionId, String openId) {
        UserOauthDO userOauthDO = new UserOauthDO();
        userOauthDO.setUserId(userId);
        userOauthDO.setChannelId(channelId);
        userOauthDO.setType(type);
        userOauthDO.setUnionId(unionId);
        userOauthDO.setOpenId(openId);
        return userOauthDAO.insert(userOauthDO) > 0;
    }

    /**
     * 绑定手机号
     *
     * @param userId
     * @param userPhone
     * @param areaCode
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean bindPhone(Long userId, String userPhone, String areaCode) {
        if (userDAO.updatePhone(userId, userPhone) == 1) {
            return userSecurityDAO.updatePhone(userId, userPhone, areaCode) == 1;
        }
        return false;
    }

    @Override
    public boolean isOauthBind(Long userId, Long channelId, Integer type) {
        return userOauthDAO.countById(userId, channelId, type) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean applyLogout(UserLogoutDO logoutDO) {
        return userLogoutDAO.insertOrUpdate(logoutDO) >= 1;
    }

    @Override
    public Integer getLogoutStatus(Long userId) {
        return userLogoutDAO.selectStatus(userId);
    }

    @Override
    public boolean isLogout(Long userId) {
        UserLogoutDO logoutDO = userLogoutDAO.selectById(userId);
        return logoutDO != null && logoutDO.getStatus() == LogoutStatusEnum.WAITING.getCode() && logoutDO.getLogoutTime() < System.currentTimeMillis();
    }

    @Override
    public boolean isCoolPeriod(Long userId) {
        UserLogoutDO logoutDO = userLogoutDAO.selectById(userId);
        return logoutDO != null && logoutDO.getStatus() == LogoutStatusEnum.WAITING.getCode() && logoutDO.getLogoutTime() >= System.currentTimeMillis();
    }

    @Override
    public Long getLogoutTime(long userId) {
        return userLogoutDAO.selectLogoutTime(userId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean cancelLogout(Long userId) {
        Long curTime = System.currentTimeMillis();
        return userLogoutDAO.updateStatus(userId, curTime, LogoutStatusEnum.RESUME.getCode()) == 1;
    }


    @Override
    public List<Long> getLogoutUserIds(Long createTime) {
        return userLogoutDAO.selectUserIds(createTime, LogoutStatusEnum.WAITING.getCode());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean executeLogout(List<Long> userIds) {
        if (CollectionUtils.isNotEmpty(userIds)) {
            for (Long userId : userIds) {
                // 清理tb_user表、tb_user_security表、tb_user_oauth
                UserDO userDO = userDAO.selectById(userId);
                UserSecurityDO userSecurityDO = userSecurityDAO.selectById(userId);
                List<UserOauthDO> userOauthDOS = userOauthDAO.selectByUserId(userId);
                // 移除token
                removeToken(userId);
                // es清理
                userSearchService.deleteById(userId);
                // 清理邀请关系+关注关系
                // 修改logout状态 = 已注销
            }
        }
        return true;
    }

    /**
     * 判断是否设置了密码
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isUserPassSet(Long userId) {
        return StringUtils.isNotEmpty(userSecurityDAO.selectUserPass(userId));
    }

    /**
     * 验证密码
     *
     * @param userId
     * @param userPass
     * @return
     */
    @Override
    public boolean verifyPass(Long userId, String userPass) {
        String dbPass = userSecurityDAO.selectUserPass(userId);
        if (userPass != null) {
            if (dbPass.equals(encryptPass(userPass))) {
                // 清除缓存
                redisUtil.delete(RedisUserKeys.getLoginPassTimes(userId));
                return true;
            } else {
                // 缓存密码错误次数
                redisUtil.increaseAndExpire(RedisUserKeys.getLoginPassTimes(userId), 1L, TimeConstants.MIN_MILLS, TimeUnit.MILLISECONDS);
                return false;
            }
        }
        return false;
    }

    /**
     * 是否密码错误超限
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isPassErrorOverTimes(Long userId) {
        int times = Optional.ofNullable(redisUtil.get(RedisUserKeys.getLoginPassTimes(userId))).map(Convert::toInt).orElse(0);
        return times > 10;
    }

    @Override
    public boolean isPhoneBind(Long userId) {
        return StringUtils.isNotEmpty(userSecurityDAO.selectPhoneById(userId));
    }

    @Override
    public boolean setUserPass(Long userId, String userPass) {
        return userSecurityDAO.updateUserPass(userId, encryptPass(userPass)) > 0;
    }

    /**
     * 是否被封禁
     *
     * @param userId
     * @param ip
     * @param deviceId
     * @return
     */
    @Override
    public boolean isForbidden(Long userId, String ip, String deviceId) {
        if (Optional.ofNullable(deviceDAO.selectById(deviceId)).map(e -> e.getDeviceLocked() == BoolIntVal.True).orElse(false)) {
            return true;
        }
        return userForbiddenDAO.selectCount(userId, ip, deviceId) > 0;
    }

    /**
     * 获取封禁方式
     *
     * @param userId
     * @param ip
     * @param deviceId
     * @return
     */
    @Override
    public Integer getForbiddenWay(Long userId, String ip, String deviceId) {
        if (Optional.ofNullable(deviceDAO.selectById(deviceId)).map(e -> e.getDeviceLocked() == BoolIntVal.True).orElse(false)) {
            return ForbiddenWayTypeEnum.DEVICE.getCode();
        }
        return userForbiddenDAO.selectWay(userId, ip, deviceId);
    }

    private String encryptPass(String pass) {
        return SecureUtil.sha256(SecureUtil.md5(pass).toLowerCase()).toLowerCase();
    }
}
