package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.user.FriendAddAgreeMsg;
import com.starbuds.server.common.pojo.message.im.user.FriendAddDenyMsg;
import com.starbuds.server.common.pojo.message.im.user.FriendAddRequestMsg;
import com.starbuds.server.common.pojo.message.im.user.UserFollowMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.UserFollowLimitProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/16 20:07
 * @Description 社交关系服务
 */
@Slf4j
@Service(version = "1.0.0")
public class SocialRelationServiceImpl implements SocialRelationService {

    @Reference(version = "1.0.0")
    ImService imService;

    @Autowired
    FriendReqDAO friendReqDAO;

    @Autowired
    FriendRelationDAO friendRelationDAO;

    @Autowired
    FollowRelationDAO followRelationDAO;

    @Resource
    UserFollowRelationDAO userFollowRelationDAO;

    @Autowired
    UserDAO userDAO;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Autowired
    UserSwitchDAO userSwitchDAO;

    @Resource
    AnchorDAO anchorDAO;

    @Resource
    UserInviteDAO userInviteDAO;

    @Resource
    UserRelationStatsDAO userRelationStatsDAO;

    @Resource
    UserFollowLimitProvider userFollowLimitProvider;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;


    /**
     * 关注用户
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean followUser(Long userId, Long targetId) {

        long current = System.currentTimeMillis();

        String relationIdA = SecureUtil.md5(String.format("%d:%d", userId, targetId));
        String relationIdB = SecureUtil.md5(String.format("%d:%d", targetId, userId));
        boolean isFirstFollow = false;

        // 如果记录存在 更新关系
        FollowRelationDO followRelationDO = followRelationDAO.selectByIds(relationIdA, relationIdB);
        if (followRelationDO != null && followRelationDO.getStatus() != CommonStatus.Deleted) {

            // 判断是否重复设置
            FollowRelationDirectionEnum followRelationDirection = FollowRelationDirectionEnum.toEnum(followRelationDO.getRelationDirection());
            boolean isRelation = userId.equals(followRelationDO.getUserId()) && FollowRelationDirectionEnum.ATOB == followRelationDirection
                    || userId.equals(followRelationDO.getTargetId()) && FollowRelationDirectionEnum.BTOA == followRelationDirection
                    || FollowRelationDirectionEnum.EACH == followRelationDirection;
            if (isRelation) {
                // 重复关注
                return false;
            } else {
                // 设置为互相关注
                try {
                    int updated = followRelationDAO.updateByDirection(followRelationDO.getRelationId(),
                            FollowRelationDirectionEnum.EACH.getValue(),
                            followRelationDO.getStatus(),
                            followRelationDO.getRelationDirection(),
                            followRelationDO.getStatus());

                    if (updated == 0) {
                        throw new Exception("关注关系设置失败");
                    }
                    updateUserRelationInc(userId, targetId);
                } catch (Exception e) {
                    return false;
                }
            }
        } else {
            if (followRelationDO == null) {
                isFirstFollow = true;

                followRelationDO = new FollowRelationDO();
                followRelationDO.setRelationId(relationIdA);
                followRelationDO.setUserId(userId);
                followRelationDO.setTargetId(targetId);
                followRelationDO.setRelationDirection(FollowRelationDirectionEnum.ATOB.getValue());
                followRelationDO.setFollowTime(current);
                followRelationDO.setStatus(CommonStatus.Normal);
                followRelationDO.setUpdateTime(current);

                // 插入
                followRelationDAO.insertWhereNotExists(followRelationDO, relationIdB);

                //统计增加粉丝和关注数
                updateUserRelationInc(userId, targetId);
            } else {
                FollowRelationDirectionEnum followRelationDirection;
                if (followRelationDO.getUserId().equals(userId)) {
                    // 如果用户 id 在 userId 上，则说明是正向关注
                    followRelationDirection = FollowRelationDirectionEnum.ATOB;
                } else {
                    // 否则即为反向关注
                    followRelationDirection = FollowRelationDirectionEnum.BTOA;
                }
                // 更新
                try {
                    int updated = followRelationDAO.updateByDirection(followRelationDO.getRelationId(), followRelationDirection.getValue(), CommonStatus.Normal, followRelationDO.getRelationDirection(), followRelationDO.getStatus());
                    if (updated == 0) {
                        throw new Exception("关注关系设置失败");
                    }
                    updateUserRelationInc(userId, targetId);
                } catch (Exception e) {
                    log.error("关注失败", e);
                    return false;
                }
            }
        }

        // 关注目标是主播且是首次关注
//        if (anchorService.isAnchor(targetId)) {
//            // 发送关注消息
//            LiveUserFollowMsg liveUserFollowMsg = new LiveUserFollowMsg();
//            imService.sendLiveRoomMsg(ImTargetType.GROUP, userId, targetId, liveUserFollowMsg);
//        }

        // 增加次数
        userFollowLimitProvider.increase(userId);

        // 关注成功 发送im通知
//        UserFollowMsg userFollowMsg = new UserFollowMsg();
//        userFollowMsg.setUserId(userId);
//        userFollowMsg.setUserCardVO(userDAO.selectCardById(userId));
//        imService.sendCustomImMsg(null, targetId, userFollowMsg, null);
        imService.sendTextMsg(userId, targetId, "我关注了你");
        return true;
    }

    /**
     * 取消关注
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean unFollowUser(Long userId, Long targetId) {

        String relationIdA = SecureUtil.md5(String.format("%d:%d", userId, targetId));
        String relationIdB = SecureUtil.md5(String.format("%d:%d", targetId, userId));

        // 查询原关系
        FollowRelationDO followRelationDO = followRelationDAO.selectByIds(relationIdA, relationIdB);
        // 如果不存在关系说明原本就没有关注关系
        if (followRelationDO == null || CommonStatus.Deleted == followRelationDO.getStatus()) {
            return false;
        }

        // 判断原关系
        FollowRelationDirectionEnum followRelationDirection = FollowRelationDirectionEnum.toEnum(followRelationDO.getRelationDirection());

        // 判断是否原本就是取消关注状态
        boolean isRelation = userId.equals(followRelationDO.getUserId()) && FollowRelationDirectionEnum.BTOA == followRelationDirection
                || userId.equals(followRelationDO.getTargetId()) && FollowRelationDirectionEnum.ATOB == followRelationDirection;
        if (isRelation) {
            return false;
        }

        // 如果是互相关注
        FollowRelationDirectionEnum newRelation;
        Integer newStatus;
        if (FollowRelationDirectionEnum.EACH == followRelationDirection) {
            // 变为单项关注
            if (userId.equals(followRelationDO.getUserId())) {
                newRelation = FollowRelationDirectionEnum.BTOA;
            } else {
                // 说明 userId 在 targetId 上
                newRelation = FollowRelationDirectionEnum.ATOB;
            }

            // 更新关注关系
            newStatus = followRelationDO.getStatus();
        } else {
            // 删除关注关系
            newRelation = followRelationDirection;
            newStatus = CommonStatus.Deleted;
        }

        try {
            int updated = followRelationDAO.updateByDirection(followRelationDO.getRelationId(), newRelation.getValue(), newStatus, followRelationDO.getRelationDirection(), followRelationDO.getStatus());
            if (updated == 0) {
                throw new Exception("关注关系更新失败");
            }
            //统计减少粉丝和关注数
            updateUserRelationDec(userId, targetId);

        } catch (Exception e) {
            log.error("取消关注失败", e);
            return false;
        }

        return true;
    }

    /**
     * 添加好友请求
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @param reqMsg   请求添加好友时的验证消息
     * @return 是否成功
     */
    @Override
    public boolean addFriendReq(Long userId, Long targetId, String reqMsg) {
        long current = System.currentTimeMillis();

        try {
            // 不允许添加好友 直接拒绝
            int allowAdd = Optional.ofNullable(userSwitchDAO.getSwitchStatus(String.format("%d:%d", targetId, SwitchTypeEnum.ALLOW_ADD_FRIENDS.getCode()))).orElse(SwitchTypeEnum.ALLOW_ADD_FRIENDS.getDefaultValue());
            log.info("用户id{}，添加好友开关状态：{}", targetId, allowAdd);
            if (BoolIntVal.False == allowAdd) {
                FriendReqDO addReq = new FriendReqDO();
                addReq.setUserId(userId);
                addReq.setTargetId(targetId);
                addReq.setReqMsg(reqMsg);
                addReq.setOptMsg("该用户不允许添加好友");
                addReq.setCreateTime(current);
                addReq.setUpdateTime(current);
                addReq.setStatus(FriendReqStatusEnum.DENY.getCode());
                friendReqDAO.insert(addReq);
            } else {
                FriendReqDO addReq = new FriendReqDO();
                addReq.setUserId(userId);
                addReq.setTargetId(targetId);
                addReq.setReqMsg(reqMsg);
                addReq.setCreateTime(current);
                addReq.setUpdateTime(current);
                addReq.setStatus(FriendReqStatusEnum.WAIT.getCode());
                friendReqDAO.insert(addReq);

                // 请求发送对方
                FriendAddRequestMsg requestMsg = new FriendAddRequestMsg();
                requestMsg.setReqId(addReq.getReqId());
                requestMsg.setUserId(userId);
                requestMsg.setTargetId(targetId);
                requestMsg.setUserName(userDAO.selectNameById(userId));
                requestMsg.setContent(String.format("%s申请加你为好友", requestMsg.getUserName()));
                imService.sendCustomImMsg(null, targetId, requestMsg, null);

                // 是否允许自动添加
                int autoPass = Optional.ofNullable(userSwitchDAO.getSwitchStatus(String.format("%d:%d", targetId, SwitchTypeEnum.AUTO_PASS_FRIENDS.getCode()))).orElse(SwitchTypeEnum.AUTO_PASS_FRIENDS.getDefaultValue());
                if (BoolIntVal.True == autoPass) {
                    agreeFriendReq(addReq.getReqId(), SwitchTypeEnum.AUTO_PASS_FRIENDS.getDesc());
                }
            }

            return true;
        } catch (Exception e) {
            log.error("好友申请失败", e);
        }

        return false;
    }

    /**
     * 同意好友申请
     *
     * @param reqId  好友申请请求的id
     * @param optMsg 处理意见
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean agreeFriendReq(Long reqId, String optMsg) {

        FriendReqDO reqDO = friendReqDAO.selectById(reqId);
        if (reqDO != null) {
            // 通过
            long current = System.currentTimeMillis();
            FriendReqDO update = new FriendReqDO();
            update.setReqId(reqId);
            update.setStatus(FriendReqStatusEnum.AGREE.getCode());
            update.setOptMsg(optMsg);
            update.setUpdateTime(current);
            friendReqDAO.updateById(update);

            Long[] ids = identityService.getIds(2);

            // 好友
            FriendRelationDO relationDO = new FriendRelationDO();
            relationDO.setRelationId(Convert.toStr(ids[0]));
            relationDO.setReqId(reqId);
            relationDO.setUserId(reqDO.getUserId());
            relationDO.setTargetId(reqDO.getTargetId());
            relationDO.setCreateTime(current);
            relationDO.setUpdateTime(current);
            relationDO.setStatus(CommonStatus.Normal);
            friendRelationDAO.insertOrUpdate(relationDO);

            FriendRelationDO friendRelationDO = new FriendRelationDO();
            friendRelationDO.setRelationId(Convert.toStr(ids[1]));
            friendRelationDO.setReqId(reqId);
            friendRelationDO.setUserId(reqDO.getTargetId());
            friendRelationDO.setTargetId(reqDO.getUserId());
            friendRelationDO.setCreateTime(current);
            friendRelationDO.setUpdateTime(current);
            friendRelationDO.setStatus(CommonStatus.Normal);
            friendRelationDAO.insertOrUpdate(friendRelationDO);

            // 同意消息发送给发起人
            FriendAddAgreeMsg agreeMsg = new FriendAddAgreeMsg();
            agreeMsg.setReqId(reqId);
            agreeMsg.setUserId(reqDO.getUserId());
            agreeMsg.setTargetId(reqDO.getTargetId());
            agreeMsg.setTargetName(userDAO.selectNameById(reqDO.getTargetId()));
            agreeMsg.setContent(String.format("%s通过了你的好友请求", agreeMsg.getTargetName()));
            imService.sendCustomImMsg(null, reqDO.getUserId(), agreeMsg, null);

            return true;
        }
        log.info("未找到该条好友申请记录");
        return false;
    }

    /**
     * 获取好友申请
     *
     * @param reqId 好友申请请求的id
     * @return 好友申请
     */
    @Override
    public FriendReqDO getFriendReqById(Long reqId) {
        return friendReqDAO.selectById(reqId);
    }

    /**
     * 是否好友关系
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @return 是否好友关系
     */
    @Override
    public boolean isFriend(Long userId, Long targetId) {
        return isFriendFromDb(userId, targetId);
    }

    /**
     * 获取好友状态
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public Integer getFriendStatus(Long userId, Long targetId) {
        Integer status = friendReqDAO.selectLastFriendReqStatus(userId, targetId);
        // 从未添加或拒绝时
        if (status == null || status == FriendReqStatusEnum.DENY.getCode()) {
            return FriendStatusEnum.ADD_FRIEND.getCode();
        }
        // 等待通过
        if (status == FriendReqStatusEnum.WAIT.getCode()) {
            return FriendStatusEnum.ALREADY_APPLY.getCode();
        }
        // 已是好友
        return FriendStatusEnum.ALREADY_ADD.getCode();
    }

    @Override
    public boolean isFriendFromDb(Long userId, Long targetId) {
        return friendRelationDAO.isFriendRelation(userId, targetId) > 0;
    }

    @Override
    public List<Long> getFriendUserIds(Long userId) {
        return friendRelationDAO.selectTargetIds(userId);
    }

    /**
     * 是否关注
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @return 是否关注
     */
    @Override
    public boolean isFollower(Long userId, Long targetId) {
        return isFollowerFromDb(userId, targetId);
    }

    @Override
    public boolean isFollowerFromDb(Long userId, Long targetId) {
        return followRelationDAO.isFollower(userId, targetId) > 0;
    }

    /**
     * 通过粉丝推断
     *
     * @param userId
     * @param targetId
     * @param list     userId 的粉丝用户
     * @return
     */
    public boolean isFollowerByFansList(Long userId, Long targetId, List<RelationDTO> list) {
        if (CollectionUtil.isEmpty(list)) {
            // 如果没有关系，从数据库查询 userId 是否关注 targetId
            return isFollowerFromDb(userId, targetId);
        }
        // 过滤后，判断userId 和 targetId 的关系
        List<Integer> directions = list.stream().filter(e -> e.getUserId().equals(targetId)).map(e -> e.getRelationDirection()).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(directions)) {
            // 如果没有关系，从数据库查询 userId 是否关注 targetId
            return isFollowerFromDb(userId, targetId);
        }
        // 相互关注 或 正向关注
        if (directions.get(0) > FollowRelationDirectionEnum.BTOA.getValue()) {
            return true;
        }
        return false;
    }

    /**
     * 通过关注推断
     *
     * @param targetId
     * @param userId
     * @param list     userId 关注的用户
     * @return
     */
    public boolean isFollowerByFollowList(Long targetId, Long userId, List<RelationDTO> list) {
        if (CollectionUtil.isEmpty(list)) {
            // 如果没有关系，从数据库查询 targetId 是否关注 userId
            return isFollowerFromDb(targetId, userId);
        }
        List<Integer> directions = list.stream().filter(e -> e.getUserId().equals(targetId)).map(e -> e.getRelationDirection()).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(directions)) {
            // 如果没有关系，从数据库查询 targetId 是否关注 userId
            return isFollowerFromDb(targetId, userId);
        }
        // 相互关注 或 反向关注
        if (directions.get(0) < FollowRelationDirectionEnum.ATOB.getValue()) {
            return true;
        }
        return false;
    }

    @Override
    public Map<Long, Boolean> isFriends(Long userId, Set<Long> targetIds) {
        Map<Long, Boolean> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(targetIds)) {
            for (Long targetId : targetIds) {
                map.put(targetId, isFriend(userId, targetId));
            }
        }
        return map;
    }

    @Override
    public Map<Long, Boolean> isFollowers(Long userId, Set<Long> targetIds) {
        if (CollectionUtil.isEmpty(targetIds)) {
            return Collections.EMPTY_MAP;
        }

        // 先把关系都搂出来
        List<FollowRelationDO> list = followRelationDAO.selectAllByUserId(userId, new ArrayList<>(targetIds));
        Map<Long, Boolean> map = new HashMap<>();
        for (FollowRelationDO followRelation : list) {
            if (followRelation.getUserId().equals(userId)) {
                if (followRelation.getRelationDirection() > -1) {
                    map.put(followRelation.getTargetId(), true);
                }
            } else {
                if (followRelation.getRelationDirection() < 1) {
                    map.put(followRelation.getUserId(), true);
                }
            }
        }

        for (Long targetId : targetIds) {
            if (!map.containsKey(targetId)) {
                map.put(targetId, false);
            }
        }

        return map;
    }

    /**
     * 拒绝好友申请
     *
     * @param reqId  好友申请请求的id
     * @param optMsg 处理意见
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean denyFriendReq(Long reqId, String optMsg) {
        FriendReqDO reqDO = friendReqDAO.selectById(reqId);
        if (reqDO != null) {
            long curTime = System.currentTimeMillis();
            FriendReqDO update = new FriendReqDO();
            update.setReqId(reqId);
            update.setStatus(FriendReqStatusEnum.DENY.getCode());
            update.setOptMsg(optMsg);
            update.setUpdateTime(curTime);
            friendReqDAO.updateById(update);

            // 拒绝消息发送给发起人
            FriendAddDenyMsg denyMsg = new FriendAddDenyMsg();
            denyMsg.setReqId(reqId);
            denyMsg.setUserId(reqDO.getUserId());
            denyMsg.setTargetId(reqDO.getTargetId());
            denyMsg.setTargetName(userDAO.selectNameById(reqDO.getTargetId()));
            denyMsg.setContent(String.format("%s拒绝了你的好友请求", denyMsg.getTargetName()));
            imService.sendCustomImMsg(null, reqDO.getUserId(), denyMsg, null);

            return true;
        }
        log.info("未找到该条好友申请记录");
        return false;
    }

    /**
     * 删除好友
     *
     * @param userId   用户id
     * @param targetId 目标用户id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delFriend(Long userId, Long targetId) {

        long current = System.currentTimeMillis();
        // 删除关系
        FriendRelationDO self = friendRelationDAO.selectByUserIdAndTargetId(userId, targetId, CommonStatus.Normal);
        if (self != null) {
            self.setUpdateTime(current);
            self.setStatus(CommonStatus.Deleted);
            friendRelationDAO.updateById(self);
        }

        FriendRelationDO friend = friendRelationDAO.selectByUserIdAndTargetId(targetId, userId, CommonStatus.Normal);
        if (friend != null) {
            friend.setUpdateTime(current);
            friend.setStatus(CommonStatus.Deleted);
            friendRelationDAO.updateById(friend);
        }

        return true;
    }

    /**
     * 获取关注列表
     *
     * @param currentUserId 当前用户id
     * @param userId        用户id
     * @param pageIndex     当前页码
     * @param pageSize      每页大小
     * @return 关注列表
     */
    @Override
    public PageList<FollowUser> getFollowList(Long currentUserId, Long userId, int pageIndex, int pageSize) {
        PageList<FollowUser> page = new PageList<>(pageIndex, pageSize);

        List<FollowUser> followUsers = followRelationDAO.selectFollowsUser(userId, page.getPageCursor(), pageSize);

        if (CollectionUtils.isEmpty(followUsers)) {
            page.setList(new ArrayList<>());
            page.setTotal(0);
            return page;
        }

        List<Long> userIds = followUsers.stream().map(FollowUser::getUserId).collect(Collectors.toList());

        List<RelationDTO> relationList = new ArrayList<>();
        for (FollowUser item : followUsers) {
            RelationDTO relation = new RelationDTO();
            relation.setUserId(item.getUserId());
            relation.setRelationDirection(item.getRelationDirection());
            relationList.add(relation);
        }

        List<RelationDTO> currentRelations = new ArrayList<>();
        // 如果查看别人，当前用户与列表元素的关系
        if (!currentUserId.equals(userId)) {
            currentRelations = followRelationDAO.selectFollows(currentUserId);
        }

        // 好友关系
        List<Long> friendUserIds = getFriendUserIds(userId);

        for (FollowUser item : followUsers) {
            item.setUserSex(Optional.ofNullable(item.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
            item.setUserAge(Optional.ofNullable(item.getUserAge()).orElse(18));
            // 是否互相关注 userId 已经关注 item.getUserId() ， 现需判断 item.getUserId() 是否关注 userId
            item.setMutualFollower(relationList.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() < FollowRelationDirectionEnum.ATOB.getValue()).count() > 0);
            // userId = 当前用户
            if (currentUserId.equals(userId)) {
                item.setCurrentUserFollower(true);
                item.setCurrentMutualFollower(item.isMutualFollower());
            } else {
                item.setCurrentUserFollower(currentRelations.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() > FollowRelationDirectionEnum.BTOA.getValue()).count() > 0);
                item.setCurrentMutualFollower(item.isCurrentUserFollower() && currentRelations.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() < FollowRelationDirectionEnum.ATOB.getValue()).count() > 0);
            }
            item.setFriend(friendUserIds.contains(item.getUserId()));

            // 显示用户直播状态与所在房间
            // 关注列表 操作按钮有直播/派对 聊天
            RtcRoomListItem userStayRoom = rtcRoomService.getUserStayRoom(item.getUserId());
            if (userStayRoom == null) {
                item.setUserState(FollowUserStateEnum.CHAT.getValue());
            } else {
                item.setUserState(RtcBusinessTypeEnum.RTC_SINGLE.equals(RtcBusinessTypeEnum.toEnum(userStayRoom.getBusinessType())) ? FollowUserStateEnum.LIVE.getValue() : FollowUserStateEnum.PARTY.getValue());
                item.setRoomId(userStayRoom.getRoomId());
            }
        }
        page.setList(followUsers);
        page.setTotal(userDAO.countByIds(userIds));
        return page;
    }

    /**
     * 获取关注,
     *
     * @param userId
     * @param userIds
     * @return
     */
    @Override
    public Set<Long> getFollowSet(Long userId, Set<Long> userIds) {
        Set<Long> followUserIds = followRelationDAO.selectFollowsUserIds(userId);
        // 取交集
        userIds.retainAll(followUserIds);
        return userIds;
    }

    @Override
    public Set<Long> selectFollowsUserIds(Long userId) {
        return followRelationDAO.selectFollowsUserIds(userId);
    }

    // 设置关注用户区域
    private void setFollowArea(Map<Integer, DataAreaDO> areaIdMap, DataAreaDO defaultArea, UserDO item, FollowUser user) {
        Integer areaId = item.getAreaId();
        if (areaId != null) {
            DataAreaDO dataAreaDO = areaIdMap.get(areaId);
            if (dataAreaDO != null) {
                user.setAreaFullName(dataAreaDO.getAreaFullName());
                user.setAreaName(dataAreaDO.getAreaShortName());
                user.setAreaPath(dataAreaDO.getAreaPath());
            }
        } else {
            user.setAreaFullName(defaultArea.getAreaFullName());
            user.setAreaName(defaultArea.getAreaShortName());
            user.setAreaPath(defaultArea.getAreaPath());
        }
    }

    /**
     * 获取粉丝列表
     *
     * @param currentUserId 当前用户id
     * @param userId        用户id
     * @param pageIndex     当前页码
     * @param pageSize      每页大小
     * @return 粉丝列表
     */
    @Override
    public PageList<FansUser> getFansList(Long currentUserId, Long userId, int pageIndex, int pageSize) {
        PageList<FansUser> page = new PageList<>(pageIndex, pageSize);

        List<FansUser> fansUsers = followRelationDAO.selectFansUser(userId, page.getPageCursor(), pageSize);

        if (CollectionUtils.isEmpty(fansUsers)) {
            page.setList(new ArrayList<>());
            page.setTotal(0);
            return page;
        }

        List<Long> userIds = fansUsers.stream().map(FansUser::getUserId).collect(Collectors.toList());

        List<RelationDTO> relationList = new ArrayList<>();
        for (FansUser item : fansUsers) {
            RelationDTO relation = new RelationDTO();
            relation.setUserId(item.getUserId());
            relation.setRelationDirection(item.getRelationDirection());
            relationList.add(relation);
        }

        List<RelationDTO> currentRelations = new ArrayList<>();
        // 如果查看别人，当前用户与列表元素的关系
        if (!currentUserId.equals(userId)) {
            currentRelations = followRelationDAO.selectFollows(currentUserId);
        }

        // 好友关系
        List<Long> friendUserIds = getFriendUserIds(userId);

        for (FansUser item : fansUsers) {
            // 是否互相关注 , item.getUserId() 已经关注 userId ,现需判断 userId 是否关注 item.getUserId()
            item.setMutualFollower(relationList.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() < FollowRelationDirectionEnum.ATOB.getValue()).count() > 0);
            // userId = 当前用户
            if (currentUserId.equals(userId)) {
                item.setCurrentUserFollower(item.isMutualFollower());
                item.setCurrentMutualFollower(item.isMutualFollower());
            } else {
                if (CollectionUtil.isEmpty(currentRelations)) {
                    item.setCurrentUserFollower(false);
                    item.setCurrentMutualFollower(false);
                } else {
                    item.setCurrentUserFollower(currentRelations.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() < FollowRelationDirectionEnum.ATOB.getValue()).count() > 0);
                    item.setCurrentMutualFollower(item.isCurrentUserFollower() && currentRelations.stream().filter(e -> e.getUserId().equals(item.getUserId()) && e.getRelationDirection() > FollowRelationDirectionEnum.BTOA.getValue()).count() > 0);
                }
            }
            //是否好友
            item.setFriend(friendUserIds.contains(item.getUserId()));


            // 显示用户直播状态与所在房间
            // 粉丝列表 操作按钮有关注 聊天
            item.setUserState(item.isMutualFollower() ? FollowUserStateEnum.CHAT.getValue() : FollowUserStateEnum.FOLLOW.getValue());
        }
        page.setList(fansUsers);
        page.setTotal(userDAO.countByIds(userIds));
        return page;
    }

    @Override
    public List<Long> getFansUserIds(Long userId) {
        return followRelationDAO.selectFansUserIds(userId);
    }

    // 设置粉丝用户区域
    private void setFansArea(Map<Integer, DataAreaDO> areaIdMap, DataAreaDO defaultArea, FansUser user) {
        Integer areaId = user.getAreaId();
        if (areaId != null) {
            DataAreaDO dataAreaDO = areaIdMap.get(areaId);
            if (dataAreaDO != null) {
                user.setAreaFullName(dataAreaDO.getAreaFullName());
                user.setAreaName(dataAreaDO.getAreaShortName());
                user.setAreaPath(dataAreaDO.getAreaPath());
            }
        } else {
            user.setAreaFullName(defaultArea.getAreaFullName());
            user.setAreaName(defaultArea.getAreaShortName());
            user.setAreaPath(defaultArea.getAreaPath());
        }
    }

    /**
     * 获取好友列表
     *
     * @param userId   用户id
     * @param pageNum  当前页码
     * @param pageSize 每页大小
     * @return 好友列表
     */
    @Override
    public PageList<FriendUser> getFriendList(Long userId, int pageNum, int pageSize) {
        PageList<FriendUser> page = new PageList<>(pageNum, pageSize);

        List<Long> userIds = getFriendUserIds(userId);

        if (CollectionUtils.isEmpty(userIds)) {
            page.setList(new ArrayList<>());
            page.setTotal(0);
            return page;
        }
        List<UserDO> userDOList = userDAO.findByIds(userIds, page.getPageCursor(), pageSize);
        List<FriendUser> friendUsers = new ArrayList<>();
        for (UserDO item : userDOList) {
            FriendUser user = new FriendUser();
            user.setUserId(item.getUserId());
            user.setUserName(item.getUserName());
            user.setUserAvatar(item.getUserAvatar());
            user.setUserSex(item.getUserSex());
            user.setUserAge(item.getUserAge());
            user.setIsOnline(item.getIsOnline());
            friendUsers.add(user);
        }
        page.setList(friendUsers);
        page.setTotal(userDAO.countByIds(userIds));
        return page;
    }

    /**
     * 获取好友申请列表
     *
     * @param curUserId 用户id
     * @param pageNum   当前页码
     * @param pageSize  每页大小
     * @return 好友申请列表
     */
    @Override
    public PageList<FriendReqVO> getFriendReqList(Long curUserId, int pageNum, int pageSize) {
        PageList<FriendReqVO> page = new PageList<>(pageNum, pageSize);
        page.setList(friendReqDAO.selectFriendReqList(curUserId, page.getPageCursor(), pageSize));
        page.setTotal(friendReqDAO.countFriendReqList(curUserId));
        return page;
    }


    /**
     * 获取粉丝数量
     *
     * @param userId 用户id
     * @return 粉丝数量
     */
    @Override
    public int getFansCount(Long userId) {

        if (releaseConfigService.getReleaseConfig("enable_show_new").equals("true")) {
            QueryWrapper qw = new QueryWrapper();
            qw.eq("userId", userId);
            UserRelationStatsDO ur = userRelationStatsDAO.selectOne(qw);
            if (ur != null && ur.getFansCount() > 0) {
                return ur.getFansCount().intValue();
            } else {
                return 0;
            }
        } else {
            return followRelationDAO.countFans(userId);
        }


    }

    /**
     * 获取粉丝数量
     *
     * @param userIds 用户id
     * @return 粉丝数量
     */
    @Override
    public Map<Long, Integer> getFansCounts(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Integer> resultMap = new HashMap<>();
        for (Long userId : userIds) {
//            resultMap.put(userId, followRelationDAO.countFans(userId));
            resultMap.put(userId, getFansCount(userId));
        }
        return resultMap;
    }

    /**
     * 获取关注数量
     *
     * @param userId 用户id
     * @return 关注数量
     */
    @Override
    public int getFollowCount(Long userId) {
        if (releaseConfigService.getReleaseConfig("enable_show_new").equals("true")) {
            QueryWrapper qw = new QueryWrapper();
            qw.eq("userId", userId);
            UserRelationStatsDO ur = userRelationStatsDAO.selectOne(qw);
            if (ur != null) {
                return ur.getFollowCount().intValue();
            } else {
                return 0;
            }
        } else {
            return followRelationDAO.countFollow(userId);
        }

    }

    /**
     * 获取关注数量
     *
     * @param userIds 用户id
     * @return 关注数量
     */
    @Override
    public Map<Long, Integer> getFollowCountMap(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Integer> resultMap = new HashMap<>();
        for (Long userId : userIds) {
//            resultMap.put(userId, followRelationDAO.countFollow(userId));
            resultMap.put(userId, getFollowCount(userId));
        }
        return resultMap;
    }

    /**
     * 好友申请是否已发出
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public boolean isFriendReqExist(Long userId, Long targetId) {
        return friendReqDAO.countByUserId(userId, targetId, FriendReqStatusEnum.WAIT.getCode()) > 0;
    }

    @Override
    public List<SingleChatUserVO> getSingleChatFollowList(Long userId, Long channelId) {

        List<Long> anchorUserIds = getFollowUserIds(userId);

        if (CollectionUtils.isEmpty(anchorUserIds)) {
            return new ArrayList<>();
        }
        List<SingleChatUserVO> list = anchorDAO.selectSingleChatUserByIds(anchorUserIds);
        if (CollectionUtil.isNotEmpty(list)) {
            for (SingleChatUserVO singleChatUserVO : list) {
                if (StringUtils.isBlank(singleChatUserVO.getAreaCity())) {
                    singleChatUserVO.setAreaCity("北京市");
                }
                if (BoolIntVal.False == singleChatUserVO.getIsOnline() || LiveStatusEnum.Living == LiveStatusEnum.toEnum(singleChatUserVO.getLiveStatus())) {
                    singleChatUserVO.setSingleChatStatus(SingleChatStatus.BUSY);
                }
                singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
            }
        }
        return list;
    }

    @Override
    public List<Long> getFollowUserIds(Long userId) {
        List<RelationDTO> dtoList = followRelationDAO.selectFollows(userId);
        if (CollectionUtil.isEmpty(dtoList)) {
            return new ArrayList<>();
        }
        return dtoList.stream().map(RelationDTO::getUserId).collect(Collectors.toList());
    }

    @Override
    public int getInviteCount(Long userId) {
        return userInviteDAO.countByUserId(userId, null);
    }

    /**
     * 添加黑名单
     *
     * @param curUserId
     * @param userId
     * @return
     */
    @Override
    public boolean addBlack(Long curUserId, Long userId) {
        return imService.addBlack(curUserId, userId);
    }

    /**
     * 删除黑名单
     *
     * @param curUserId
     * @param userId
     * @return
     */
    @Override
    public boolean delBlack(Long curUserId, Long userId) {
        return imService.delBlack(curUserId, userId);
    }

    /**
     * 是否关注限制
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isFollowLimit(Long userId) {
        int level = Optional.ofNullable(userDAO.selectWealthLevel(userId)).orElse(1);
        return userFollowLimitProvider.limited(userId, level);
    }


    /**
     * 新版构建关注用户
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void followUserNew(Long userId, Long targetId) {
        Long now = System.currentTimeMillis();
        List<UserFollowRelationDO> userFansRelation = userFollowRelationDAO.getUserRelation(userId, targetId);
        if (CollectionUtil.isNotEmpty(userFansRelation)) {
            // 这里需要进行判断, 是否是朋友关系, 之前是否已经插入过了
            if (userFansRelation.size() > 1) {
                return;
            } else {
                UserFollowRelationDO relationDO = userFansRelation.get(0);
                if (relationDO.getFollowType() == UserFollowRelationEnum.FOLLOW.getCode()) {
                    // 说明之前存在关注数据, 是朋友关系
                    List<UserFollowRelationDO> relations = doBuildFollowRelationItem(userId, targetId, 1, now, now);
                    userFollowRelationDAO.insertBatch(relations);
                    // 顺便更新之前的朋友关系
                    userFollowRelationDAO.updateIsFriend(userId, targetId, 1, now);
                }
            }
        } else {
            // 说明是空的关系, 直接插入就行了
            List<UserFollowRelationDO> relations = doBuildFollowRelationItem(userId, targetId, 0, now, now);
            userFollowRelationDAO.insertBatch(relations);
        }
    }


    /**
     * 取消关注用户
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void unFollowNew(Long userId, Long targetId) {
        Long now = System.currentTimeMillis();
        userFollowRelationDAO.updateIsFriend(userId, targetId, 0, now);
        userFollowRelationDAO.delRelation(userId, targetId, UserFollowRelationEnum.FANS.getCode(), now);
        userFollowRelationDAO.delRelation(targetId, userId, UserFollowRelationEnum.FOLLOW.getCode(), now);
    }

    /**
     * 删除好友
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void delFriendNew(Long userId, Long targetId) {
        userFollowRelationDAO.delFriend(userId, targetId, System.currentTimeMillis());
    }

    /**
     * 构建历史关注
     */
    @Override
    @Async
    public void buildHistorySocialRelation() {
        // 获取最新的一条数据, 以此为基准时间
        Long begin = System.currentTimeMillis();
        UserFollowRelationDO followRelationDO = userFollowRelationDAO.getFirstUserFollow();
        Long endTime = System.currentTimeMillis();
        if (followRelationDO != null) {
            endTime = followRelationDO.getCreateTime();
        }
        int page = 1;
        int pageSize = 500;
        while (true) {
            List<FollowRelationDO> followHisList = followRelationDAO.getFollowHis(endTime, (page - 1) * pageSize, pageSize);
            if (CollectionUtil.isNotEmpty(followHisList)) {
                // 这里构建插入数据
                List<UserFollowRelationDO> insertList = new ArrayList<>();
                followHisList.forEach(i -> {
                    List<UserFollowRelationDO> insertItem = buildFollowRelationData(i);
                    insertList.addAll(insertItem);
                });
                userFollowRelationDAO.insertBatch(insertList);
                page++;
            } else {
                break;
            }
        }
        Long end = System.currentTimeMillis();
        log.info("构建历史关注数据完成,共耗时 :" + (end - begin) + "毫秒");
    }

    /**
     * 构建需要插入的数据
     *
     * @return
     */
    private List<UserFollowRelationDO> buildFollowRelationData(FollowRelationDO item) {
        List<UserFollowRelationDO> res = new ArrayList<>();
        if (item.getRelationDirection() == FollowRelationDirectionEnum.ATOB.getValue()) {
            List<UserFollowRelationDO> fans = doBuildFollowRelationItem(item.getUserId(), item.getTargetId(), 0, item.getFollowTime(), item.getUpdateTime());
            res.addAll(fans);
        } else if (item.getRelationDirection() == FollowRelationDirectionEnum.BTOA.getValue()) {
            List<UserFollowRelationDO> fans = doBuildFollowRelationItem(item.getTargetId(), item.getUserId(), 0, item.getFollowTime(), item.getUpdateTime());
            res.addAll(fans);
        } else {
            // 互相关注就是互为粉丝
            List<UserFollowRelationDO> fans = doBuildFollowRelationItem(item.getTargetId(), item.getUserId(), 1, item.getFollowTime(), item.getUpdateTime());
            res.addAll(fans);
            List<UserFollowRelationDO> follow = doBuildFollowRelationItem(item.getTargetId(), item.getUserId(), 1, item.getFollowTime(), item.getUpdateTime());
            res.addAll(follow);
        }
        return res;
    }

    /***
     * 好友关系增加
     * @param userId 关注人
     * @param targetUserId 被关注人
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUserRelationInc(long userId, long targetUserId) {
        //粉丝
//        QueryWrapper q = new QueryWrapper();
//        q.eq("userId", targetUserId);
//        int ufan_count = userRelationStatsDAO.selectCount(q);
//        if (ufan_count > 0) {
//            userRelationStatsDAO.updateRelationInc(targetUserId, 1L, null);
//        } else {
//            UserRelationStatsDO ufans = new UserRelationStatsDO();
//            ufans.setUserId(targetUserId);
//            ufans.setFansCount(1L);
//            userRelationStatsDAO.insert(ufans);
//        }
//
//        //关注
//        QueryWrapper q1 = new QueryWrapper();
//        q1.eq("userId", userId);
//        int uflow_count = userRelationStatsDAO.selectCount(q1);
//        if (uflow_count > 0) {
//            userRelationStatsDAO.updateRelationInc(userId, null, 1L);
//        } else {
//            UserRelationStatsDO uflm = new UserRelationStatsDO();
//            uflm.setUserId(userId);
//            uflm.setFollowCount(1L);
//            userRelationStatsDAO.insert(uflm);
//        }

        boolean b_ufans = userRelationStatsDAO.updateRelationInc(targetUserId, 1L, null) <= 0;
        if (b_ufans) {
            UserRelationStatsDO ufans = new UserRelationStatsDO();
            ufans.setUserId(targetUserId);
            ufans.setFansCount(1L);
            userRelationStatsDAO.insert(ufans);
        }

        //关注
        boolean b_uflow = userRelationStatsDAO.updateRelationInc(userId, null, 1L) <= 0;
        if (b_uflow) {
            UserRelationStatsDO uflm = new UserRelationStatsDO();
            uflm.setUserId(userId);
            uflm.setFollowCount(1L);
            userRelationStatsDAO.insert(uflm);
        }


    }

    /**
     * 好友关系减少
     *
     * @param userId       取消关注人
     * @param targetUserId 被取消关注人
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUserRelationDec(long userId, long targetUserId) {
        //粉丝
        QueryWrapper q = new QueryWrapper();
        q.eq("userId", targetUserId);
        UserRelationStatsDO ut = userRelationStatsDAO.selectOne(q);
        if (ut != null && ut.getFansCount() > 0) {
            userRelationStatsDAO.updateRelationDec(targetUserId, 1L, null);
        }

        //关注
        QueryWrapper q1 = new QueryWrapper();
        q1.eq("userId", userId);
        UserRelationStatsDO us = userRelationStatsDAO.selectOne(q1);
        if (us != null && us.getFollowCount() > 0) {
            userRelationStatsDAO.updateRelationDec(userId, null, 1L);
        }

    }


    /**
     * 这里构建的是粉丝的关系
     *
     * @param userId
     * @param targetId
     * @param isFriend
     * @param createTime
     * @param updateTime
     * @return
     */
    private List<UserFollowRelationDO> doBuildFollowRelationItem(Long userId, Long targetId, Integer isFriend, Long createTime, Long updateTime) {
        if (updateTime == null) {
            updateTime = createTime;
        }
        List<UserFollowRelationDO> res = new ArrayList<>();
        UserFollowRelationDO fans = new UserFollowRelationDO(getIdsByPool(), userId, targetId, UserFollowRelationEnum.FANS.getCode(), isFriend, 0, createTime, updateTime);
        UserFollowRelationDO follow = new UserFollowRelationDO(getIdsByPool(), targetId, userId, UserFollowRelationEnum.FOLLOW.getCode(), isFriend, 0, createTime, updateTime);
        res.add(fans);
        res.add(follow);
        return res;
    }

    private static Stack<Long> idPool = new Stack<>();

    private static final int poolSize = 20;

    private Long getIdsByPool() {
        if (idPool.isEmpty()) {
            Long[] ids = identityService.getIds(poolSize);
            idPool.addAll(Arrays.asList(ids));
        }
        return idPool.pop();
    }


}
