package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.pojo.user.GiftWallItem;
import com.starbuds.server.common.pojo.user.UserGiftWallItem;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserGiftWallService;
import com.starbuds.server.common.service.user.UserGiftWallStat;
import com.starbuds.server.service.user.dao.UserGiftWallDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static com.starbuds.server.common.utils.DateUtil.getWeekEnd;
import static com.starbuds.server.common.utils.DateUtil.getWeekStart;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/8/4
 */
@Service(version = "1.0.0")
public class UserGiftWallServiceImpl implements UserGiftWallService {

    @Resource
    UserGiftWallDAO userGiftWallDAO;

    @Reference(version = "1.0.0")
    GiftService giftService;

    /**
     * 获取礼物墙列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<UserGiftWallItem> getGiftWallList(Long userId) {

        // 查询已获取礼物列表
        List<UserGiftWallItem> list = userGiftWallDAO.selectWallGifts(userId, null, null);

        List<Long> giftIds = null;
        if (CollectionUtil.isNotEmpty(list)) {
            // 处理礼物 id
            giftIds = list.stream().map(UserGiftWallItem::getGiftId).collect(Collectors.toList());

            // 价值倒序
            list.sort(Comparator.comparingLong(UserGiftWallItem::getGiftPrice).reversed());
        }


        // 查询未获取礼物列表
        List<UserGiftWallItem> otherGifts = giftService.getWallItemsByIds(giftIds);
        if (CollectionUtil.isNotEmpty(otherGifts)) {
            otherGifts.sort(Comparator.comparingLong(UserGiftWallItem::getGiftPrice).reversed());
            list.addAll(otherGifts);
        }

        return list;
    }

    /**
     * 获取礼物墙列表
     *
     * @param userId
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<UserGiftWallItem> getGiftWallList(Long userId, Long startTime, Long endTime) {
        return userGiftWallDAO.selectWallGifts(userId, startTime, endTime);
    }

    /**
     * 获取礼物墙统计
     *
     * @param userId
     * @return
     */
    @Override
    public UserGiftWallStat getGiftWallStatistics(Long userId) {
        UserGiftWallStat userGiftWallStat = new UserGiftWallStat();
        userGiftWallStat.setTotalNum(giftService.getWallItemsCount());
        userGiftWallStat.setTotalGotNum(userGiftWallDAO.countNum(userId, null, null));
        userGiftWallStat.setWeekGotNum(userGiftWallDAO.countNum(userId, getWeekStart(), getWeekEnd()));
        return userGiftWallStat;
    }

    /**
     * 获取礼物
     *
     * @param userId
     * @param startDate
     * @param endDate
     * @param giftIds
     * @return
     */
    @Override
    public List<GiftWallItem> getWallItems(Long userId, Long startDate, Long endDate, List<Long> giftIds) {
        return userGiftWallDAO.selectWallItems(userId, startDate, endDate, giftIds);
    }
}
