package com.starbuds.server.service.user.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.data.*;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.rtc.RtcRoomSummary;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.common.service.user.UserMedalService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.user.config.ReleaseConfig;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.*;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;
import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 用户服务实现
 *
 * @author Lay
 * @date 2019/11/27
 */
@Service(version = "1.0.0")
@Slf4j
public class UserServiceImpl implements UserService {

    @Resource
    ReleaseConfig releaseConfig;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    FileService fileService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    SecurityService securityService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    ModuleService moduleService;

    @Resource
    UserDAO userDAO;

    @Resource
    AnchorAuditDAO anchorAuditDAO;

    @Resource
    AnchorDAO anchorDAO;

    @Resource
    AgentDAO agentDAO;

    @Resource
    UserSecurityDAO userSecurityDAO;

    @Resource
    UserNumberPoolDAO userNumberPoolDAO;

    @Resource
    UserInviteDAO userInviteDAO;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Resource
    LiveUserProvider liveUserProvider;

    @Resource
    ChatUserProvider chatUserProvider;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    UserSwitchDAO userSwitchDAO;

    @Resource
    VisitProvider visitProvider;

    @Resource
    UserOauthDAO userOauthDAO;

    @Resource
    UserLoginHisDAO userLoginHisDAO;

    @Resource
    UserOnlineHisDAO userOnlineHisDAO;

    @Resource
    UserAuthDAO userAuthDAO;

    @Resource
    UserIdCardDAO userIdCardDAO;

    @Resource
    ScoreProvider scoreProvider;

    @Resource
    ChineseNameProvider nameProvider;

    @Resource
    AdminDAO adminDAO;

    @Resource
    QuestionAnswerDAO questionAnswerDAO;

    @Resource
    UserNobleDAO userNobleDAO;

    @Resource
    UserForbiddenDAO userForbiddenDAO;

    @Resource
    UserFirstRechargeRemindDAO userFirstRechargeRemindDAO;

    @Value("${avatar.male}")
    String avatarMale;

    @Value("${avatar.female}")
    String avatarFemale;

    @Reference(version = "1.0.0")
    RiskService riskService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Resource
    private ChineseNameProvider chineseNameProvider;

    @Resource
    private UserMedalService userMedalService;

    @Resource
    private UserVipDAO userVipDAO;

    @Resource
    private NiceNoPoolDAO niceNoPoolDAO;

    /**
     * 最大尝试次数
     */
    private static final int RANDOM_ID_MAX_TIMES = 100;

    /**
     * 创建用户
     *
     * @param userAddDTO
     * @return 用户 id
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long addUser(UserAddDTO userAddDTO) {
        Long userId = identityService.getId();
        try {
            long curTime = System.currentTimeMillis();

            //数据库开启注册风控并且请求带了风控设备ID
            if (riskService.getConfig("regist") && StringUtils.isNotEmpty(userAddDTO.getRiskDeviceId())) {
                //注册风控
                RiskParms p = new RiskParms();
                if (StringUtils.isNotEmpty(userAddDTO.getUserPhone())) {
                    p.setPhone(userAddDTO.getUserPhone());
                    p.setCountryCode("0086");
                }
                p.setIp(userAddDTO.getClientIp());
                p.setUserID(userId);
                p.setTimestamp(System.currentTimeMillis());
                p.setDeviceId(userAddDTO.getRiskDeviceId());
                p.setOs(userAddDTO.getClientType().getName().toLowerCase());
                p.setAppVersion(userAddDTO.getAppVersion());
                p.setLat(userAddDTO.getUserLatitude());
                p.setLng(userAddDTO.getUserLongitude());
                GreenAuditTrace g = riskService.registAudit(p);

                //审核不通过
                if (g.getRobotAuditStatus() != GreenAuditStatusEnum.PASSED_ROBOT) {
//                    //接口出问题
                    if (g.getRobotAuditStatus() != GreenAuditStatusEnum.BLOCK_EXCEPTION) {
//                        //接口解析出问题
                        if (g.getRobotAuditStatus() != GreenAuditStatusEnum.BLOCK_TIMEOUT) {
                            //开关控制是否采用拒绝结果
                            if (riskService.getConfig("useRegistRelust")) {
                                log.error("risk-result" + g.getBizId());
                                return -1L;
                            }
                        }
                    }
                }
            }

            // 默认渠道处理
            if (userAddDTO.getChannelId() == null) {
                userAddDTO.setChannelId(0L);
            }

            String userNickname = userAddDTO.getNickName();
            if (StringUtils.isBlank(userNickname)) {
                userNickname = nameProvider.get();
                if (isNameExist(userNickname, userId)) {
                    Random random = new Random();
                    userNickname = String.format("%s%d", userNickname, random.nextInt(599) + 1);
                }
            }

            // 创建用户
            UserDO userDO = new UserDO();
            BeanUtils.copyProperties(userAddDTO, userDO);
            userDO.setUserId(userId);
            userDO.setUserStatus(CommonStatus.Normal);
            userDO.setUserCreateTime(curTime);
            userDO.setUserUpdateTime(curTime);
            userDO.setUserName(userNickname);
            userDO.setUserSex(userAddDTO.getSex() == null ? SexEnum.MALE.getCode() : userAddDTO.getSex());
            userDO.setIsAnchor(BoolIntVal.False);
            userDO.setIsOnline(BoolIntVal.False);
            userDO.setChannelId(userAddDTO.getChannelId());
            userDO.setUserPhone(userAddDTO.getUserPhone());
            userDO.setCharmLevel(1);
            userDO.setWealthLevel(1);
            userDO.setStarLevel(1);
            userDO.setRenameTimes(0);
            userDO.setHasRecharged(BoolIntVal.False);
            userDO.setCountryId(userAddDTO.getCountryId());
            userDO.setAreaId(userAddDTO.getAreaId());
            if (StringUtils.isBlank(userAddDTO.getUserAvatar())) {
                // 默认男头像
                userDO.setUserAvatar(avatarMale);
            }

            // 用户号 从号池中随机选择一个
            int times = 0;
            while (times < RANDOM_ID_MAX_TIMES) {
                Long userNo = userNumberPoolDAO.selectRandomId();
                if (userNo != null) {
                    // 占用
                    int updated = userNumberPoolDAO.updateId(userNo, userId);
                    if (updated > 0) {
                        userDO.setUserNo(userNo);
                        break;
                    }
                } else {
                    // 号池耗尽
                    throw new Exception("号池耗尽");
                }

                times++;
            }
            userDAO.insert(userDO);

            // 安全信息
            UserSecurityDO userSecurityDO = new UserSecurityDO();
            userSecurityDO.setUserId(userId);
            userSecurityDO.setUserPhone(userAddDTO.getUserPhone());
            userSecurityDO.setUserLocked(BoolIntVal.False);
            userSecurityDO.setChannelId(userAddDTO.getChannelId());
            userSecurityDO.setUserIsRobot(userAddDTO.getUserIsRobot());
            userSecurityDO.setUserIsMarketAuditor(userAddDTO.getUserIsMarketAuditor());
            userSecurityDO.setAreaCode(userAddDTO.getAreaCode());

            // 邀请关系写入
            if (userAddDTO.getInviterUserId() != null) {
                UserInviteDO userInviteDO = new UserInviteDO();
                userInviteDO.setUserId(userId);
                userInviteDO.setInviterUserId(userAddDTO.getInviterUserId());
                userInviteDO.setInviteTime(curTime);
                userInviteDAO.insert(userInviteDO);
            }

            userSecurityDAO.insert(userSecurityDO);

            // 丢出 MQ 消息
            QueueUserCreateMsg queueUserCreateMsg = new QueueUserCreateMsg();
            queueUserCreateMsg.setUserId(userId);
            queueUserCreateMsg.setChannelId(userAddDTO.getChannelId());
            queueUserCreateMsg.setDeviceId(userAddDTO.getDeviceId());
            queueUserCreateMsg.setClientType(userAddDTO.getClientType().getValue());
            queueUserCreateMsg.setRegisterType(userAddDTO.getRegisterType());
            queueUserCreateMsg.setSex(userAddDTO.getSex());
            queueUserCreateMsg.setIdfa(userAddDTO.getIdfa());
            queueUserCreateMsg.setImeiMd5(userAddDTO.getImeiMd5());
            queueUserCreateMsg.setClientIp(userAddDTO.getClientIp());
            queueUserCreateMsg.setHeadImgUrl(userAddDTO.getHeadImgUrl());
            queueUserCreateMsg.setInviterUserId(userAddDTO.getInviterUserId());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.USER_CREATE, queueUserCreateMsg.serialize());
            //广告引流用户处理消息
            adCreateUser(queueUserCreateMsg);

            return userId;
        } catch (Exception e) {
            log.error("用户创建失败", e);
            userSearchService.deleteById(userId);
            throw new RuntimeException("用户创建失败");
        }
    }

    /**
     * 广告引流用户处理消息
     */
    private void adCreateUser(QueueUserCreateMsg queueUserCreateMsg) {
        String config = releaseConfigService.getReleaseConfig("is_ad_create_user_open");
        if (StringUtils.isNotEmpty(config) && config.equals("open")) {
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.USER_CREATE_TO_AD, queueUserCreateMsg.serialize());
        }
    }

    /**
     * 根据邀请码获取用户 id
     *
     * @param inviteCode
     * @return
     */
    @Override
    public Long getIdByInviteCode(String inviteCode) {
        try {
            // 转回十进制
            Long userNo = Long.parseLong(inviteCode, 16);

            // 查询用户 id
            return userDAO.selectIdByNo(userNo);
        } catch (Exception e) {
            log.error("邀请码进制转换失败", e);
        }

        return null;
    }

    /**
     * 修改个人资料
     *
     * @param userDO
     * @return
     */
    @Override
    public boolean editProfile(UserDO userDO) {
        try {
            // 判断是否首次修改
            Integer renameTimes = userDAO.selectRenameTimesById(userDO.getUserId());
            if (renameTimes > 0) {
                // 不是首次修改昵称
                userDO.setRenameTimes(++renameTimes);
            } else {
                userDO.setUserSex(Optional.ofNullable(userDO.getUserSex()).orElse(SexEnum.MALE.getCode()));
                // 首次的话直接更新次数
                userDO.setRenameTimes(1);
                // 没有头像的话按照性别来
                if (StringUtils.isBlank(userDO.getUserAvatar())) {
                    if (SexEnum.FEMALE.getCode() == userDO.getUserSex()) {
                        userDO.setUserAvatar(avatarFemale);
                    }
                }
            }

            // 更新资料
            userDAO.updateById(userDO);

            // 丢出 MQ 消息
            QueueUserEditMsg userEditMsg = new QueueUserEditMsg();
            userEditMsg.setUserDO(userDO);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.USER_EDIT, userEditMsg.serialize());

            return true;
        } catch (Exception e) {
            log.error("个人资料修改失败", e);
        }
        return false;
    }

    /**
     * 修改昵称
     *
     * @param userId
     * @param userName
     * @return
     */
    @Override
    public boolean setName(Long userId, String userName) {
        try {
            userDAO.updateName(userId, userName);
            // 同步 es
            userSearchService.setUserName(userId, userName);

            // 同步腾讯云
            ImUserProfile imUserProfile = new ImUserProfile();
            imUserProfile.setUserName(userName);
            imUserProfile.setUserId(userId);
            imService.syncUserProfile(imUserProfile);
            return true;
        } catch (Exception e) {
            log.error("昵称修改失败", e);
        }
        return false;
    }

    /**
     * 获取个人资料
     *
     * @param userId
     * @return
     */
    @Override
    public UserProfile getProfile(Long userId) {
        UserDO userDO = userDAO.selectById(userId);
        if (userDO != null) {
            UserProfile userProfile = new UserProfile();
            BeanUtils.copyProperties(userDO, userProfile);
            userProfile.setStarLevel(Optional.ofNullable(userProfile.getStarLevel()).orElse(1));
            // 认证状态
            userProfile.setApplyStatus(Optional.ofNullable(anchorAuditDAO.getLastApplyStatus(userId)).orElse(-1));

            // 区域 海外显示国家 国内显示具体城市
            if (userDO.getCountryId() != null && !userDO.getCountryId().equals(1)) {
                DataCountryDO countryDO = dataService.getCountry(userDO.getCountryId());
                userProfile.setCityInfo(DataAreaVO.builder().areaFullName(countryDO.getCountry()).areaCity(countryDO.getCountry()).build());
            } else {
                DataAreaDO area = areaService.getAreaById(Optional.ofNullable(userDO.getAreaId()).orElse(110100));
                if (area != null) {
                    userProfile.setCityInfo(DataAreaVO.builder().areaFullName(area.getAreaFullName()).areaPath(area.getAreaPath()).areaCity(area.getAreaCityShort()).build());
                }
            }

            // 年龄默认18  性别默认女
            userProfile.setUserSex(Optional.ofNullable(userProfile.getUserSex()).orElse(SexEnum.FEMALE.getCode()));
            userProfile.setUserAge(Optional.ofNullable(userProfile.getUserAge()).orElse(18));

            Long agentId = anchorDAO.getAgentId(userId);
            if (agentId != null) {
                AgentVO agentDetail = agentDAO.getAgentDetail(agentId);
                if (agentDetail != null) {
                    userProfile.setAgentNo(agentDetail.getAgentNo());
                    userProfile.setAgentName(agentDetail.getAgentName());
                    userProfile.setAgentLogo(agentDetail.getAgentLogo());
                }
                userProfile.setAgentId(agentId);
            }

            Long familyId = familyService.getFamilyId(userId);
            if (familyId != null) {
                FamilyDO familyDO = familyService.getFamily(familyId);
                if (familyDO != null) {
                    userProfile.setFamilyNo(familyDO.getFamilyNo());
                    userProfile.setFamilyName(familyDO.getFamilyName());
                    userProfile.setFamilyLogo(familyDO.getFamilyLogo());
                }
                userProfile.setFamilyId(familyId);
            }

            if (BoolIntVal.True == Optional.ofNullable(userProfile.getIsAnchor()).orElse(BoolIntVal.False)) {
                AnchorDO anchorDO = anchorDAO.selectById(userId);
                if (anchorDO != null) {
                    userProfile.setLiveStatus(LiveStatusEnum.toEnum(anchorDO.getLiveStatus()).getValue());
                    userProfile.setLiveCover(anchorDO.getLiveCover());
                    userProfile.setSingleChatEnable(anchorDO.getSingleChatEnable());
                }
            }

            // 区号
            userProfile.setAreaCode(Optional.ofNullable(userSecurityDAO.selectAreaCode(userId)).orElse("86"));

            // 默认中国
            Integer countryId = userDO.getCountryId();
            if (userDO.getCountryId() == null) {
                countryId = 1;
            }
            userProfile.setCountryId(countryId);

            DataCountryDO country = dataService.getCountry(countryId);
            if (country != null && StringUtils.isBlank(country.getCountry())) {
                PhoneAreaCodeDO pac = dataService.getPhoneAreaCodeById(countryId);
                if (pac != null && countryId.equals(pac.getCountryId())) {
                    country = new DataCountryDO();
                    BeanUtils.copyProperties(pac, country);
                }
            }
            String countryName = "中国大陆";
            String countryNameEn = "China mainland";
            if (country != null) {
                countryName = country.getCountry();
                countryNameEn = dataService.getCountryNameEn(country.getCountry());
            }
            userProfile.setCountryName(countryName);
            userProfile.setCountryNameEn(countryNameEn);
            userProfile.setIsAuth(userAuthDAO.countById(userId) > 0 ? BoolIntVal.True : BoolIntVal.False);
            userProfile.setIsBankcardBind(walletService.isBankcardBind(userId) ? BoolIntVal.True : BoolIntVal.False);
            userProfile.setIsPassSet(securityService.isUserPassSet(userId) ? BoolIntVal.True : BoolIntVal.False);
            userProfile.setIsPhoneBind(securityService.isPhoneBind(userId) ? BoolIntVal.True : BoolIntVal.False);
            userProfile.setIsOfficial(isOfficial(userId) ? BoolIntVal.True : BoolIntVal.False);

            MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(userId);
            userProfile.setMedalShinyStatus(medalShinyStatus);
            return userProfile;
        }
        return null;
    }

    /**
     * 获取个人资料
     *
     * @param userId
     * @return 用户资料
     */
    @Override
    public List<UserProfileGroup> getProfileNew(Long userId) {
        try {
            UserDO userDO = userDAO.selectById(userId);
            if (userDO != null) {
                Class<UserDO> clazz = UserDO.class;
                Field[] fields = clazz.getDeclaredFields();
                List<UserModuleVO> userModules = moduleService.getUserModules();
                // 主分类
                Map<String, UserModuleVO> masterModuleMap = userModules.stream().collect(Collectors.toMap(UserModuleVO::getModuleKey, Function.identity()));
                // 模块key和子分类
                Map<String, List<UserModuleVO>> childModuleMap = userModules.stream().filter(e -> StringUtils.isNotEmpty(e.getModuleKey())).collect(Collectors.toMap(UserModuleVO::getModuleKey, e -> Optional.ofNullable(e.getChildModules()).orElse(Collections.EMPTY_LIST), (value1, value2) -> value2));
                // 主分类
                List<UserProfileGroup> groups = new ArrayList<>();
                // 自问自答
                List<QuestionAnswerDTO> questionAnswers = getQuestionAnswers(userId);

                for (String moduleKey : masterModuleMap.keySet()) {
                    boolean isQA = false;
                    String qaModuleKey = "";
                    // 子条目
                    List<UserProfileGroup.Item> childItems = new ArrayList<>();
                    // 子分类集合
                    List<UserModuleVO> childModules = childModuleMap.get(moduleKey);
                    // 子分类key => 子分类
                    Map<String, UserModuleVO> childMap = childModules.stream().collect(Collectors.toMap(UserModuleVO::getModuleKey, Function.identity(), (value1, value2) -> value2));
                    // 遍历子分类key
                    for (String childKey : childMap.keySet()) {
                        // 子分类属性值
                        Object childValue = null;
                        for (Field field : fields) {
                            String fieldName = field.getName();
                            if (childKey.equals(fieldName)) {
                                String upperChar = fieldName.substring(0, 1).toUpperCase();
                                String anotherStr = fieldName.substring(1);
                                String methodName = "get" + upperChar + anotherStr;
                                Method method = clazz.getMethod(methodName, new Class[]{});
                                method.setAccessible(true);
                                childValue = method.invoke(userDO, new Object[]{});
                            }
                        }

                        // 内容类型
                        Integer contentType = childMap.get(childKey).getContentType();
                        if (contentType == ModuleContentTypeEnum.ADD_QA.getCode()) {
                            isQA = true;
                            qaModuleKey = childKey;
                            break;
                        } else {
                            UserProfileGroup.Item item = new UserProfileGroup.Item();
                            item.setItemKey(childKey);
                            item.setItemName(childMap.get(childKey).getModuleName());
                            item.setItemValue(Convert.toStr(childValue));
                            item.setItemIcon(childMap.get(childKey).getModuleIcon());
                            item.setTagColor(childMap.get(childKey).getTagColor());
                            item.setTextColor(childMap.get(childKey).getTextColor());
                            item.setItemType(childMap.get(childKey).getContentType());
                            item.setItemOptions(childMap.get(childKey).getContent());
                            item.setWeight(childMap.get(childKey).getWeight());
                            childItems.add(item);
                        }
                    }

                    // 创建分组
                    UserProfileGroup group = new UserProfileGroup();
                    group.setQA(isQA);
                    group.setWeight(masterModuleMap.get(moduleKey).getWeight());
                    group.setGroupKey(moduleKey);
                    group.setGroupName(masterModuleMap.get(moduleKey).getModuleName());
                    group.setGroupIcon(masterModuleMap.get(moduleKey).getModuleIcon());
                    // 自问自答
                    if (isQA) {
                        String moduleIcon = childMap.get(qaModuleKey).getModuleIcon();
                        Integer contentType = childMap.get(qaModuleKey).getContentType();
                        Long moduleId = childMap.get(qaModuleKey).getModuleId();
                        if (CollectionUtil.isNotEmpty(questionAnswers)) {
                            String content = childMap.get(qaModuleKey).getContent();
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("list", JSON.parseArray(content, ModuleItem.class));
                            jsonObject.put("moduleId", moduleId);

                            for (QuestionAnswerDTO answerDTO : questionAnswers) {
                                UserProfileGroup.Item item = new UserProfileGroup.Item();
                                item.setItemKey(qaModuleKey);
                                item.setItemName(answerDTO.getQuestion());
                                item.setItemValue(answerDTO.getAnswer());
                                item.setWeight(answerDTO.getWeight());
                                item.setItemIcon(moduleIcon);
                                item.setItemType(contentType);
                                item.setQuestionId(answerDTO.getQuestionId());
                                item.setItemOptions(JSON.toJSONString(jsonObject));
                                childItems.add(item);
                            }
                        }
                    }
                    Integer groupScore = 0;
                    if (CollectionUtil.isNotEmpty(childItems)) {
                        // 子模块降序
                        childItems.sort(Comparator.comparingInt(UserProfileGroup.Item::getWeight).reversed());
                        groupScore = Convert.toInt(Convert.toBigDecimal(childItems.stream().filter(e -> StringUtils.isNotEmpty(e.getItemValue())).count()).divide(Convert.toBigDecimal(childItems.size()), 2, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)));
                    }
                    group.setGroupScore(groupScore);
                    group.setChildItems(childItems);
                    groups.add(group);
                }
                // 降序
                groups.sort(Comparator.comparingInt(UserProfileGroup::getWeight).reversed());
                return groups;
            }
        } catch (Exception e) {
            log.info("出现错误", e);
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 获取个人信息卡片
     *
     * @param userId
     * @return
     */
    @Override
    public UserCardVO getCard(Long userId) {
//        return userDAO.selectCardById(userId);
        UserCardVO userCard = userDAO.selectCardOnlyReturnUserInfo(userId);
        if (userCard == null) {
            return null;
        }
        Integer nobleLevel = userNobleDAO.selectNobleLevel(userId);
        userCard.setNobleLevel(nobleLevel);
        Integer vipLevel = userVipDAO.selectVipLevel(userId);
        userCard.setVipLevel(vipLevel);
        Long niceNo = userCard.getNiceNo();
        if (niceNo != null) {
            Integer niceLevel = niceNoPoolDAO.selectNiceLevel(niceNo, userId, System.currentTimeMillis());
            userCard.setNiceLevel(niceLevel);
        }
        return userCard;
    }

    /**
     * 获取个人信息卡片
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, UserCardVO> getCardMap(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserCardVO> cardMap = userDAO.selectCardMapByIds(userIds);
        if (CollectionUtil.isEmpty(cardMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserCardVO> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, cardMap.get(userId));
        }
        return resultMap;
    }

    /**
     * 用户是否存在
     *
     * @param userId
     * @return 是否存在
     */
    @Override
    public boolean isUserExist(Long userId) {
        return userDAO.countById(userId) > 0;
    }

    /**
     * 判断昵称是否存在
     *
     * @param userName
     * @param userId
     * @return
     */
    @Override
    public boolean isNameExist(String userName, Long userId) {
        return userDAO.countByName(userName, userId) > 0;
    }

    /**
     * 根据用户id查询
     *
     * @param userId
     * @return 用户
     */
    @Override
    public UserDO getUserById(Long userId) {
        return userDAO.selectById(userId);
    }

    @Override
    public int setUserIsFirstRechargedClick(Long userId) {
        return userDAO.setUserIsFirstRechargedClick(userId);
    }

    /**
     * 判断用户是否在线
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isOnline(Long userId) {
        return BoolIntVal.True == Optional.ofNullable(userDAO.selectIsOnlineById(userId)).orElse(BoolIntVal.False);
    }

    /**
     * 用户在线数量
     *
     * @return
     */
    @Override
    public Long selectOnlineCount() {
        return Optional.ofNullable(userDAO.selectOnlineCount()).orElse(0L);
    }

    @Override
    public Map<Long, Integer> isOnlineByIds(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserDO> userMap = userDAO.selectIsOnlineByIds(userIds);
        Map<Long, Integer> resultMap = new HashMap<>(userMap.size());
        userMap.forEach((key, user) -> {
            if (user != null) {
                resultMap.put(key, user.getIsOnline());
            }
        });
        return resultMap;
    }

    /**
     * 设置用户在线状态
     *
     * @param userId
     * @param isOnline
     * @param clientIp
     * @param platform
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean setIsOnline(Long userId, Integer isOnline, String clientIp, String platform) {
        try {
            // 更新在线状态
            userDAO.updateIsOnline(userId,
                    BoolIntVal.True == isOnline ? BoolIntVal.True : BoolIntVal.False,
                    System.currentTimeMillis());

            // 查询用户最后登录渠道
            Long channelId = 0L;
            String headers = userLoginHisDAO.selectLastHeaders(userId);
            if (StringUtils.isNotBlank(headers)) {
                HeaderPackage headerPackage = JSON.parseObject(headers, HeaderPackage.class);
                if (headerPackage != null) {
                    channelId = Optional.ofNullable(headerPackage.getChannelId()).orElse(0L);
                }
            }

            // 写入在线记录
            Long curTime = System.currentTimeMillis();
            UserOnlineHisDO userOnlineHisDO = new UserOnlineHisDO();
            userOnlineHisDO.setHisId(SecureUtil.md5(String.format("%d:%d:%d", userId, isOnline, curTime)));
            userOnlineHisDO.setUserId(userId);
            userOnlineHisDO.setChannelId(channelId);
            userOnlineHisDO.setHisCreateTime(curTime);
            userOnlineHisDO.setHisIp(clientIp);
            userOnlineHisDO.setHisPlatform(platform);
            userOnlineHisDO.setOnlineStatus(isOnline);
            userOnlineHisDAO.insert(userOnlineHisDO);

            // 丢出队列消息
            QueueUserOnlineChangeMsg userOnlineChangeMsg = new QueueUserOnlineChangeMsg();
            userOnlineChangeMsg.setUserId(userId);
            userOnlineChangeMsg.setOnlineStatus(isOnline);
            userOnlineChangeMsg.setChannelId(channelId);
            userOnlineChangeMsg.setTimestamp(curTime);
            userOnlineChangeMsg.setClientType(ClientType.fromRongCloudPlatform(platform).getValue());

            if (isDev()) {
                log.info("发送用户在线状态改变mq消息:{}", JSON.toJSONString(userOnlineChangeMsg));
            }

            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.USER_ONLINE_CHANGE, userOnlineChangeMsg.serialize());

            return true;
        } catch (Exception e) {
            log.error("用户在线状态设置失败", e);
        }
        return false;
    }

    /**
     * 获取财富等级
     *
     * @param userId 用户id
     * @return 财富等级
     */
    @Override
    public int getWealthLevelByUserId(Long userId) {
        return Optional.ofNullable(userDAO.selectWealthLevel(userId)).orElse(1);
    }

    /**
     * 获取魅力等级
     *
     * @param userId 用户id
     * @return Integer 魅力等级
     */
    @Override
    public int getCharmLevelByUserId(Long userId) {
        return Optional.ofNullable(userDAO.selectCharmLevel(userId)).orElse(1);
    }

    /**
     * 获取星耀等级
     *
     * @param userId 用户id
     * @return 魅力等级
     */
    @Override
    public int getStarLevelByUserId(Long userId) {
        return Optional.ofNullable(userDAO.selectStarLevel(userId)).orElse(1);
    }

    /**
     * 获取积分值
     *
     * @param userId
     * @param scoreType
     * @return
     */
    @Override
    public Long getScoreValue(Long userId, Integer scoreType) {
        return scoreProvider.getScoreValue(userId, ScoreTypeEnum.toEnum(scoreType));
    }

    /**
     * 获取积分值
     *
     * @param userIds
     * @param scoreType
     * @return
     */
    @Override
    public Map<Long, Long> getScoreValueMap(Set<Long> userIds, Integer scoreType) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Long> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, getScoreValue(userId, scoreType));
        }
        return resultMap;
    }

    /**
     * 更新财富等级
     *
     * @param userId 用户id
     * @param level  财富等级
     * @return int 更新条数
     */
    @Override
    public boolean setWealthLevel(Long userId, Integer level) {
        try {
            // 更新财富等级
            userDAO.updateWealthLevel(userId, level);
            return true;
        } catch (Exception e) {
            log.error("财富等级更新失败");
        }
        return false;
    }

    /**
     * 更新魅力等级
     *
     * @param userId 用户id
     * @param level  魅力等级
     * @return int 更新条数
     */
    @Override
    public boolean setCharmLevel(Long userId, Integer level) {
        try {
            userDAO.updateCharmLevel(userId, level);
            return true;
        } catch (Exception e) {
            log.error("魅力等级更新失败", e);
        }
        return false;
    }

    /**
     * 更新积分等级
     *
     * @param userId   用户id
     * @param typeEnum 积分类型
     * @param level    等级
     * @return 更新条数
     */
    @Override
    public boolean setScoreLevel(Long userId, ScoreTypeEnum typeEnum, int level) {
        switch (typeEnum) {
            case WEALTH:
                return setWealthLevel(userId, level);
            case CHARM:
                return setCharmLevel(userId, level);
            default:
                log.error("暂不支持更新该等级");
                break;
        }
        return false;
    }

    /**
     * 封号
     *
     * @param userId 用户id
     * @return 是否成功
     */
    @Override
    public boolean forbid(Long userId) {
        return userDAO.updateStatus(userId, CommonStatus.Disabled) == 1;
    }

    /**
     * 获取用户状态
     *
     * @param userId 用户id
     * @return 状态
     */
    @Override
    public Integer getStatus(Long userId) {
        return userDAO.getStatus(userId);
    }

    /**
     * 获取用户基本信息
     *
     * @param userId 用户id
     * @return 基本信息
     */
    @Override
    public UserBasicInfo getBasicInfo(Long userId) {
        return userDAO.selectBasicInfo(userId);
    }

    /**
     * 获取用户基本信息
     *
     * @param userIds
     * @return 基本信息
     */
    @Override
    public Map<Long, UserBasicInfo> getBasicInfoMap(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserBasicInfo> basicInfoMap = userDAO.selectBasicInfoMap(userIds);
        if (CollectionUtil.isEmpty(basicInfoMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserBasicInfo> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, basicInfoMap.get(userId));
        }
        return resultMap;
    }

    @Override
    public UserBasicInfo getUserByUserNoOrNiceNo(Long params) {
        return userDAO.selectByUserNoOrNiceNo(params);
    }

    /**
     * 根据id查询
     *
     * @param userId
     * @return
     */
    @Override
    public UserDO selectById(Long userId) {
        return userDAO.selectById(userId);
    }

    @Override
    public PageList<UserDO> getUserPageList(UserPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<UserDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(userDAO.selectPageList(qw));
        page.setTotal(userDAO.selectCount(qw));
        return page;
    }

    /**
     * 获取用户直播间资料字符串
     *
     * @param userId
     * @return
     */
    @Override
    public String getLiveChatUserProfileString(Long userId, Long anchorUserId) {
        return liveUserProvider.getLiveChatUserProfileString(userId, anchorUserId);
    }

    /**
     * 获取用户在直播间的资料
     *
     * @param userId
     * @param anchorUserId
     * @return
     */
    @Override
    public LiveUserProfile getLiveUserProfile(Long userId, Long anchorUserId) {
        return liveUserProvider.getLiveUserProfile(userId, anchorUserId);
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @return
     */
    @Override
    public LiveUserProfile cacheLiveUserProfile(Long userId, Long anchorUserId) {
        return liveUserProvider.cacheLiveUserProfile(userId, anchorUserId);
    }


    /**
     * 移除用户直播间缓存资料
     *
     * @param userId
     * @return
     */
    @Override
    public boolean removeLiveUserProfile(Long userId, Long anchorUserId) {
        return liveUserProvider.removeLiveUserProfile(userId, anchorUserId);
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @param anchorUserId
     * @return
     */
    @Override
    public boolean refreshLiveUserProfile(Long userId, Long anchorUserId) {
        return liveUserProvider.refreshLiveUserProfile(userId, anchorUserId);
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @return
     */
    @Override
    public boolean refreshLiveUserProfile(Long userId) {
        return liveUserProvider.refreshLiveUserProfile(userId);
    }

    /**
     * 获取用户所在直播间 id
     *
     * @param userId
     * @return
     */
    @Override
    public Long getLiveUserRoom(Long userId) {
        return liveUserProvider.getUserRoom(userId);
    }

    /**
     * 获取用户聊天群组资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    @Override
    public LiveUserProfile getChatUserProfile(Long userId, String groupId) {
        return chatUserProvider.getUserProfile(userId, groupId);
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    @Override
    public LiveUserProfile cacheChatUserProfile(Long userId, String groupId) {
        return chatUserProvider.cacheChatUserProfile(userId, groupId);
    }

    /**
     * 移除用户直播间缓存资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    @Override
    public boolean removeChatUserProfile(Long userId, String groupId) {
        return chatUserProvider.removeUserProfile(userId, groupId);
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    @Override
    public boolean refreshChatUserProfile(Long userId, String groupId) {
        return chatUserProvider.refreshUserProfile(userId, groupId);
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @return
     */
    @Override
    public boolean refreshChatUserProfile(Long userId) {
        return chatUserProvider.refreshUserProfile(userId);
    }

    /**
     * 查询用户id
     *
     * @param pageNum  当前页码
     * @param pageSize 分页大小
     * @return 用户id集合
     */
    @Override
    public List<Long> getUserIds(int pageNum, int pageSize) {
        return userDAO.getUserIds((pageNum - 1) * pageSize, pageSize);
    }

    /**
     * 获取直播间粉丝团用户列表
     *
     * @param anchorUserId
     * @return
     */
    @Override
    public List<LiveUserProfile> getLiveFansList(Long anchorUserId, long start, long end) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisCacheKeys.getLiveFansKey(anchorUserId), start, end);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            List<LiveUserProfile> liveUserProfiles = new ArrayList<>();
            for (Tuple tuple : tupleSet) {
                Long userId = Convert.toLong(tuple.getElement());
                LiveUserProfile liveUserProfile = getLiveUserProfile(userId, anchorUserId);
                liveUserProfile.setUserId(userId);
                liveUserProfiles.add(liveUserProfile);
            }
            return liveUserProfiles;
        }
        return Collections.emptyList();
    }

    /**
     * 获取直播间贵族用户列表
     *
     * @param anchorUserId
     * @param start
     * @param end
     * @return
     */
    @Override
    public List<LiveUserProfile> getLiveNobleList(Long anchorUserId, long start, long end) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisCacheKeys.getLiveNobleKey(anchorUserId), start, end);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            List<LiveUserProfile> liveUserProfiles = new ArrayList<>();
            for (Tuple tuple : tupleSet) {
                Long userId = Convert.toLong(tuple.getElement());
                LiveUserProfile liveUserProfile = getLiveUserProfile(userId, anchorUserId);
                liveUserProfile.setUserId(userId);
                liveUserProfiles.add(liveUserProfile);
            }
            return liveUserProfiles;
        }
        return Collections.emptyList();
    }

    /**
     * 获取直播间在线用户列表
     *
     * @param anchorUserId
     * @param start
     * @param end
     * @return
     */
    @Override
    public List<LiveUserProfile> getLiveUserList(Long anchorUserId, long start, long end) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisCacheKeys.getLiveOnlineKey(anchorUserId), start, end);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            List<Tuple> tupleList = new ArrayList<>(tupleSet);
            tupleList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
            List<LiveUserProfile> liveUserProfiles = new ArrayList<>();
            for (Tuple tuple : tupleList) {
                Long userId = Convert.toLong(tuple.getElement());
                LiveUserProfile liveUserProfile = getLiveUserProfile(userId, anchorUserId);
                liveUserProfile.setUserId(userId);
                liveUserProfiles.add(liveUserProfile);
            }
            return liveUserProfiles;
        }
        return Collections.emptyList();
    }

    /**
     * 根据用户 id 获取头像和昵称
     *
     * @param userIds
     * @return
     */
    @Override
    public List<UserCardVO> getNameAndAvatarByIds(Collection<Long> userIds) {
        return userDAO.selectNameAndAvatarByIds(userIds);
    }

    /**
     * 根据用户ID获取头像和昵称
     *
     * @param userId
     * @return
     */
    @Override
    public UserCardVO getNameAndAvatarById(Long userId) {
        return userDAO.selectNameAndAvatarById(userId);
    }

    @Override
    public Map<Long, PeopleNearbyVO> getUserMapByByIds(Collection<Long> userIds) {
        Set<Long> userIdSet = new HashSet<>(userIds);
        Map<Long, PeopleNearbyVO> voMap = userDAO.selectPeopleNearbyMapByIds(userIds);
        if (CollectionUtil.isNotEmpty(voMap)) {
            // 头饰
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(voMap.keySet()));
            // 语音房
            Map<Long, Long> userRoomMap = rtcUserService.getUserRoomIdMap(userIds);
            // 实名认证状态
            Map<Long, Boolean> authStatus = getAuthStatus(userIdSet);
            // 贵族图标
            Map<Long, UserNobleDO> nobleMap = userNobleDAO.selectByUserIds(userIdSet, System.currentTimeMillis());

            for (Map.Entry<Long, PeopleNearbyVO> entry1 : voMap.entrySet()) {
                // 头饰
                entry1.getValue().setHeadwear(CollectionUtil.isNotEmpty(headwearMap) ? headwearMap.get(entry1.getKey()) : null);
                // 语音房
                entry1.getValue().setRoomId(CollectionUtil.isNotEmpty(userRoomMap) ? userRoomMap.get(entry1.getKey()) : null);
                // 是否实名认证
                entry1.getValue().setAuth(CollectionUtil.isNotEmpty(authStatus) ? authStatus.get(entry1.getKey()) : false);
                // 贵族等级
                entry1.getValue().setNobleLevel(CollectionUtil.isNotEmpty(nobleMap) ? Optional.ofNullable(nobleMap.get(entry1.getKey())).map(UserNobleDO::getNobleLevel).orElse(null) : null);
            }
        }
        return voMap;
    }

    /**
     * 获取用户名
     *
     * @param userId
     * @return
     */
    @Override
    public String getUserName(Long userId) {
        return userDAO.selectNameById(userId);
    }

    /**
     * 获取手机号
     *
     * @param userId
     * @return
     */
    @Override
    public String getPhone(Long userId) {
        return userSecurityDAO.selectPhoneById(userId);
    }

    /**
     * 获取星芽号
     *
     * @param userId
     * @return
     */
    @Override
    public Long getUserNo(Long userId) {
        return userDAO.selectUserNo(userId);
    }

    /**
     * 根据星芽号获取用户 id
     *
     * @param userNo
     * @return
     */
    @Override
    public Long getUserIdByNo(Long userNo) {
        return userDAO.selectIdByNo(userNo);
    }

    /**
     * 是否正在直播
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isLive(Long userId) {
        return LiveStatusEnum.Living == LiveStatusEnum.toEnum(anchorDAO.getLiveStatus(userId));
    }

    /**
     * 设置开关
     *
     * @param switchDO
     * @return
     */
    @Override
    public boolean setSwitch(UserSwitchDO switchDO) {
        return userSwitchDAO.insertOrUpdate(switchDO) > 0;
    }

    /**
     * 获取开关状态
     *
     * @param userId
     * @param switchType
     * @return
     */
    @Override
    public int getSwitchStatus(Long userId, SwitchTypeEnum switchType) {
        return Optional.ofNullable(userSwitchDAO.getSwitchStatus(String.format("%d:%d", userId, switchType.getCode()))).orElse(switchType.getDefaultValue());
    }

    /**
     * 获取物品id
     *
     * @param userId
     * @param switchType
     * @return
     */
    @Override
    public Long getSwitchItemId(Long userId, SwitchTypeEnum switchType) {
        return userSwitchDAO.getItemIdById(String.format("%d:%d", userId, switchType.getCode()));
    }

    /**
     * 获取开关
     *
     * @param userId
     * @param switchType
     * @return
     */
    @Override
    public UserSwitchDO getSwitch(Long userId, SwitchTypeEnum switchType) {
        return userSwitchDAO.selectById(String.format("%d:%d", userId, switchType.getCode()));
    }

    /**
     * 获取开关
     *
     * @param userIds
     * @param switchType
     * @return
     */
    @Override
    public Map<Long, UserSwitchDO> getSwitchMap(Set<Long> userIds, SwitchTypeEnum switchType) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Set<String> switchIds = new HashSet<>();
        for (Long userId : userIds) {
            switchIds.add(String.format("%d:%d", userId, switchType.getCode()));
        }
        return userSwitchDAO.selectByIds(switchIds);
    }

    /**
     * 获取用户开关
     *
     * @param userId
     * @return
     */
    @Override
    public List<UserSwitchVO> getSwitchList(Long userId) {
        // 用户开关
        List<UserSwitchVO> userSwitchList = userSwitchDAO.getSwitchList(userId);
        List<UserSwitchVO> switchVOList = new ArrayList<>();

        // 总开关
        List<Integer> totalSwitch = Arrays.stream(SwitchTypeEnum.values()).map(switchTypeEnum -> switchTypeEnum.getCode()).collect(Collectors.toList());

        if (isDev()) {
            log.info("获取开关：{}", JSON.toJSONString(totalSwitch));
        }

        // 已经保存的开关
        List<Integer> hasSwitch = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(userSwitchList)) {
            for (UserSwitchVO switchVO : userSwitchList) {
                SwitchTypeEnum switchType = SwitchTypeEnum.toEnum(switchVO.getType());
                if (switchType == null) {
                    continue;
                }
                hasSwitch.add(switchType.getCode());
                switchVO.setSwitchName(switchType.getDesc());
                switchVOList.add(switchVO);
            }
        }
        totalSwitch.removeAll(hasSwitch);

        if (isDev()) {
            log.info("获取开关totalSwitch：{}", JSON.toJSONString(totalSwitch));
        }

        // 应该补全的开关 给默认值
        List<Integer> defaultSwitch = totalSwitch;
        defaultSwitch.stream().forEach(e -> {
            UserSwitchVO vo = new UserSwitchVO();
            vo.setType(e);
            vo.setUserId(userId);
            vo.setSwitchId(String.format("%d:%d", userId, e));
            vo.setSwitchStatus(SwitchTypeEnum.toEnum(e).getDefaultValue());
            vo.setSwitchName(SwitchTypeEnum.toEnum(e).getDesc());
            switchVOList.add(vo);
        });

        if (isDev()) {
            log.info("获取开关switchVOList：{}", JSON.toJSONString(switchVOList));
        }
        return switchVOList;
    }

    /**
     * 创建访问记录
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public boolean addVisitHis(Long userId, Long targetUserId) {
        return visitProvider.addVisitHis(userId, targetUserId);
    }

    /**
     * 获取访客列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<VisitorVO> getVisitorList(Long userId, int pageIndex, int pageSize) {
        return visitProvider.getVisitorList(userId, pageIndex, pageSize);
    }

    /**
     * 获取足迹列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<IntervieweeVO> getIntervieweeList(Long userId, int pageIndex, int pageSize) {
        return visitProvider.getIntervieweeList(userId, pageIndex, pageSize);
    }

    @Override
    public List<RecentlySeenUser> getRecentlySeenUsers(Long visitorUserId, int limit) {
        List<Long> userIds = visitProvider.getRecentlySeenUserIds(visitorUserId, limit);
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }

        Map<Long, UserOnline> userOnlineMap = getUserOnlineMap(userIds);
        if (CollectionUtils.isEmpty(userOnlineMap)) {
            return Collections.emptyList();
        }

        Collection<UserOnline> users = userOnlineMap.values();
        List<RecentlySeenUser> recentlySeenUsers = addOnlineStatusToRecentlySeenUsers(users);
        List<Long> onlineUserIds = getOnlineUserIds(users);
        Map<Long, RtcRoomSummary> userIdRtcRoomSummaryMap = getUserIdRtcRoomSummaryMap(onlineUserIds);
        return addRtcRoomToRecentlySeenUsers(recentlySeenUsers, userIdRtcRoomSummaryMap);
    }

    /**
     * 我看过谁
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<VisitorVOV2> getVisitorListV2(Long userId, int pageIndex, int pageSize) {

        PageList<VisitorVOV2> pageList = visitProvider.getVisitorListV2(userId, pageIndex, pageSize);
        if (pageList.getTotal() == 0) {
            return pageList;
        }
        List<VisitorVOV2> list = pageList.getList();
        Set<Long> userIds = list.stream().filter(Objects::nonNull).map(VisitorVOV2::getUserId).collect(Collectors.toSet());

        Map<Long, UserDO> userMaps = getUserMapByIds(userIds);

        list.forEach(val -> {
            if (MapUtils.isEmpty(userMaps)) {
                return;
            }
            if (!userMaps.containsKey(val.getUserId())) {
                return;
            }

            UserDO user = userMaps.get(val.getUserId());
            val.setUserAge(user.getUserAge());
            val.setUserAvatar(user.getUserAvatar());
            val.setUserName(user.getUserName());
            val.setUserSex(user.getUserSex());
            val.setIsOnline(user.getIsOnline());

            // 统计每个用户访问次数
            val.setHisTimes(visitProvider.getVisitorCount(userId, val.getUserId(), val.getHisDate()));
            val.setUserState(getUserStateByUserId(userId, val.getUserId()));
        });
        return pageList;
    }

    private Integer getUserStateByUserId(Long userId, Long targetUserId) {
        // 未关注显示关注
        // 关注了，不在房间内显示聊天，在房间内显示直播或派对
        boolean isFollow = socialRelationService.isFollower(userId, targetUserId);
        if (!isFollow) {
            return FollowUserStateEnum.FOLLOW.getValue();
        }

        Long roomId = rtcUserService.getUserStayRoomId(targetUserId);
        if (roomId == null) {
            return FollowUserStateEnum.CHAT.getValue();
        }

        if (roomId == 0) {
            return FollowUserStateEnum.CHAT.getValue();
        }

        boolean isSingleRtcRoom = rtcRoomService.isSingleRtcRoom(roomId);
        if (isSingleRtcRoom) {
            return FollowUserStateEnum.LIVE.getValue();
        } else {
            return FollowUserStateEnum.PARTY.getValue();
        }
    }

    /**
     * 谁看过我
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<IntervieweeVOV2> getIntervieweeListV2(Long userId, int pageIndex, int pageSize) {

        PageList<IntervieweeVOV2> pageList = visitProvider.getIntervieweeListV2(userId, pageIndex, pageSize);
        if (pageList.getTotal() == 0) {
            return pageList;
        }
        List<IntervieweeVOV2> list = pageList.getList();
        Set<Long> userIds = list.stream().filter(Objects::nonNull).map(IntervieweeVOV2::getUserId).collect(Collectors.toSet());

        Map<Long, UserDO> userMaps = getUserMapByIds(userIds);

        list.forEach(val -> {
            if (MapUtils.isEmpty(userMaps)) {
                return;
            }

            if (!userMaps.containsKey(val.getUserId())) {
                return;
            }

            UserDO user = userMaps.get(val.getUserId());
            val.setUserAge(user.getUserAge());
            val.setUserAvatar(user.getUserAvatar());
            val.setUserName(user.getUserName());
            val.setUserSex(user.getUserSex());
            val.setIsOnline(user.getIsOnline());

            // 统计每个用户访问次数
            val.setHisTimes(visitProvider.getInterviewCount(userId, val.getUserId(), val.getHisDate()));
            val.setUserState(getUserStateByUserId(userId, val.getUserId()));
        });

        // 已读
        visitProvider.resetInterviewCount(userId);
        return pageList;
    }

    /**
     * 谁看过我统计
     * 总访客
     *
     * @return
     */
    @Override
    public Long getIntervieweeTotal(Long userId) {
        return visitProvider.getInterviewCountByUser(userId, null);
    }

    /**
     * 谁看过我总用户数统计
     * 总访客
     *
     * @return
     */
    @Override
    public Long getIntervieweeTotalByUser(Long userId) {
        return visitProvider.getIntervieweeTotalByUser(userId);
    }

    /**
     * 我的足迹总统计
     *
     * @return
     */
    @Override
    public Long getVisitorTotal(Long userId) {
        return visitProvider.getVisitorCountByUser(userId, null);
    }

    /**
     * 我的足迹总用户统计
     *
     * @return
     */
    @Override
    public Long getVisitorTotalByUser(Long userId) {
        return visitProvider.getVisitorTotalByUser(userId);
    }

    /**
     * 未读访客数
     *
     * @return
     */
    @Override
    public Long getIntervieweeUnReadMsgCount(Long userId) {
        return visitProvider.getUnReadInterviewCount(userId);
    }

    /**
     * 谁看过我统计
     * 今日访客
     *
     * @return
     */
    @Override
    public Long getIntervieweeToday(Long userId) {
        int dateInt = DateUtil.getIntDate();
        return visitProvider.getInterviewCountByUser(userId, dateInt);
    }

    /**
     * 更新位置
     *
     * @param userId
     * @param userLatitude
     * @param userLongitude
     * @param areaId
     * @return
     */
    @Override
    public boolean setPosition(Long userId, Double userLatitude, Double userLongitude, Integer areaId) {
        String locationCity = areaService.getLocationCity(userLongitude, userLatitude);
        userDAO.updatePosition(userId, userLatitude, userLongitude, areaId, locationCity);
        userSearchService.setPosition(userId, userLatitude, userLongitude, areaId, locationCity);
        return true;
    }

    /**
     * 获取绑定账户
     *
     * @param userId
     * @param channelId
     * @return
     */
    @Override
    public UserBindAccountsVO getBingAccount(Long userId, Long channelId) {
        UserBindAccountsVO vo = new UserBindAccountsVO();
        vo.setUserId(userId);
        String userPhone = userSecurityDAO.selectPhoneById(userId);
        if (StringUtils.isNotEmpty(userPhone) && userPhone.length() > 4) {
            vo.setUserPhone("*******" + userPhone.substring(userPhone.length() - 4));
        }
        vo.setWx(userOauthDAO.countById(userId, channelId, OAuthTypeEnum.WEIXIN_APP.getType()) > 0);
        vo.setQq(userOauthDAO.countById(userId, channelId, OAuthTypeEnum.QQ_APP.getType()) > 0);
        vo.setWeibo(userOauthDAO.countById(userId, channelId, OAuthTypeEnum.WEIBO_APP.getType()) > 0);
        vo.setHuawei(userOauthDAO.countById(userId, channelId, OAuthTypeEnum.HUAWEI_APP.getType()) > 0);
        vo.setIsPassSet(securityService.isUserPassSet(userId));
        return vo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean bindOauth(UserOauthDO oauthDO) {
        return userOauthDAO.insert(oauthDO) == 1;
    }

    /**
     * 获取当日当时前在线离线次数
     *
     * @param userId
     * @param onlineStatus
     * @param timestamp
     * @return
     */
    @Override
    public int getOnlineTimesDayBefore(Long userId, Integer onlineStatus, Long timestamp) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(timestamp);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        long beginTime = calendar.getTimeInMillis();

        return userOnlineHisDAO.countByStatusWithTime(userId, onlineStatus, beginTime, timestamp);
    }

    /**
     * 是否充值过
     *
     * @param userId
     * @return
     */
    @Override
    public boolean hasRecharged(Long userId) {
        return userDAO.countHasRechargedById(userId) > 0;
    }

    /**
     * 是否超管
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isSuper(Long userId) {
        return liveUserProvider.isSuper(userId);
    }

    @Override
    public Long getInviterUserId(Long userId) {
        return userInviteDAO.selectByUserId(userId);
    }

    /**
     * 获取邀请人
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, Long> getInviterUserIdMap(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Long> resultMap = new HashMap<>();
        Map<Long, UserInviteDO> inviteUserVOMap = userInviteDAO.selectByUserIdMap(userIds);
        if (CollectionUtil.isEmpty(inviteUserVOMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Long> userInviteMap = new HashMap<>();
        for (Map.Entry<Long, UserInviteDO> entry : inviteUserVOMap.entrySet()) {
            userInviteMap.put(entry.getKey(), entry.getValue().getInviterUserId());
        }
        for (Long userId : userIds) {
            resultMap.put(userId, CollectionUtil.isEmpty(userInviteMap) ? null : userInviteMap.get(userId));
        }
        return resultMap;
    }

    /**
     * 获取用户认证信息
     *
     * @param userId
     * @return
     */
    @Override
    public UserAuthDO getUserAuth(Long userId) {
        return userAuthDAO.selectById(userId);
    }


    @Override
    public Integer getUserConstellation(Long userId) {
        return userDAO.selectUserConstellation(userId);
    }

    @Override
    public Long getLastOnlineTime(Long userId) {
        return userOnlineHisDAO.selectLastOnlineTime(userId);
    }

    @Override
    public Long getLastOnlineTimeBeforeOffline(Long userId) {
        return userOnlineHisDAO.selectLastOnlineTimeBeforeOffline(userId);
    }

    @Override
    public String getAreaCode(Long userId) {
        return userSecurityDAO.selectAreaCode(userId);
    }

    @Override
    public Integer getCountryId(Long userId) {
        return userDAO.selectCountryId(userId);
    }

    /**
     * 更新用户 areaId
     *
     * @param userId
     * @param areaId
     * @return
     */
    @Override
    public boolean setAreaId(Long userId, Integer areaId) {
        return userDAO.updateAreaId(userId, areaId) > 0;
    }

    @Override
    public Long getChannelId(Long userId) {
        return userDAO.selectChannelId(userId);
    }

    /**
     * 获取用户邀请列表
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @param hasRecharged
     * @return
     */
    @Override
    public PageList<InviteUserVO> getInviteUserList(int pageIndex, int pageSize, Long userId, Integer hasRecharged) {
        PageList<InviteUserVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(userInviteDAO.selectAllByUserId(pageList.getPageCursor(), pageSize, userId, hasRecharged));
        pageList.setTotal(userInviteDAO.countByUserId(userId, hasRecharged));
        return pageList;
    }

    @Override
    public List<UserDTO> getUsers(List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.EMPTY_LIST;
        }
        return userDAO.selectUsers(userIds);
    }

    @Override
    public boolean addId(UserIdCardDO idCardDO) {
        int count = userIdCardDAO.insertOrUpdate(idCardDO);
        if (count > 0) {
            // 发消息
            QueueIdCardAddMsg addMsg = new QueueIdCardAddMsg();
            addMsg.setUserId(idCardDO.getUserId());
            addMsg.setFaceImg(idCardDO.getFaceImg());
            addMsg.setNationImg(idCardDO.getNationImg());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ID_CARD_ADD, addMsg.serialize());
            return true;
        }
        return false;
    }

    @Override
    public boolean isAuthExist(Long userId) {
        return userAuthDAO.countById(userId) > 0;
    }

    /**
     * 是否实名认证
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, Boolean> getAuthStatus(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Boolean> resultMap = new HashMap<>();
        for (Long userId : userIds) {
            resultMap.put(userId, isAuthExist(userId));
        }
        return resultMap;
    }

    @Override
    public UserIdCardDO getIdCard(Long userId) {
        return userIdCardDAO.selectById(userId);
    }

    /**
     * 根据手机号码获得用户id
     *
     * @param phoneList 用户手机列表
     * @param areaCode  区号
     * @return key:手机号; value:用户id
     */
    @Override
    public Map<String, Long> getPhoneUserIdMap(Set<String> phoneList, String areaCode) {
        if (CollectionUtil.isEmpty(phoneList)) {
            return Collections.EMPTY_MAP;
        }
        Map<String, UserDO> userMap = userSecurityDAO.selectPhoneUserIdMap(phoneList, areaCode);
        if (CollectionUtil.isEmpty(userMap)) {
            return Collections.EMPTY_MAP;
        }
        Map<String, Long> map = new HashMap<>(userMap.size());
        for (Map.Entry<String, UserDO> entry : userMap.entrySet()) {
            UserDO user = entry.getValue();
            if (user == null) {
                continue;
            }
            Long userId = user.getUserId();
            if (userId == null) {
                continue;
            }
            map.put(entry.getKey(), userId);
        }
        return map;
    }

    /**
     * 根据用户id列表查询总数（判断是否用户id都存在）
     *
     * @param userIds
     * @return
     */
    @Override
    public int getCountByUserIds(Set<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return 0;
        }
        return userDAO.selectCountByUserIds(userIds);
    }

    /**
     * 获取用户注册时间
     *
     * @param userId
     * @return
     */
    @Override
    public Long getCreateTime(Long userId) {
        return userDAO.selectCreateTime(userId);
    }

    /**
     * 获取用户Map
     *
     * @param userIds 用户id集合
     * @return
     */
    @Override
    public Map<Long, UserDO> getUserMapByIds(Collection<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, UserDO> map = userDAO.selectUserMapByIds(userIds);
        if (CollectionUtils.isEmpty(map)) {
            return Collections.EMPTY_MAP;
        }
        return map;
    }

    /**
     * 判断用户是否被封禁
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isDisabled(Long userId) {
        return Optional.ofNullable(userDAO.selectUserStatus(userId)).map(e -> e < CommonStatus.Normal).orElse(false);
    }

    /**
     * 获取语音聊天状态
     *
     * @param userId
     * @return
     */
    @Override
    public Integer getVoiceChatStatus(Long userId) {
        return Optional.ofNullable(userDAO.selectVoiceChatStatus(userId)).orElse(VoiceChatStatus.IDLE);
    }

    /**
     * 更新用户语音状态
     *
     * @param userId
     * @param voiceChatStatus
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateVoiceChatStatus(Long userId, Integer voiceChatStatus) {
        return userDAO.updateVoiceChatStatus(userId, voiceChatStatus) > 0;
    }

    /**
     * 获得附近的人搜索半径
     *
     * @return 搜索半径（单位：米）
     */
    @Override
    public double getPeopleNearbyDistance() {
        String distance = redisService.get(RedisUserKeys.PEOPLE_NEARBY_DISTANCE);
        if (StringUtils.isBlank(distance) || !NumberUtil.isInteger(distance)) {
            return 100_000;
        }
        return Double.parseDouble(distance);
    }

    /**
     * 获得搜索默认关键词
     *
     * @return
     */
    @Override
    public String getSearchDefaultKeywords() {
        return redisService.get(RedisUserKeys.SEARCH_DEFAULT_KEYWORDS);
    }

    /**
     * 判断是否为官方账号
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isOfficial(Long userId) {
        return adminDAO.countByUserId(userId) > 0;
    }

    /**
     * 判断是否为官方账号
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, Boolean> getOfficialMap(List<Long> userIds) {
        if (CollectionUtil.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Boolean> supperMap = new HashMap<>();
        List<Long> supperUserIds = adminDAO.selectSuppersByUserIds(userIds);
        for (Long userId : userIds) {
            supperMap.put(userId, supperUserIds.contains(userId));
        }
        return supperMap;
    }

    /**
     * 获取问答
     *
     * @param userId
     * @return
     */
    @Override
    public List<QuestionAnswerDTO> getQuestionAnswers(Long userId) {
        // 用户已添加的问答
        List<QuestionAnswerDO> list = questionAnswerDAO.selectByUserId(userId);
        // 问答模块id
        Set<Long> qaModuleIds = new HashSet<>();
        // 已添加问答模块id
        Set<Long> addModuleIds = new HashSet<>();
        if (CollectionUtil.isEmpty(list)) {
            qaModuleIds = moduleService.getModuleIdsByType(ModuleContentTypeEnum.ADD_QA.getCode());
        } else {
            // 已添加
            addModuleIds = list.stream().map(QuestionAnswerDO::getModuleId).collect(Collectors.toSet());
        }
        qaModuleIds.addAll(addModuleIds);
        // 模块集合
        Map<Long, UserModuleDO> moduleMap = moduleService.getModules(qaModuleIds);
        Set<QuestionAnswerDTO> questionAnswers = new HashSet<>();
        List<Long> unAddList = new ArrayList<>();
        // 已添加的问题
        Set<String> questions = list.stream().map(QuestionAnswerDO::getQuestion).collect(Collectors.toSet());
        // 遍历所有模块
        for (Long moduleId : qaModuleIds) {
            // 已添加
            if (addModuleIds.contains(moduleId)) {
                UserModuleDO moduleDO = moduleMap.get(moduleId);
                if (moduleDO == null) {
                    continue;
                }
                String content = moduleDO.getContent();
                // 已添加的模块 对应的问题集合
                List<ModuleItem> moduleItems = JSON.parseArray(content, ModuleItem.class);
                if (CollectionUtil.isNotEmpty(moduleItems)) {
                    for (ModuleItem moduleItem : moduleItems) {
                        QuestionAnswerDTO answerDTO = null;
                        // 未添加的问题
                        if (!questions.contains(moduleItem.getContent())) {
                            answerDTO = new QuestionAnswerDTO();
                            answerDTO.setWeight(moduleItem.getWeight());
                            answerDTO.setQuestion(moduleItem.getContent());
                            answerDTO.setModuleId(moduleId);
                            answerDTO.setSet(false);
                        } else {
                            answerDTO = new QuestionAnswerDTO();
                            // 已添加的问题
                            QuestionAnswerDO answerDO = list.stream().filter(e -> moduleId.equals(e.getModuleId()) && e.getQuestion().equals(moduleItem.getContent())).collect(Collectors.toList()).get(0);
                            if (answerDO != null) {
                                BeanUtil.copyProperties(answerDO, answerDTO);
                                answerDTO.setSet(true);
                                // 排序靠前
                                answerDTO.setWeight(moduleItem.getWeight());
                            }
                        }
                        if (answerDTO != null) {
                            questionAnswers.add(answerDTO);
                        }
                    }
                }
            } else {
                // 暂存未添加
                unAddList.add(moduleId);
            }
        }

        // 未添加的模块 => 添加到问答中
        for (Long moduleId : unAddList) {
            String content = moduleMap.get(moduleId).getContent();
            List<ModuleItem> moduleItems = JSON.parseArray(content, ModuleItem.class);
            if (CollectionUtil.isNotEmpty(moduleItems)) {
                Long qaModuleId = moduleMap.get(moduleId).getModuleId();
                for (ModuleItem moduleItem : moduleItems) {
                    QuestionAnswerDTO answerDTO = new QuestionAnswerDTO();
                    answerDTO.setSet(false);
                    answerDTO.setWeight(moduleItem.getWeight());
                    answerDTO.setQuestion(moduleItem.getContent());
                    answerDTO.setModuleId(qaModuleId);
                    questionAnswers.add(answerDTO);
                }
            }
        }
        // 排序
        List<QuestionAnswerDTO> sortedQuestions = questionAnswers.stream().sorted(Comparator.comparing(QuestionAnswerDTO::isSet).reversed().thenComparingLong(QuestionAnswerDTO::getWeight).reversed()).collect(Collectors.toList());
        return sortedQuestions;
    }

    /**
     * 获取用户资料评分
     *
     * @param userId
     * @return
     */
    @Override
    public Integer getProfileScore(Long userId) {
        List<UserProfileGroup> profileNew = getProfileNew(userId);
        return CollectionUtil.isNotEmpty(profileNew) ? Convert.toInt(profileNew.stream().mapToDouble(UserProfileGroup::getGroupScore).average().getAsDouble()) : 0;
    }

    /**
     * 恢复默认头像
     *
     * @param userId
     * @param reason
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setDefaultAvatar(Long userId, String reason) {
        UserDO userDO = userDAO.selectById(userId);
        // 更新的用户
        UserDO update = new UserDO();
        update.setUserId(userId);
        update.setUserUpdateTime(System.currentTimeMillis());
        // 女
        if (userDO.getUserSex() == null || userDO.getUserSex() == SexEnum.UNKNOWN.getCode() || userDO.getUserSex() == SexEnum.FEMALE.getCode()) {
            update.setUserAvatar(avatarFemale);
        } else {
            // 男
            update.setUserAvatar(avatarMale);
        }
        if (editProfile(update)) {
            // 系统消息
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(ContentType.TEXT);
            systemMsg.setTitle("恢复默认头像");
            systemMsg.setBody("尊敬的用户，您的头像因" + String.format("%s%s%s", '"', reason, '"') + "，被系统恢复为默认头像。");
            systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
            imService.sendCustomImMsg(null, userId, systemMsg, null);
            return true;
        }
        return false;
    }

    /**
     * 设置自问自答
     *
     * @param answerDO
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addQuestionAnswer(QuestionAnswerDO answerDO) {
        return questionAnswerDAO.insertOrUpdate(answerDO) > 0;
    }

    /**
     * 编辑问答
     *
     * @param answerDO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean editQuestionAnswer(QuestionAnswerDO answerDO) {
        return questionAnswerDAO.updateById(answerDO) > 0;
    }

    /**
     * 判断问题是否存在
     *
     * @param questionId
     * @param userId
     * @return
     */
    @Override
    public boolean isQuestionExist(Long questionId, Long userId) {
        return questionAnswerDAO.countById(questionId, userId) > 0;
    }

    /**
     * 设置问题状态
     *
     * @param questionId
     * @param status
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setQuestionStatus(Long questionId, Integer status) {
        return questionAnswerDAO.updateStatus(questionId, status) > 0;
    }

    /**
     * 用户封禁到期
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @Async
    public void handleUserForbiddenExpire() {
        log.info("处理用户封禁到期");
        // 判断是否有过期的 每次50条, 防止dubbo超时
        List<Long> userIds = userForbiddenDAO.selectWaitUpdateUserId();
        if (CollectionUtil.isNotEmpty(userIds)) {
            // 解封
            userForbiddenDAO.updateForbiddenStatus(userIds, CommonStatus.Disabled);
            // 更新状态
            userDAO.updateUserStatus(userIds, CommonStatus.Normal);
            // 同步到es
            userIds.forEach(i -> {
                // 这里因为传输多ID会导致dubbo超时, 所以这么写
                userSearchService.setUserStatus(Arrays.asList(i), CommonStatus.Normal);
            });

            // 用户基本信息
            Map<Long, UserBasicInfo> basicInfoMap = userDAO.selectBasicInfoMap(new HashSet<>(userIds));
            if (CollectionUtil.isNotEmpty(basicInfoMap)) {
                for (Long userId : userIds) {
                    // 推送
                    UserBasicInfo userBasicInfo = basicInfoMap.get(userId);
                    if (userBasicInfo != null) {
                        String pushBody = String.format("【%s】尊敬的用户您好，您的账号（id:%d）已被解禁，快来一块玩耍吧！", releaseConfig.getAppName(), userBasicInfo.getUserNo());
                        SystemMsg systemMsg = new SystemMsg();
                        systemMsg.setContentType(ContentType.TEXT);
                        systemMsg.setTitle("解禁通知");
                        systemMsg.setBody(pushBody);
                        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
                        imService.sendCustomImMsg(null, userId, systemMsg, null);
                    }
                }
            }
        }
        log.info("处理更新用户完成 ， 共解封{}用户", userIds.size());
    }

    /**
     * 获取用户头像
     *
     * @param userId
     * @return
     */
    @Override
    public String getUserAvatar(Long userId) {
        UserDO userDO = getUserById(userId);
        if (userDO != null) {
            return userDO.getUserAvatar();
        } else {
            return "";
        }
    }


    /**
     * 根据用户id获得简单用户信息的Map
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, UserSimpleVO> getSimpleUserMap(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyMap();
        }
        return userDAO.selectSimpleInfoMap(userIds);
    }

    /**
     * 获取用户在线状态信息
     *
     * @param userIds
     * @return
     */
    @Override
    public Map<Long, UserOnline> getUserOnlineMap(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyMap();
        }
        return userDAO.selectUserOnlineMap(userIds);
    }

    /**
     * 获取所有用户数量
     *
     * @return
     */
    @Override
    public Integer countOnlineUser() {
        return userDAO.countOnlineUser();
    }

    /**
     * 获取所有在线用户
     *
     * @return
     */
    @Override
    public List<Long> selectPageListOnlineUser(int pageCursor, int pageSize) {
        return userDAO.selectPageListOnlineUser(pageCursor, pageSize);
    }

    /**
     * 获取传入参数里是管理员的列表
     *
     * @param userIds
     * @return
     */
    @Override
    public Set<Long> getAdminSet(Set<Long> userIds) {
        if (CollectionUtil.isNotEmpty(userIds)) {
            return adminDAO.getAdminSet(userIds);
        } else {
            return new HashSet<>();
        }
    }

    /**
     * 用户封禁原因
     *
     * @param userId
     * @return
     */
    @Override
    public UserForbiddenDO getForbiddenHis(long userId) {
        return userForbiddenDAO.getLastForbiddenHis(userId);
    }

    @Override
    public UserFirstRechargeRemindDO getUserFirstRechargeRemind(long userId, long version) {
        return userFirstRechargeRemindDAO.selectOne(userId, version);
    }

    @Override
    public boolean addUserFirstRechargeRemind(UserFirstRechargeRemindDO userFirstRechargeRemindDO) {
        return userFirstRechargeRemindDAO.insert(userFirstRechargeRemindDO) > 0;
    }

    @Override
    public String getRandomUserName() {
        return chineseNameProvider.getRandomUserName();
    }

    private List<RecentlySeenUser> addRtcRoomToRecentlySeenUsers(List<RecentlySeenUser> recentlySeenUsers, Map<Long, RtcRoomSummary> rtcMap) {
        if (CollectionUtils.isEmpty(recentlySeenUsers)) {
            return Collections.emptyList();
        }
        return recentlySeenUsers.stream()
                .map(recentlySeenUser -> {
                    RtcRoomSummary rtcRoomSummary = rtcMap.get(recentlySeenUser.getUserId());
                    if (rtcRoomSummary != null) {
                        recentlySeenUser.setRtcRoom(rtcRoomSummary);
                    }
                    return recentlySeenUser;
                })
                .collect(Collectors.toList());
    }

    private List<Long> getOnlineUserIds(Collection<UserOnline> users) {
        if (CollectionUtils.isEmpty(users)) {
            return Collections.emptyList();
        }
        List<Long> onlineUserIds = users.stream()
                .filter(Objects::nonNull)
                .filter(user -> user.getIsOnline() - BoolIntVal.True == 0)
                .map(UserOnline::getUserId)
                .collect(Collectors.toList());
        return onlineUserIds;
    }

    private Map<Long, Long> getUserRoomMap(List<Long> onlineUserIds) {
        if (CollectionUtils.isEmpty(onlineUserIds)) {
            return Collections.emptyMap();
        }
        return rtcUserService.getUserRoomIdMap(onlineUserIds);
    }

    private List<Long> getRoomIds(Map<Long, Long> userRoomMap) {
        if (CollectionUtils.isEmpty(userRoomMap)) {
            return Collections.emptyList();
        }
        return userRoomMap.values()
                .stream()
                .filter(Objects::nonNull)
                .distinct()
                .collect(Collectors.toList());
    }

    private Map<Long, RtcRoomSummary> getRtcRoomSummaryMap(List<Long> roomIds) {
        if (CollectionUtils.isEmpty(roomIds)) {
            return Collections.emptyMap();
        }
        return rtcRoomService.getRtcRoomSummaryMap(roomIds);
    }

    /**
     * @param onlineUserIds
     * @return key:userId
     */
    private Map<Long, RtcRoomSummary> getUserIdRtcRoomSummaryMap(List<Long> onlineUserIds) {
        // userId:roomId
        Map<Long, Long> userRoomMap = getUserRoomMap(onlineUserIds);
        List<Long> roomIds = getRoomIds(userRoomMap);
        Map<Long, RtcRoomSummary> rtcRoomSummaryMap = getRtcRoomSummaryMap(roomIds);

        Map<Long, RtcRoomSummary> map = new HashMap<>();
        userRoomMap.forEach((userId, roomId) -> {
            RtcRoomSummary rtcRoomSummary = rtcRoomSummaryMap.get(roomId);
            if (rtcRoomSummary == null) {
                return;
            }
            map.put(userId, rtcRoomSummary);
        });
        return map;
    }

    private List<RecentlySeenUser> addOnlineStatusToRecentlySeenUsers(Collection<UserOnline> users) {
        if (CollectionUtils.isEmpty(users)) {
            return Collections.emptyList();
        }
        return users.stream()
                .map(user -> {
                    RecentlySeenUser recentlySeenUser = new RecentlySeenUser();
                    BeanUtils.copyProperties(user, recentlySeenUser);
                    return recentlySeenUser;
                })
                .collect(Collectors.toList());
    }
}
