package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisWatchKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.user.UserIntimacyCreateReasonEnum;
import com.starbuds.server.common.constant.user.UserWatchCreateByEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.service.user.UserWatchRelationService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.user.dao.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @Author lujiao
 * @DateTime 2021/7/21 18:19
 * @Description
 */
@Service(version = "1.0.0")
@Slf4j
public class UserWatchRelationServiceImpl implements UserWatchRelationService {

    @Resource
    private UserWatchRelationDAO userWatchRelationDAO;

    @Resource
    private WatchDurationDAO watchDurationDAO;

    @Resource
    private WatchConfigDAO watchConfigDAO;

    @Resource
    private UserIntimacyRelationDAO userIntimacyRelationDAO;

    @Resource
    private UserIntimacyRelationHisDAO userIntimacyRelationHisDAO;

    @Resource
    private WatchNotifyHisDAO watchNotifyHisDAO;

    @Resource
    private WatchScoreRankHisDAO watchScoreRankHisDAO;

    @Resource
    private UserWatchRelationHisDAO userWatchRelationHisDAO;

    @Resource
    private UserWatchRelationFlowHisDAO userWatchRelationFlowHisDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    private static final BigDecimal ONE_HUNDRED = new BigDecimal("100");

    private static final long DAY_OF_MILLION = 24 * 60 * 60 * 1000;

    private static final long HOUR_OF_MILLION = 60 * 60 * 1000;

    private static final long WEEK_OF_MILLION = 7 * 24 * 60 * 60 * 1000;

    /**
     * 获取可用的守护时长
     *
     * @return
     */
    @Override
    public List<Integer> getWatchDurationList() {
        List<WatchDurationDO> resModel = watchDurationDAO.getEnbaleWatchDurationList();
        if (CollectionUtil.isNotEmpty(resModel)) {
            List<Integer> res = new ArrayList<>(resModel.size());
            resModel.forEach(i -> res.add(i.getDurationDays()));
            return res;
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * 获取可用的守护时长名称
     *
     * @return
     */
    @Override
    public List<WatchDurationDO> getWatchDurationDoList() {
        return watchDurationDAO.getEnbaleWatchDurationList();
    }

    /**
     * 获取可用的守护类型
     *
     * @param isAuctionOnly 是否仅拍卖可见
     * @return
     */
    @Override
    public List<WatchConfigDO> getWatchConfigList(Integer isAuctionOnly) {
        if (isAuctionOnly == BoolIntVal.Ignore) {
            return watchConfigDAO.getEnableWatchConfigList();
        } else {
            return watchConfigDAO.getEnableWatchConfigListByAuction(isAuctionOnly);
        }
    }

    /**
     * 开通守护操作
     *
     * @return 过期时间
     */
    @Transactional(rollbackFor = Exception.class)
    public Long doNewWatchRelation(UserWatchCreateByEnum createByEnum, Integer days, String orderId, Long watchRelationId, Long userId, Long targetId, Long watchId, Long totalCost) {
        // 判断之前是否开通过守护
        // 如果开通过的话, 进行续费, 如果没开通, 添加记录
        UserWatchRelationHisDO checkDo = userWatchRelationHisDAO.getWatchRelationByOrder(createByEnum.getValue(), orderId);
        if (checkDo != null && checkDo.getHisId() > 0) {
            return checkDo.getExpireTime();
        } else {
            Long now = System.currentTimeMillis();
            Long increaseNum = DAY_OF_MILLION * days;
            Long expireTime;
            Integer isNew = 1;
            Integer createDateInt = DateUtil.getIntDate();
            WatchConfigDO watchConfigDO = watchConfigDAO.getWatchConfigById(watchId);
            UserWatchRelationDO watchRelationDO = userWatchRelationDAO.getExistWatch(userId, targetId, watchId, now);
            if (watchRelationDO != null && watchRelationDO.getUserWatchRelationId() > 0) {
                int increaseWatchValue = totalCost.intValue();
                if (createByEnum.getValue() == UserWatchCreateByEnum.AUCTION.getValue()) {
                    // 这里要设置为 0 , 因为赠送礼物已经在之前相加过了
                    increaseWatchValue = 0;
                }
                userWatchRelationDAO.updateExpireTime(watchRelationDO.getUserWatchRelationId(), increaseNum, increaseWatchValue, now);
                expireTime = watchRelationDO.getExpireTime() + increaseNum;
                isNew = 0;
            } else {
                watchRelationDO = new UserWatchRelationDO();
                watchRelationDO.setUserWatchRelationId(watchRelationId);
                watchRelationDO.setUserId(userId);
                watchRelationDO.setTargetUserId(targetId);
                watchRelationDO.setWatchId(watchId);
                watchRelationDO.setWatchValue(totalCost.intValue());
                watchRelationDO.setCreateBy(createByEnum.getValue());
                watchRelationDO.setOrderId(orderId);
                watchRelationDO.setExpireTime(now + increaseNum);
                watchRelationDO.setWatchName(watchConfigDO.getName());
                watchRelationDO.setCreateTime(now);
                watchRelationDO.setCreateDate(createDateInt);
                watchRelationDO.setUpdateTime(now);
                watchRelationDO.setIsDeleted(BoolIntVal.False);
                userWatchRelationDAO.insert(watchRelationDO);
                expireTime = now + increaseNum;
            }
            // 插入记录
            UserWatchRelationHisDO userWatchRelationHisDO = new UserWatchRelationHisDO();
            userWatchRelationHisDO.setHisId(identityService.getId());
            userWatchRelationHisDO.setUserId(userId);
            userWatchRelationHisDO.setTargetUserId(targetId);
            userWatchRelationHisDO.setWatchId(watchId);
            userWatchRelationHisDO.setWatchName(watchConfigDO.getName());
            userWatchRelationHisDO.setWatchValue(totalCost.intValue());
            userWatchRelationHisDO.setCreateBy(createByEnum.getValue());
            userWatchRelationHisDO.setOrderId(orderId);
            userWatchRelationHisDO.setExpireTime(expireTime);
            userWatchRelationHisDO.setCreateTime(now);
            userWatchRelationHisDO.setIsNew(isNew);
            userWatchRelationHisDO.setCreateDate(createDateInt);
            userWatchRelationHisDAO.insert(userWatchRelationHisDO);
            return expireTime;
        }
    }

    /**
     * 通过钻石开通守护
     *
     * @param watchConfigId
     * @param userId        用户ID 开通的人
     * @param targetId      目标接受者ID
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Result newWatchRelationByDiamond(Long watchConfigId, Long userId, Long targetId) {
        // 这里进行扣费
        WatchConfigDO watchConfigDO = watchConfigDAO.getWatchConfigById(watchConfigId);
        if (watchConfigDO == null) {
            return new Result(GlobalApiError.OperationFailed, "未找到该类型守护");
        }
        Long now = System.currentTimeMillis();
        String userName = userService.getUserName(userId);
        UserBasicInfo userBasicInfo = userService.getBasicInfo(targetId);
        UserWatchRelationDO watchRelationDO = userWatchRelationDAO.getExistWatch(userId, targetId, watchConfigId, now);

        Long watchRelationId;
        boolean isExist = false;
        if (watchRelationDO != null && watchRelationDO.getUserWatchRelationId() > 0) {
            isExist = true;
            watchRelationId = watchRelationDO.getUserWatchRelationId();
        } else {
            watchRelationId = identityService.getId();
        }
        Long orderId = identityService.getId();
        String targetName = userService.getUserName(targetId);
        Long balance = walletService.reduceCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), watchConfigDO.getPrice(), String.format("为%s开通%s守护", targetName, watchConfigDO.getName()), orderId);
        // 主播增加收益
        if (balance == null || balance < 0) {
            return new Result(GlobalApiError.OperationFailed, "资金余额不足");
        }
        BigDecimal rate = BigDecimal.ONE;
        if (StringUtils.isNotBlank(watchConfigDO.getDivideRate())) {
            rate = new BigDecimal(watchConfigDO.getDivideRate()).divide(ONE_HUNDRED);
        }
        BigDecimal benefitIncome = rate.multiply(new BigDecimal(watchConfigDO.getPrice()));
        walletService.addCoin(targetId, WalletTypeEnum.STAR_DUST.getValue(), benefitIncome.longValue(), String.format("%s开通%s守护", userName, watchConfigDO.getName()), watchRelationId);


        long expiredTime = doNewWatchRelation(UserWatchCreateByEnum.DIAMOND, 7, orderId.toString(), watchRelationId, userId, targetId, watchConfigId, watchConfigDO.getPrice());
        // 这里添加亲密度
        increaseIntimacy(UserIntimacyCreateReasonEnum.NEW_WATCH, new BigDecimal(watchConfigDO.getPrice()).divide(ONE_HUNDRED), userId, targetId, orderId.toString());
        // 插入流水
        increaseWatchFlow(UserIntimacyCreateReasonEnum.NEW_WATCH, watchConfigDO.getPrice(), userId, targetId, orderId.toString(), watchConfigId, 0L, 0L);
        Map<String, String> res = new HashMap<>();
        res.put("expireTime", String.valueOf(expiredTime));
        res.put("userName", userBasicInfo.getUserName());
        res.put("avatar", userBasicInfo.getUserAvatar());
        res.put("watchName", watchConfigDO.getName());
        // 最后发送系统消息
        sendMsg(userName, userId, targetId, watchConfigDO.getName(), watchConfigDO.getWatchConfigId(), 7, isExist);
        return new Result(res);
    }


    private void sendMsg(String userName, Long userId, Long targetUserId, String watchName, Long watchId, Integer days, boolean isExist) {
        try {
            // 系统消息
            Long now = System.currentTimeMillis();
            SystemMsg systemMsg = new SystemMsg();
            systemMsg.setContentType(com.starbuds.server.common.constant.message.im.ContentType.TEXT);
            systemMsg.setTitle("守护关系");
            String bodyFormat;
            UserWatchRelationDO existWatch = userWatchRelationDAO.getExistWatch(userId, targetUserId, watchId, now);
            if (isExist) {
                bodyFormat = "【%s】给你续费了【%s】守护，时长%d天。";
            } else {
                bodyFormat = "【%s】给你开通了【%s】守护，时长%d天。";
            }
            String body = String.format(bodyFormat, userName, watchName, days);
            systemMsg.setBody(body);
            systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
            imService.sendCustomImMsg(null, targetUserId, systemMsg, null);
        } catch (Exception e) {
            log.error("出现错误", e);
        }
    }

    /**
     * 通过竞拍开通守护
     */
    @Override
    public void newWatchRelationByAuction(Integer days, Long auctionId, Long userId, Long targetId, Long watchId, Long totalCost) {
        Long watchRelationId;
        Long now = System.currentTimeMillis();
        UserWatchRelationDO watchRelationDO = userWatchRelationDAO.getExistWatch(userId, targetId, watchId, now);
        if (watchRelationDO != null && watchRelationDO.getUserWatchRelationId() > 0) {
            watchRelationId = watchRelationDO.getUserWatchRelationId();
        } else {
            watchRelationId = identityService.getId();
        }
        doNewWatchRelation(UserWatchCreateByEnum.AUCTION, days, auctionId.toString(), watchRelationId, userId, targetId, watchId, totalCost);
    }

    /**
     * 获取即将过期的守护的用户ID用于生成提醒
     */
    @Override
    public List<WatchNotifyHisDO> getLimitWatch(Long userId) {
        return watchNotifyHisDAO.getNotifyWatch(userId);
    }

    /**
     * 获取守护排行
     *
     * @param targetUserId 这里的用户ID是targetId, 因为是被守护对象
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<UserWatchRelationDO> getUserWatchRelation(Long targetUserId, Integer pageIndex, Integer pageSize) {
        PageList<UserWatchRelationDO> res = new PageList<>(pageIndex, pageSize);
        res.setList(userWatchRelationDAO.getUserWatchRelation(targetUserId, System.currentTimeMillis(), res.getPageCursor(), res.getPageSize()));
        res.setTotal(userWatchRelationDAO.getUserWatchRelationCount(targetUserId, System.currentTimeMillis()));
        return res;
    }

    /**
     * 获取亲密度排行
     *
     * @param targetUserId
     * @return
     */
    @Override
    public List<UserIntimacyRelationDO> getUserIntimacyRelation(Long targetUserId) {
        return userIntimacyRelationDAO.getUserIntimacyRelation(targetUserId);
    }


    /**
     * 守护流水, 增加守护值
     *
     * @param reasonEnum
     * @param giftPrice
     * @param userId
     * @param targetId
     * @param orderId
     * @param watchConfigId
     * @param roomId
     * @param auctionId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void increaseWatchValue(UserIntimacyCreateReasonEnum reasonEnum, Long giftPrice, Long userId, Long targetId, String orderId, Long watchConfigId, Long roomId, Long auctionId) {
        UserWatchRelationFlowHisDO flowHisDO = userWatchRelationFlowHisDAO.getHistoryByOrderId(reasonEnum.getValue(), orderId);
        if (flowHisDO != null && flowHisDO.getHisId() > 0) {
            return;
        }
        Long now = System.currentTimeMillis();
        UserWatchRelationDO userWatchRelation = userWatchRelationDAO.getExistWatch(userId, targetId, watchConfigId, now);
        if (userWatchRelation != null && userWatchRelation.getUserWatchRelationId() > 0) {
            // 增加守护值
            userWatchRelationDAO.updateExpireTime(userWatchRelation.getUserWatchRelationId(), 0, giftPrice.intValue(), now);
        }
        increaseWatchFlow(reasonEnum, giftPrice, userId, targetId, orderId, watchConfigId, roomId, auctionId);
    }

    private void increaseWatchFlow(UserIntimacyCreateReasonEnum reasonEnum, Long giftPrice, Long userId, Long targetId, String orderId, Long watchConfigId, Long roomId, Long auctionId) {
        UserWatchRelationFlowHisDO flowHisDO = new UserWatchRelationFlowHisDO();
        Long now = System.currentTimeMillis();
        Integer createDateInt = DateUtil.getIntDate();
        WatchConfigDO watchConfigDO = getWatchConfig(watchConfigId);
        flowHisDO.setHisId(identityService.getId());
        flowHisDO.setUserId(userId);
        flowHisDO.setTargetUserId(targetId);
        flowHisDO.setWatchId(watchConfigId);
        flowHisDO.setWatchName(watchConfigDO.getName());
        flowHisDO.setCostValue(giftPrice);
        flowHisDO.setCreateBy(reasonEnum.getValue());
        flowHisDO.setRoomId(roomId);
        flowHisDO.setOrderId(orderId);
        flowHisDO.setAuctionId(auctionId);
        flowHisDO.setCreateTime(now);
        flowHisDO.setCreateDate(createDateInt);
        userWatchRelationFlowHisDAO.insert(flowHisDO);
    }

    /**
     * 上升亲密度
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void increaseIntimacy(UserIntimacyCreateReasonEnum reasonEnum, BigDecimal increaseValue,
                                 Long userId, Long targetId, String orderId) {
        UserIntimacyRelationHisDO hisDO = userIntimacyRelationHisDAO.getIntimacyHisByOrderId(reasonEnum.getValue(), orderId);
        if (hisDO != null && hisDO.getHisId() > 0) {
            return;
        }
        Long now = System.currentTimeMillis();
        UserIntimacyRelationDO userIntimacy = userIntimacyRelationDAO.getUserIntimacy(userId, targetId);
        if (userIntimacy != null && userIntimacy.getUserIntimacyRelationId() > 0) {
            userIntimacyRelationDAO.increaseUserIntimacyVal(userIntimacy.getUserIntimacyRelationId(), increaseValue, now);
        } else {
            userIntimacy = new UserIntimacyRelationDO();
            userIntimacy.setUserIntimacyRelationId(identityService.getId());
            userIntimacy.setUserId(userId);
            userIntimacy.setTargetId(targetId);
            userIntimacy.setIntimacyValue(increaseValue);
            userIntimacy.setAccompanyMinutes(0);
            userIntimacy.setCreateTime(now);
            userIntimacy.setUpdateTime(now);
            userIntimacyRelationDAO.insert(userIntimacy);
        }
        UserIntimacyRelationHisDO hisDAO = new UserIntimacyRelationHisDO();
        hisDAO.setHisId(identityService.getId());
        hisDAO.setUserId(userId);
        hisDAO.setTargetId(targetId);
        hisDAO.setIncreaseValue(increaseValue);
        hisDAO.setReason(reasonEnum.getValue());
        hisDAO.setOrderId(orderId);
        hisDAO.setCreateTime(now);
        hisDAO.setCreateDate(DateUtil.getIntDate());
        hisDAO.setRemarks(reasonEnum.getDesc());
        userIntimacyRelationHisDAO.insert(hisDAO);
    }

    /**
     * 通过ID获取守护
     *
     * @param watchConfigId
     * @return
     */
    @Override
    public WatchConfigDO getWatchConfig(Long watchConfigId) {
        return watchConfigDAO.getWatchConfigById(watchConfigId);
    }


    /**
     * 构建到期消息
     */
    @Override
    public void buildWatchNotify() {
        // 获取即将到期的关系
        Long now = System.currentTimeMillis();
        Long endTime = now + DAY_OF_MILLION;
        Long startTime = endTime - HOUR_OF_MILLION;
        List<UserWatchRelationDO> expireRelation = userWatchRelationDAO.getExpiredRelation(startTime, endTime);
        if (CollectionUtil.isNotEmpty(expireRelation)) {
            List<WatchNotifyHisDO> res = new ArrayList<>();
            expireRelation.forEach(i -> {
                WatchNotifyHisDO tmp = new WatchNotifyHisDO();
                tmp.setHisId(identityService.getId());
                tmp.setUserId(i.getUserId());
                tmp.setTargetId(i.getTargetUserId());
                tmp.setIsNotified(BoolIntVal.False);
                tmp.setWatchRelationId(i.getUserWatchRelationId());
                tmp.setCreateTime(now);
                tmp.setExpireTime(i.getExpireTime());
                tmp.setWatchId(i.getWatchId());
                tmp.setWatchName(i.getWatchName());
                res.add(tmp);
            });
            watchNotifyHisDAO.insertBatch(res);
        }
    }

    @Override
    public void setWatchNotifyRead(Long userId) {
        watchNotifyHisDAO.updateNotifyWatchRead(userId);
    }

    /**
     * 获取亲密度关系
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public UserIntimacyRelationDO getUserIntimacyRelationByUser(Long userId, Long targetUserId) {
        return userIntimacyRelationDAO.getUserIntimacy(userId, targetUserId);
    }

    /**
     * 记录守护统计Top20, 并把胜利者进行记录
     */
    @Override
    public void buildWatchIntimacyTopHis() {
        // 这里获取的是上个星期的数据进行落地
        Long weekStart = DateUtil.getWeekStart(new Date()) - WEEK_OF_MILLION;
        Long weekEnd = DateUtil.getWeekEnd(new Date()) - WEEK_OF_MILLION;
        // 获取亲密度表周数据前50 因为可能有损耗值, 就是有亲密度但是没有守护
        List<WatchScoreRankHisDO> res = doGetWeekTopWatchIntimacy(weekStart, weekEnd);
        if (CollectionUtil.isNotEmpty(res)) {
            res.forEach(i -> i.setHisId(identityService.getId()));
        }
        if (CollectionUtil.isNotEmpty(res)) {
            watchScoreRankHisDAO.insertBatch(res);
        }
    }


    /**
     * 获取本周排行榜
     *
     * @return
     */
    @Override
    public List<WatchScoreRankHisDO> getWeekTopWatchIntimacy() {
        String str = redisService.get(RedisWatchKeys.WATCH_RANK_DATA);
        if (StringUtils.isNotBlank(str)) {
            return JSON.parseArray(str, WatchScoreRankHisDO.class);
        } else {
            Long weekStart = DateUtil.getWeekStart();
            Long weekEnd = DateUtil.getWeekEnd();
            List<WatchScoreRankHisDO> res = doGetWeekTopWatchIntimacy(weekStart, weekEnd);
            redisService.set(RedisWatchKeys.WATCH_RANK_DATA, JSON.toJSONString(res), 1, TimeUnit.MINUTES);
            return res;
        }
    }

    /**
     * 获取上周排行
     *
     * @return
     */
    @Override
    public List<WatchScoreRankHisDO> getLastWeekTopWatchIntimacy() {
        Long weekStart = DateUtil.getWeekStart(new Date()) - WEEK_OF_MILLION;
        return watchScoreRankHisDAO.getWeekTop(weekStart);
    }

    /**
     * 获取上周冠军
     *
     * @return
     */
    @Override
    public WatchScoreRankHisDO getLastWeekWinner() {
        Long weekStart = DateUtil.getWeekStart(new Date()) - WEEK_OF_MILLION;
        return watchScoreRankHisDAO.getLastWeekWinner(weekStart);
    }

    /**
     * 获取记录
     *
     * @param limit
     * @return
     */
    @Override
    public List<UserWatchRelationDO> getWatchWallHis(Long limit, Integer pageIndex, Integer pageSize) {
        if (pageIndex == null) {
            pageIndex = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        // 最多给200条数据
        if (pageIndex * pageSize > 200) {
            return new ArrayList<>();
        }
        Integer cursor = (pageIndex - 1) * pageSize;
        return userWatchRelationDAO.getHisList(limit, cursor, pageSize);
    }

    private List<WatchScoreRankHisDO> doGetWeekTopWatchIntimacy(Long weekStart, Long weekEnd) {
        List<UserIntimacyRelationDO> watchIntimacyTop = userIntimacyRelationHisDAO.getUserWeekWatchIntimacy(weekStart, weekEnd);
        if (CollectionUtil.isNotEmpty(watchIntimacyTop)) {
            int rank = 1;
            List<WatchScoreRankHisDO> res = new ArrayList<>();
            for (UserIntimacyRelationDO item : watchIntimacyTop) {
                UserWatchRelationDO userWatchRelationDO = getUserTopWatchRelation(item.getUserId(), item.getTargetId(), weekEnd);
                if (userWatchRelationDO != null) {
                    WatchScoreRankHisDO tmp = new WatchScoreRankHisDO();
                    tmp.setUserId(item.getUserId());
                    tmp.setTargetUserId(item.getTargetId());
                    tmp.setWatchId(userWatchRelationDO.getWatchId());
                    tmp.setWatchName(userWatchRelationDO.getWatchName());
                    tmp.setIntimacyValue(item.getIntimacyValue().intValue());
                    tmp.setSort(rank);
                    tmp.setSortStartTime(weekStart);
                    tmp.setSortEntTime(weekEnd);
                    res.add(tmp);
                }
                rank++;
                if (rank > 20) {
                    break;
                }
            }
            return res;
        }
        return new ArrayList<>();
    }

    /**
     * 获取区间时间内守护排行
     *
     * @param startTime
     * @param endTime
     * @param limit     人数限制
     * @return
     */
    @Override
    public List<Long> getDurationRank(Long startTime, Long endTime, Integer limit) {
        List<UserIntimacyRelationDO> watchIntimacyTop = userIntimacyRelationHisDAO.getUserWeekWatchIntimacy(startTime, endTime);
        List<Long> res = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(watchIntimacyTop)) {
            if (watchIntimacyTop.size() < limit) {
                limit = watchIntimacyTop.size();
            }
            for (int i = 0; i < limit; i++) {
                res.add(watchIntimacyTop.get(i).getUserId());
            }
        }
        return res;
    }

    /**
     * 获取历史胜利者
     *
     * @return
     */
    @Override
    public PageList<WatchScoreRankHisDO> getHistoryWinner(Integer pageIndex, Integer pageSize) {
        PageList<WatchScoreRankHisDO> res = new PageList<>(pageIndex, pageSize);
        res.setTotal(watchScoreRankHisDAO.getHistoryWinnerCount());
        res.setList(watchScoreRankHisDAO.getHistoryWinner(res.getPageCursor(), pageSize));
        return res;
    }


    /**
     * 获取用户守护值最高的那个
     *
     * @param userId
     * @param targetId
     * @return
     */
    private UserWatchRelationDO getUserTopWatchRelation(Long userId, Long targetId, Long expireTime) {
        return userWatchRelationDAO.getUserTopWatchRelation(userId, targetId, expireTime);
    }

    /**
     * 获取周排行
     *
     * @param startTime 周一起始时间戳
     * @return
     */
    @Override
    public List<WatchScoreRankHisDO> getWeekTop(Long startTime) {
        return watchScoreRankHisDAO.getWeekTop(startTime);
    }


    /**
     * 获取所有守护的Map
     *
     * @param watchIds
     * @return
     */
    @Override
    public Map<Long, WatchConfigDO> getWatchConfigMap(Set<Long> watchIds) {
        Map<Long, WatchConfigDO> res = new HashMap<>();
        if (CollectionUtil.isNotEmpty(watchIds)) {
            List<WatchConfigDO> watchConfigDOList = watchConfigDAO.getWatchConfigListByIds(watchIds);
            if (CollectionUtil.isNotEmpty(watchConfigDOList)) {
                watchConfigDOList.forEach(i -> res.put(i.getWatchConfigId(), i));
            }
        }
        return res;
    }

    /**
     * 获取所有已开通的守护的ID
     *
     * @param userId
     * @param targetId
     * @return
     */
    @Override
    public Set<Long> getExistConfig(Long userId, Long targetId) {
        Long now = System.currentTimeMillis();
        List<UserWatchRelationDO> userWatchRelationDOList = userWatchRelationDAO.getExistWatchList(userId, targetId, now);
        Set<Long> res = new HashSet<>();
        if (CollectionUtil.isNotEmpty(userWatchRelationDOList)) {
            userWatchRelationDOList.forEach(i -> res.add(i.getWatchId()));
        }
        return res;
    }

    /**
     * 通过竞拍ID获取房间ID
     *
     * @param auctionIds
     * @return
     */
    @Override
    public Map<Long, Long> getRoomIdsByAuctionId(Set<Long> auctionIds) {
        if (CollectionUtil.isNotEmpty(auctionIds)) {
            List<UserWatchRelationFlowHisDO> roomIds = userWatchRelationFlowHisDAO.getRoomIdsByAuctionId(auctionIds);
            if (CollectionUtil.isNotEmpty(roomIds)) {
                Map<Long, Long> res = new HashMap();
                roomIds.forEach(i -> {
                    res.put(i.getAuctionId(), i.getRoomId());
                });
                return res;
            }
        }
        return new HashMap<>();
    }
}
