package com.starbuds.server.service.user.provider;

import cn.hutool.core.util.RandomUtil;
import com.github.javafaker.Faker;
import com.starbuds.server.service.user.dao.UserDAO;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Locale;
import java.util.Random;

/**
 * 随机名字服务
 *
 * @author Lay
 * @date 2020/9/11
 */
@Component
public class ChineseNameProvider implements INameProvider {

    @Resource
    private UserDAO userDAO;

    private static final String[] firstNames = new String[]{
            "唐", "景", "萧", "永", "爱", "宁", "蓝", "楼", "步", "彩", "卿", "连", "颜", "墨", "凌", "苏", "洛", "冷", "奈", "白", "沐", "子书", "欧阳", "端木", "上官", "司马", "东方", "独孤", "南宫", "万俟", "闻人", "夏侯", "诸葛", "尉迟", "公羊", "赫连", "澹台", "皇甫", "宗政", "濮阳", "公冶", "太叔", "申屠", "公孙", "慕容", "仲孙", "钟离", "长孙", "拓跋", "轩辕", "百里", "呼延", "东郭", "南门", "微生", "西门", "仲长", "子书", "子桑", "即墨", "达奚", "淳于", "完颜",
            "秦", "赵", "楚", "魏", "燕", "齐", "韩", "卫", "蔡", "陈", "冉", "曾", "孔", "李", "熊", "胡", "朱", "马", "和", "武", "童", "令狐", "公输", "公西", "荔菲", "栎阳", "梁丘", "梁由", "梁馀", "梁垣", "陵阳", "伶舟", "柳下", "龙丘", "卢蒲", "鲁步", "陆费", "甪里", "闾丘", "答禄", "达勃", "达步", "达奚", "淡台", "邓陵", "第五", "地连", "地伦", "东里", "房南", "东宫", "东门", "东乡", "东丹", "东郭", "东陵", "东关", "东闾", "东阳", "东野", "东莱", "豆卢", "斗于", "都尉",
            "凡闾", "范师", "范姜", "方雷", "方叔", "方吾", "方相", "房当", "防风", "蜚廉", "飞廉", "飞龙", "费莫", "费连", "费听", "费羽", "棼冒", "丰将", "封贝", "风胡", "夫蒙", "夫余", "佛图", "扶余", "扶登", "浮丘", "弗羽", "甫奚", "甫爽", "傅余", "傅其", "富父", "富宜", "复生", "复陆", "复隆", "付里", "干已", "高车", "高陵", "高堂", "高阳", "高辛", "皋落", "哥舒", "盖楼", "庚桑", "梗阳", "宫孙", "公羊", "公良", "公孙", "公罔", "公冶", "公敛", "公梁", "公山", "公户", "公玉",
            "公仪", "公仲", "公坚", "公伯", "公祖", "公乘", "公晰", "公族", "姑布", "古口", "古龙", "古孙", "谷梁", "谷浑", "瓜田", "关龙", "毌丘", "鲑阳", "漆雕", "奇介", "綦母", "綦毋", "綦连", "祁连", "乞伏", "绮里", "千代", "千乘", "青阳", "丘丽", "丘陵", "屈侯", "屈突", "屈男", "屈卢", "屈同", "屈门", "屈引"
    };

    private static final String[] symbols = new String[]{
            "あ", "い", "え", "うた", "かち", "き", "け", "こ", "さし", "すせ", "に", "のね", "はひ", "ふへ", "ほ", "まみ", "むや", "ゆよ", "ら", "るれ", "ろを", "가が", "갸ぎゃ", "거ご", "겨ぎょ", "고ご", "교ぎょ", "규ぎ", "기ぎ", "나な", "냐に", "너の", "녀にょ", "노の", "뇨にょ", "누ぬ", "뉴に", "니に", "다だ", "댜ぢゃ", "더ど", "뎌ぢょ", "도ど", "됴ぢょ", "두づ", "듀ぢゅ", "라ら",
            "랴りゃ", "러ろ", "려りょ", "로ろ", "료りょ", "루る", "부ぶ", "뷰び", "비び", "사さ", "샤しゃ", "서そ", "셔しょ", "소そ", "쇼しょ", "수す", "슈しゅ", "스す", "시し"
    };

    @Override
    public String get() {
        return randomName();
    }

    @Override
    public String getRandomUserName() {
        String userName = get();
        Integer count = userDAO.selectExistUserName(userName);
        if (count == null) {
            return userName;
        }

        Random random = new Random();
        userName = String.format("%s%d", userName, random.nextInt(599) + 1);
        return userName;
    }

    private static String randomName() {
        Faker faker = new Faker(Locale.CHINA);
        String firstName = firstNames[RandomUtil.randomInt(0, firstNames.length)];
        String lastName = faker.name().firstName();
        return firstName + lastName;
    }
}
