package com.starbuds.server.service.user.provider;

import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.pojo.user.ChannelAppConfig;
import com.starbuds.server.service.user.dao.ChannelDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/2/3 12:51
 * @Description oauth配置
 */
@Slf4j
@Component
public class OauthConfigProvider {

    @Autowired
    ChannelDAO channelDAO;

    /**
     * 获取微信oauth配置
     *
     * @param channelId
     */
    public ChannelAppConfig getWxAppConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectWxAppOAuthConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectWxAppOAuthConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig wxAppConfig = new ChannelAppConfig();
            wxAppConfig.setConfigId(channelDO.getWxAppOAuthConfigId());
            return wxAppConfig;
        }
        return null;
    }

    /**
     * 获取 qq oauth配置
     *
     * @param channelId
     * @return
     */
    public ChannelAppConfig getQqConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectQqConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectQqConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig wxAppConfig = new ChannelAppConfig();
            wxAppConfig.setConfigId(channelDO.getQqOAuthConfigId());
            return wxAppConfig;
        }
        return null;
    }

    /**
     * 获取微博oauth配置
     *
     * @param channelId
     * @return
     */
    public ChannelAppConfig getWeiboConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectWeiboConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectWeiboConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig wxAppConfig = new ChannelAppConfig();
            wxAppConfig.setConfigId(channelDO.getWbOAuthConfigId());
            return wxAppConfig;
        }
        return null;
    }

    /**
     * 获取华为 OAuth 配置
     *
     * @param channelId
     * @return
     */
    public ChannelAppConfig getHuaweiConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectHuaweiConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectHuaweiConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig wxAppConfig = new ChannelAppConfig();
            wxAppConfig.setConfigId(channelDO.getHwOAuthConfigId());
            return wxAppConfig;
        }
        return null;
    }

    /**
     * 获取facebook oauth配置
     *
     * @param channelId
     * @return
     */
    public ChannelAppConfig getFacebookConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectFacebookConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectFacebookConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig fbAppConfig = new ChannelAppConfig();
            fbAppConfig.setConfigId(channelDO.getFbOAuthConfigId());
            return fbAppConfig;
        }
        return null;
    }

    /**
     * 获取google oauth配置
     *
     * @param channelId
     * @return
     */
    public ChannelAppConfig getGoogleConfig(Long channelId) {
        ChannelDO channelDO = channelDAO.selectGoogleConfig(Optional.ofNullable(channelId).orElse(0L));
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectGoogleConfig(0L);
        }
        if (channelDO != null) {
            ChannelAppConfig fbAppConfig = new ChannelAppConfig();
            fbAppConfig.setConfigId(channelDO.getGoOAuthConfigId());
            return fbAppConfig;
        }
        return null;
    }

}
