package com.starbuds.server.service.user.provider;

import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.pojo.user.ChannelOneKeyLoginConfig;
import com.starbuds.server.service.user.dao.ChannelDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/2/3 12:41
 * @Description 一键登录配置提供者
 */
@Slf4j
@Component
public class OneKeyLoginConfigProvider {

    @Autowired
    ChannelDAO channelDAO;

    /**
     * 获取渠道一键登录配置
     *
     * @param channelId
     * @param clientType
     * @return
     */
    public ChannelOneKeyLoginConfig getOneKeyLoginConfig(Long channelId, Integer clientType) {
        ChannelDO channelDO = channelDAO.selectOneKeyLoginConfig(Optional.ofNullable(channelId).orElse(0L), clientType);
        if (channelDO == null) {
            // 返回全局默认渠道的配置
            channelDO = channelDAO.selectOneKeyLoginConfig(0L, clientType);
        }
        if (channelDO != null) {
            ChannelOneKeyLoginConfig loginConfig = new ChannelOneKeyLoginConfig();
            loginConfig.setAgentId(channelDO.getOneKeyLoginAgentId());
            return loginConfig;
        }
        return null;
    }


}
