package com.starbuds.server.service.user.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.user.OneKeyLoginEnum;
import com.starbuds.server.common.pojo.data.OneKeyLoginAgentConfig;
import com.starbuds.server.common.pojo.user.ChannelOneKeyLoginConfig;
import com.starbuds.server.common.service.data.OneKeyLoginConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/5 15:11
 * @Description 一键登录业务实现
 */
@Slf4j
@Component
public class OneKeyLoginProvider implements IOneKeyLoginProvider {

    @Autowired
    @Qualifier(value = "yuyanOneKeyLoginProvider")
    AbstractOneKeyLoginProvider yuyanOneKeyLoginProvider;

    @Autowired
    @Qualifier(value = "shanyanOneKeyLoginProvider")
    AbstractOneKeyLoginProvider shanyanOneKeyLoginProvider;

    @Autowired
    @Qualifier(value = "aliyunOneKeyLoginProvider")
    AbstractOneKeyLoginProvider aliyunOneKeyLoginProvider;

    @Reference(version = "1.0.0")
    OneKeyLoginConfigService oneKeyLoginConfigService;

    @Autowired
    OneKeyLoginConfigProvider oneKeyLoginConfigProvider;

    /**
     * 获取手机号
     *
     * @param channelId  渠道id
     * @param token      token
     * @param clientType 客户端类型
     * @return 手机号
     */
    @Override
    public String getMobile(Long channelId, String token, Integer clientType) {
        ChannelOneKeyLoginConfig config = oneKeyLoginConfigProvider.getOneKeyLoginConfig(channelId, clientType);
        if (config == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{}", channelId);
            return null;
        }
        OneKeyLoginAgentConfig agentConfig = oneKeyLoginConfigService.getOneKeyLoginAgentConfig(config.getAgentId());
        log.info("一键登录服务商详情：{}", JSON.toJSONString(agentConfig));
        if (agentConfig == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{}", channelId);
            return null;
        }
        OneKeyLoginEnum oneKey = OneKeyLoginEnum.toEnum(agentConfig.getAgentName());
        if (oneKey == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{},agentName:{}", channelId, agentConfig.getAgentName());
            return null;
        }

        ClientType type = ClientType.toEnum(clientType);
        String accessKey = "";
        String accessSecret = "";
        switch (type) {
            case Android:
                log.info("客户端类型为：安卓");
                accessKey = agentConfig.getAndroidAccessKey();
                accessSecret = agentConfig.getAndroidAccessSecret();
                break;
            case iOS:
                log.info("客户端类型为：ios");
                accessKey = agentConfig.getIosAccessKey();
                accessSecret = agentConfig.getIosAccessSecret();
                break;
            default:
                log.error("暂不支持该客户端一键登录");
                break;
        }

        switch (oneKey) {
            case YU_YAN:
                return yuyanOneKeyLoginProvider.getMobile(token, accessKey, accessSecret);
            case SHAN_YAN:
                return shanyanOneKeyLoginProvider.getMobile(token, accessKey, accessSecret);
            case ALIYUN:
                return aliyunOneKeyLoginProvider.getMobile(token, accessKey, accessSecret);
            default:
                return null;
        }
    }

    @Override
    public Boolean verifyMobile(Long channelId, String token, String mobile, Integer clientType) {
        ChannelOneKeyLoginConfig config = oneKeyLoginConfigProvider.getOneKeyLoginConfig(channelId, clientType);
        if (config == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{}", channelId);
            return null;
        }
        OneKeyLoginAgentConfig agentConfig = oneKeyLoginConfigService.getOneKeyLoginAgentConfig(config.getAgentId());
        if (agentConfig == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{}", channelId);
            return null;
        }
        OneKeyLoginEnum oneKey = OneKeyLoginEnum.toEnum(agentConfig.getAgentName());
        if (oneKey == null) {
            log.info("该渠道尚未配置一键登录服务商,channelId:{}", channelId);
            return null;
        }

        ClientType type = ClientType.toEnum(clientType);
        String accessKey = "";
        String accessSecret = "";
        switch (type) {
            case Android:
                accessKey = agentConfig.getAndroidAccessKey();
                accessSecret = agentConfig.getAndroidAccessSecret();
                break;
            case iOS:
                accessKey = agentConfig.getIosAccessKey();
                accessSecret = agentConfig.getIosAccessSecret();
                break;
            default:
                log.error("暂不支持该客户端一键登录");
                break;
        }

        switch (oneKey) {
            case YU_YAN:
                return yuyanOneKeyLoginProvider.verifyMobile(token, mobile, accessKey, accessSecret);
            case SHAN_YAN:
                return shanyanOneKeyLoginProvider.verifyMobile(token, mobile, accessKey, accessSecret);
            case ALIYUN:
                return aliyunOneKeyLoginProvider.verifyMobile(token, mobile, accessKey, accessSecret);
            default:
                return false;
        }
    }
}
