package com.starbuds.server.service.user.provider;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.service.data.RedisService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.Calendar;
import java.util.Optional;

/**
 * 用户关注限制服务
 *
 * @author Lay
 * @date 2021/4/7
 */
@Component
public class UserFollowLimitProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    public boolean limited(Long userId, int wealthLevel) {
        long curVal = Optional.ofNullable(redisService.hashGet(RedisUserKeys.USER_DAILY_FOLLOW_COUNT, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        if (curVal >= wealthLevel * 2L) {
            return true;
        } else {
            if (Optional.ofNullable(redisService.hasKey(RedisUserKeys.USER_DAILY_FOLLOW_COUNT)).orElse(false)) {
                redisService.expire(RedisUserKeys.USER_DAILY_FOLLOW_COUNT, DateUtil.endOfDay(Calendar.getInstance()).getTimeInMillis() - System.currentTimeMillis());
            }
            return false;
        }
    }

    public Long increase(Long userId) {
        return redisService.hashIncreaseLong(RedisUserKeys.USER_DAILY_FOLLOW_COUNT, String.valueOf(userId), 1);
    }
}
