package com.starbuds.server.service.user.provider;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.user.UserMedalTypeEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.UserMedalShinyStatusDO;
import com.starbuds.server.service.user.dao.UserMedalShinyStatusDAO;
import com.starbuds.server.service.user.provider.impl.UserMedalShinyDate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 用户勋章闪亮状态
 */
@Component
@Slf4j
public class UserMedalShinyStatusProvider {

    @Resource
    private UserMedalShinyStatusDAO userMedalShinyStatusDAO;

    @Resource
    private UserMedalShinyDate userMedalShinyDate;

    private String PRIMARY_KEY = "hisId";

    /**
     * 保存勋章闪亮状态
     *
     * @param userId
     * @param userMedalType
     * @return null 保存失败
     */
    public UserMedalShinyStatusDO saveShinyStatus(long userId, UserMedalTypeEnum userMedalType) {
        Preconditions.checkNotNull(userMedalType, "userMedalType is null");
        UserMedalShinyStatusDO shinyStatus = userMedalShinyStatusDAO.selectByUserIdMedalType(userId, userMedalType.value);
        if (shinyStatus == null) {
            return create(userId, userMedalType);
        }
        return updateToShiny(shinyStatus.getHisId());
    }

    private UserMedalShinyStatusDO create(long userId, UserMedalTypeEnum userMedalType) {
        UserMedalShinyStatusDO po = new UserMedalShinyStatusDO();
        po.setUserId(userId);
        po.setUserMedalType(userMedalType.value);
        putShinyDate(po);
        po.setSendMsgStatus(BoolIntVal.False);
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = userMedalShinyStatusDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("用户勋章闪亮状态创建成功 {}", json);
            return po;
        }
        log.error("用户勋章闪亮状态创建失败 {}", json);
        return null;
    }

    private UserMedalShinyStatusDO updateToShiny(long hisId) {
        UserMedalShinyStatusDO po = new UserMedalShinyStatusDO();
        po.setHisId(hisId);
        po.setUpdateTime(System.currentTimeMillis());
        putShinyDate(po);
        po.setSendMsgStatus(BoolIntVal.False);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        boolean update = update(uw);

        String json = JSON.toJSONString(po);
        if (update) {
            log.info("用户勋章闪亮状态更新成功 {}", json);
            return po;
        }
        log.error("用户勋章闪亮状态更新失败 {}", json);
        return null;
    }

    private boolean update(UpdateWrapper uw) {
        int update = userMedalShinyStatusDAO.update(uw);
        boolean flag = update > 0;
        if (isDev()) {
            log.info("更新勋章闪亮状态 update={} flag={} {}", update, flag, JSON.toJSONString(uw, true));
        }
        return flag;
    }

    private void putShinyDate(UserMedalShinyStatusDO po) {
        final LocalDate now = LocalDate.now();
        po.setLastUpdatedDate(userMedalShinyDate.getLastUpdatedDate(now));
        po.setShinyEndDate(userMedalShinyDate.getShinyEndDate(now));
        po.setPutGreyDate(userMedalShinyDate.getPutGreyDate(now));
    }
}
