package com.starbuds.server.service.user.provider.impl;

import com.alibaba.fastjson.JSON;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dypnsapi.model.v20170525.GetMobileRequest;
import com.aliyuncs.dypnsapi.model.v20170525.GetMobileResponse;
import com.aliyuncs.dypnsapi.model.v20170525.VerifyMobileRequest;
import com.aliyuncs.dypnsapi.model.v20170525.VerifyMobileResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.profile.DefaultProfile;
import com.starbuds.server.service.user.provider.AbstractOneKeyLoginProvider;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/4 20:45
 * @Description 阿里云一键登录业务
 */
@Slf4j
@Component
public class AliyunOneKeyLoginProvider implements AbstractOneKeyLoginProvider {

    @Override
    public String getMobile(String token, String accessKey, String accessSecret) {
        log.info("入参：token{},accessKey{},accessSecret{}", token, accessKey, accessSecret);
        if (StringUtil.isNullOrEmpty(token)) {
            log.info("SDK token不能为空");
            return null;
        }
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKey, accessSecret);
        IAcsClient client = new DefaultAcsClient(profile);
        GetMobileRequest request = new GetMobileRequest();
        request.setRegionId("cn-hangzhou");
        try {
            GetMobileResponse response = client.getAcsResponse(request);
            log.info("响应结果：{}", JSON.toJSONString(response));
            if (response != null && response.getGetMobileResultDTO() != null) {
                GetMobileResponse.GetMobileResultDTO resultDTO = response.getGetMobileResultDTO();
                return resultDTO.getMobile();
            }
        } catch (ServerException e) {
            log.info("阿里云服务异常");
        } catch (ClientException e) {
            log.info("ErrCode:" + e.getErrCode());
            log.info("ErrMsg:" + e.getErrMsg());
            log.info("RequestId:" + e.getRequestId());
        }
        return "";
    }

    @Override
    public Boolean verifyMobile(String token, String mobile, String accessKey, String accessSecret) {
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKey, accessSecret);
        IAcsClient client = new DefaultAcsClient(profile);
        VerifyMobileRequest request = new VerifyMobileRequest();
        request.setRegionId("cn-hangzhou");
        try {
            VerifyMobileResponse response = client.getAcsResponse(request);
            log.info("响应结果：{}", JSON.toJSONString(response));
            if (response != null && response.getGateVerifyResultDTO() != null) {
                VerifyMobileResponse.GateVerifyResultDTO resultDTO = response.getGateVerifyResultDTO();
                if ("PASS".equals(resultDTO.getVerifyResult())) {
                    return true;
                }
            }
        } catch (ServerException e) {
            log.info("阿里云服务异常");
        } catch (ClientException e) {
            log.info("ErrCode:" + e.getErrCode());
            log.info("ErrMsg:" + e.getErrMsg());
            log.info("RequestId:" + e.getRequestId());
        }
        return false;
    }


}
