package com.starbuds.server.service.user.provider.impl;

import cn.hutool.crypto.Mode;
import cn.hutool.crypto.Padding;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import cn.hutool.crypto.digest.HMac;
import cn.hutool.crypto.digest.HmacAlgorithm;
import cn.hutool.crypto.symmetric.AES;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.user.constant.StringConstant;
import com.starbuds.server.service.user.provider.AbstractOneKeyLoginProvider;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/4 20:45
 * @Description 闪验一键登录业务
 */
@Slf4j
@Component
public class ShanyanOneKeyLoginProvider implements AbstractOneKeyLoginProvider {

    /**
     * 私钥
     */
    private static final String privateKey = "";

    @Value("${oneKeyLogin.shanyan.mobileQuery}")
    private String mobileQueryUrl;

    @Value("${oneKeyLogin.shanyan.mobileValidate}")
    private String mobileValidateUrl;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Override
    public String getMobile(String token, String accessKey, String accessSecret) {
        log.info("入参：token{},accessKey{},accessSecret{}", token, accessKey, accessSecret);
        if (StringUtil.isNullOrEmpty(token)) {
            log.info("SDK token不能为空");
            return null;
        }
        String appId = accessKey;
        String appKey = accessSecret;
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("token", token);
        requestMap.put("appId", appId);
        String encryptType = StringConstant.ZERO;
        requestMap.put("encryptType", encryptType);
        requestMap.put("outId", identityService.getId().toString());
        requestMap.put("sign", new HMac(HmacAlgorithm.HmacSHA256, appKey.getBytes()).digestHex(request2Str(requestMap)));
        String response = "";
        log.info("调用闪验服务入参：{}", JSON.toJSONString(requestMap));
        try {
            response = HttpUtil.post(mobileQueryUrl, requestMap);
            log.info("调用闪验服务响应数据：{}", JSON.toJSONString(response));
        } catch (Exception e) {
            log.info("闪验查询手机号出错", e);
            return null;
        }
        if (StringUtil.isNullOrEmpty(response)) {
            log.info("闪验服务无响应");
            return null;
        }
        JSONObject jsonObject = JSON.parseObject(response);
        String code = jsonObject.getString("code");
        String message = jsonObject.getString("message");
        if (!StringConstant.TWO_HUNDRED_THOUSAND.equals(code)) {
            log.info("code:{},message:{}", code, message);
            return null;
        }
        String dataStr = jsonObject.getString("data");
        JSONObject dataObj = JSONObject.parseObject(dataStr);
        String mobile = dataObj.getString("mobileName");
        try {
            if (StringConstant.ZERO.equals(encryptType)) {
                String key = SecureUtil.md5(appKey);
                mobile = new AES(Mode.CBC, Padding.PKCS5Padding, key.substring(0, 16).getBytes(), key.substring(16).getBytes()).decryptStr(mobile);
            } else if (StringConstant.ONE.equals(encryptType)) {
                mobile = new RSA("RSA", privateKey, null).decryptStr(mobile, KeyType.PrivateKey);
            }
            return mobile;
        } catch (Exception e) {
            log.info("解密出错", e);
            return null;
        }
    }

    @Override
    public Boolean verifyMobile(String token, String mobile, String accessKey, String accessSecret) {
        String appId = accessKey;
        String appKey = accessSecret;
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put("token", token);
        requestMap.put("appId", appId);
        requestMap.put("mobile", mobile);
        requestMap.put("sign", new HMac(HmacAlgorithm.HmacSHA256, appKey.getBytes()).digestHex(request2Str(requestMap)));
        String response = "";
        try {
            response = HttpUtil.post(mobileValidateUrl, requestMap);
        } catch (Exception e) {
            log.info("调用闪验服务出错", e);
            return false;
        }
        if (StringUtil.isNullOrEmpty(response)) {
            log.info("闪验服务无响应");
            return false;
        }
        JSONObject jsonObject = JSON.parseObject(response);
        String code = jsonObject.getString("code");
        String message = jsonObject.getString("message");
        if (!StringConstant.TWO_HUNDRED_THOUSAND.equals(code)) {
            log.info(message);
            return false;
        }
        String dataStr = jsonObject.getString("data");
        JSONObject dataObj = JSONObject.parseObject(dataStr);
        String isVerify = dataObj.getString("isVerify");
        if (StringConstant.ZERO.equals(isVerify)) {
            log.info("手机号码校验不通过");
            return false;
        } else if (StringConstant.ONE.equals(isVerify)) {
            return true;
        }
        return false;
    }

    private static String request2Str(Map<String, Object> requestMap) {
        String[] keys = requestMap.keySet().toArray(new String[0]);
        Arrays.sort(keys);
        StringBuilder stringBuilder = new StringBuilder();
        for (String str : keys) {
            if (!str.equals("sign")) {
                stringBuilder.append(str).append(requestMap.get(str));
            }
        }
        return stringBuilder.toString();
    }

}
