package com.starbuds.server.service.user.provider.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 用户勋章闪亮状态时间计算
 *
 * @author aoe
 * @date 2022/1/14
 */
@Component
@Slf4j
public class UserMedalShinyDate {

    /**
     * 勋章闪亮有效天数
     */
    public final static int SHINY_VALID_DAYS = 10;

    private final static DateTimeFormatter DTF_YMD = DateTimeFormatter.ofPattern("yyyyMMdd");

    /**
     * 最后一次更新截止日期的日期（年月日）
     *
     * @param now
     * @return
     */
    public int getLastUpdatedDate(LocalDate now) {
        String ymd = now.format(DTF_YMD);
        return NumberUtils.toInt(ymd);
    }

    /**
     * 勋章闪亮截止日期（年月日））
     *
     * @param now
     * @return
     */
    public int getShinyEndDate(LocalDate now) {
        LocalDate shinyEndLocalDate = getShinyEndLocalDate(now);
        String ymd = shinyEndLocalDate.format(DTF_YMD);
        return NumberUtils.toInt(ymd);
    }

    /**
     * 勋章置灰日期（年月日）
     *
     * @param now
     * @return
     */
    public int getPutGreyDate(LocalDate now) {
        LocalDate shinyEndLocalDate = getShinyEndLocalDate(now);
        LocalDate putGreyDate = shinyEndLocalDate.plusDays(1);
        String ymd = putGreyDate.format(DTF_YMD);
        return NumberUtils.toInt(ymd);
    }

    public int getToday(LocalDate now) {
        String ymd = now.format(DTF_YMD);
        return NumberUtils.toInt(ymd);
    }

    public boolean greaterThanShinyEndDate(LocalDate now, int shinyEndDateByDB) {
        int shinyEndDate = getShinyEndDate(now);
        return shinyEndDate > shinyEndDateByDB;
    }

    private LocalDate getShinyEndLocalDate(LocalDate now) {
        return now.plusDays(SHINY_VALID_DAYS);
    }
}
