package com.starbuds.server.service.user.provider.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.service.user.constant.StringConstant;
import com.starbuds.server.service.user.provider.AbstractOneKeyLoginProvider;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/4 20:44
 * @Description 雨燕一键登录业务
 */
@Slf4j
@Component
public class YuyanOneKeyLoginProvider implements AbstractOneKeyLoginProvider {

    @Value("${oneKeyLogin.yuyan.getMobile}")
    private String getMobileUrl;

    @Value("${oneKeyLogin.yuyan.verifyMobile}")
    private String verifyMobileUrl;

    @Override
    public String getMobile(String token, String accessKey, String accessSecret) {
        if (StringUtil.isNullOrEmpty(token)) {
            log.info("SDK凭证不能为空");
            return null;
        }
        String accessId = accessKey;
        String nonce = RandomUtil.randomString(32);
        long ts = System.currentTimeMillis();
        Map<String, Object> params = new HashMap<>();
        params.put("accessId", accessId);
        params.put("ts", ts);
        params.put("nonce", nonce);
        params.put("certificate", token);
        params.put("sign", getSign(accessId, nonce, ts, accessSecret));
        String response = null;
        try {
            response = HttpUtil.post(getMobileUrl, JSON.toJSONString(params));
        } catch (Exception e) {
            log.info("调用雨燕服务出错", e);
            return null;
        }
        if (StringUtil.isNullOrEmpty(response)) {
            log.info("雨燕服务无响应");
            return null;
        }
        JSONObject jsonObject = JSON.parseObject(response);
        String code = jsonObject.getString("code");
        String data = jsonObject.getString("data");
        String msg = jsonObject.getString("msg");
        if (!StringConstant.OK.equals(code)) {
            log.info(msg);
            return null;
        }
        return SecureUtil.aes(accessSecret.getBytes()).decryptStr(data);
    }

    @Override
    public Boolean verifyMobile(String token, String mobile, String accessKey, String accessSecret) {
        if (StringUtil.isNullOrEmpty(token)) {
            log.info("SDK凭证不能为空");
            return false;
        }
        String accessId = accessKey;
        String nonce = RandomUtil.randomString(32);
        long ts = System.currentTimeMillis();
        Map<String, Object> params = new HashMap<>();
        params.put("accessId", accessId);
        params.put("ts", ts);
        params.put("nonce", nonce);
        params.put("certificate", token);
        params.put("sign", getSign(accessId, nonce, ts, accessSecret));
        String response = HttpUtil.post(verifyMobileUrl, JSON.toJSONString(params));
        if (StringUtil.isNullOrEmpty(response)) {
            log.info("雨燕服务无响应");
            return false;
        }
        JSONObject jsonObject = JSON.parseObject(response);
        String code = jsonObject.getString("code");
        String data = jsonObject.getString("data");
        String msg = jsonObject.getString("msg");
        if (!StringConstant.OK.equals(code) || !StringConstant.PASS.equals(SecureUtil.aes(accessSecret.getBytes()).decryptStr(data))) {
            log.info(msg);
            return false;
        }
        return true;
    }

    /**
     * 获取签名
     */
    private String getSign(String accessId, String nonce, long ts, String accessSecret) {
        String text = String.format("accessId=%s&nonce=%s&ts=%d&key=%s", accessId, nonce, ts, accessSecret);
        return SecureUtil.sha256(text);
    }

}
