package com.starbuds.server.service.user.rocket;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.PropertyValueConst;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.service.user.rocket.listener.DelayMsgListener;
import com.starbuds.server.service.user.rocket.listener.NormalMsgListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Class
 *
 * @author Lay
 * @date 2019/05/30
 */
@Component
@Slf4j
public class RocketConsumer implements DisposableBean {
    @Resource
    RocketConfig rocketConfig;

    @Resource
    NormalMsgListener normalMsgListener;

    @Resource
    DelayMsgListener delayMsgListener;

    private Consumer consumer;

    @Override
    public void destroy() throws Exception {
        if (consumer != null) {
            consumer.shutdown();
        }
    }

    public void start() {
        log.info("RocketMQ 消费者初始化");
        Properties properties = rocketConfig.getProps();

        // 集群模式订阅，共同分担消息处理
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.CLUSTERING);
        consumer = ONSFactory.createConsumer(properties);

        log.info("RocketMQ 消费者启动");
        consumer.subscribe(RocketTopic.TPC_NORMAL, getSubscribeTags(Arrays.asList(
                RocketTag.GIFT_PAID,
                RocketTag.USER_CREATE,
                RocketTag.USER_CREATE_TO_AD,
                RocketTag.USER_FIRST_RECHARGE,
                RocketTag.USER_EDIT,
                RocketTag.SCORE_LEVEL_UP,
                RocketTag.NOBLE_NEW,
                RocketTag.NOBLE_RENEW,
                RocketTag.NOBLE_UPGRADE,
                RocketTag.DEVICE_CONNECT,
                RocketTag.RTC_GIFT_PAID,
                RocketTag.RTC_UNDER_MAI_GIFT_PAID,
                RocketTag.VOICE_GIFT_PAID,
                RocketTag.PRIVATE_CHAT_GIFT_PAID,
                RocketTag.ID_CARD_ADD,
                RocketTag.USER_ONLINE_CHANGE,
                RocketTag.RTC_AUCTION_END,
                RocketTag.RTC_AUCTION_BID,
                RocketTag.LOTTERY_GAME_COMPLETE,
                RocketTag.SEND_GIFT_SNAPSHOT
        )), normalMsgListener);
        consumer.subscribe(RocketTopic.TPC_DELAY, getSubscribeTags(Arrays.asList(
                RocketTag.NICE_NO_EXPIRE,
                RocketTag.NOBLE_EXPIRE
        )), delayMsgListener);
        consumer.start();
    }

    private String getSubscribeTags(List<String> tags) {
        int size = tags.size();
        StringBuilder sbTags = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sbTags.append(tags.get(i));
            if (i != size - 1) {
                sbTags.append("||");
            }
        }
        return sbTags.toString();
    }
}
