package com.starbuds.server.service.user.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.starbuds.server.common.constant.activity.TriggerEventEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.data.BaiduATypeEnum;
import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.data.DataMsgHitBussinessEnum;
import com.starbuds.server.common.constant.data.GreetTypeEnum;
import com.starbuds.server.common.constant.finance.LotteryPayTypeEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rtc.RtcAuctionStatus;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatUserTypeEnum;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.data.BaiduNoticeDTO;
import com.starbuds.server.common.pojo.data.IpLocation;
import com.starbuds.server.common.pojo.data.OcrResultVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.ImUserProfile;
import com.starbuds.server.common.pojo.message.im.activity.ActivityMedalLevelUpLevelMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveNobleNoticeMsg;
import com.starbuds.server.common.pojo.message.im.user.ScoreLevelUpMsg;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.user.NoticeUserRegistered;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.LotteryActivityService;
import com.starbuds.server.common.service.activity.TriggerService;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.finance.CloudAccountService;
import com.starbuds.server.common.service.finance.OcrService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.statistics.StatUserService;
import com.starbuds.server.common.service.user.*;
import com.starbuds.server.common.utils.UserUtil;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.LiveUserProvider;
import com.starbuds.server.service.user.provider.ScoreProvider;
import com.starbuds.server.service.user.provider.ScoreRankProvider;
import com.starbuds.server.service.user.rocket.RocketConfig;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/3
 */
@Component
@Slf4j
public class NormalMsgHandler {

    //    2021.12.30 12:00-2022.01.21 24:00
    private final long startGrowUpTime = 1640836800000L;
    private final long endGrowUpTime = 1642780799999L;
//
//    private final long startGrowUpTime = 1639497599000L;
//    private final long endGrowUpTime = 1671551999000L;

    @Value("${spring.application.name}")
    String applicationName;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    TriggerService triggerService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    StatUserService statUserService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    FileService fileService;

    @Reference(version = "1.0.0")
    ThirdPartyService thirdPartyService;

    @Reference(version = "1.0.0")
    CloudAccountService cloudAccountService;

    @Reference(version = "1.0.0")
    OcrService ocrService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    LotteryActivityService lotteryActivityService;

    @Reference(version = "1.0.0")
    BroadcastHisService broadcastHisService;

    @Resource
    ScoreProvider scoreProvider;

    @Resource
    LiveUserProvider liveUserProvider;

    @Resource
    UserDAO userDAO;

    @Resource
    DeviceDAO deviceDAO;

    @Resource
    ChannelDAO channelDAO;

    @Resource
    UserAuthDAO userAuthDAO;

    @Resource
    UserIdCardDAO userIdCardDAO;

    @Resource
    UserGiftWallDAO userGiftWallDAO;

    @Resource
    AnchorDAO anchorDAO;

    @Resource
    UserGrantAuthDAO userGrantAuthDAO;

    @Resource
    UserForbiddenDAO userForbiddenDAO;

    @Resource
    UserAdHisDAO userAdHisDAO;


    @Resource
    private ScoreRankProvider scoreRankProvider;

    @Resource
    private UserWatchRelationService userWatchRelationService;

    @Resource
    RocketConfig rocketConfig;

    @Reference(version = "1.0.0")
    DataMsgHisService dataMsgHisService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    /**
     * 礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleGiftPaid(QueueGiftPaidMsg msg) {
        // 查询礼物
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 接收人
        UserCardVO receiver = userDAO.selectCardById(msg.getReceiverUserId());

        // 赠送人
        UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());

        // 增加用户财富值
        scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.WEALTH, giftDO.getWealthValue() * msg.getQuantity(), String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), msg.getReceiverUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), msg.getReceiverUserId());

        // 增加主播魅力值
        scoreProvider.addScore(msg.getReceiverUserId(), ScoreTypeEnum.CHARM, giftDO.getCharmValue() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), msg.getReceiverUserId());

        // 礼物墙
        if (BoolIntVal.True == Optional.ofNullable(giftDO.getGiftWall()).orElse(BoolIntVal.False)) {
            // 日期处理
            Long curTime = System.currentTimeMillis();
            Calendar calendar = Calendar.getInstance();
            calendar.setTimeInMillis(curTime);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            Integer dateNum = Convert.toInt(sdf.format(calendar.getTime()));

            UserGiftWallDO userGiftWall = new UserGiftWallDO();
            userGiftWall.setDateNum(dateNum);
            userGiftWall.setUserId(msg.getReceiverUserId());
            userGiftWall.setGiftId(msg.getGiftId());
            userGiftWall.setFirstUserId(msg.getSenderUserId());
            userGiftWall.setFirstQuantity(msg.getQuantity());
            userGiftWall.setFirstTime(curTime);
            userGiftWall.setQuantity(msg.getQuantity());
            userGiftWall.setUpdateTime(curTime);
            userGiftWallDAO.insertOrUpdate(userGiftWall);
        }
    }

    /**
     * RTC 礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleRtcGiftPaid(QueueRtcGiftPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.RTC_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }


        // 查询礼物
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 赠送人
        UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());

        Map<Long, UserCardVO> receiverMap = userDAO.selectNameAndAvatarByIds(new ArrayList<>(msg.getTargetUserSeat().keySet())).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));

        // 日期处理
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(curTime);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer dateNum = Convert.toInt(sdf.format(calendar.getTime()));
        List<UserGiftWallDO> giftWallList = new ArrayList<>();

        for (Long receiverId : msg.getTargetUserSeat().keySet()) {
            UserCardVO receiver = receiverMap.get(receiverId);

            // 礼物变更价值
            long giftChangeValue = giftDO.getWealthValue() * msg.getQuantity();
            // 增加用户财富值
            scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.WEALTH, giftChangeValue, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

            // 增加目标用户星耀值
            scoreProvider.addScore(receiverId, ScoreTypeEnum.STAR, giftDO.getStarValue() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

            long now = System.currentTimeMillis();
            if ((!msg.isFromBag()) && now >= startGrowUpTime && now <= endGrowUpTime) {
                // 增加活动娱乐勋章积分值
                scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.GIFT_SEND_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

                // 增加活动偶像勋章积分值
                scoreProvider.addScore(receiverId, ScoreTypeEnum.GIFT_RECEIVE_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);
            }
            broadcastHisService.insert(msg.getSenderUserId(), receiverId, BroadcastTypeEnum.SEND,
                    giftDO.getStarValue() * msg.getQuantity(), giftDO.getGiftIcon(), msg.getQuantity(), msg.getRoomId(), 0L, "礼物赠送");


            // 礼物墙
            if (BoolIntVal.True == Optional.ofNullable(giftDO.getGiftWall()).orElse(BoolIntVal.False)) {
                UserGiftWallDO userGiftWall = new UserGiftWallDO();
                userGiftWall.setDateNum(dateNum);
                userGiftWall.setUserId(receiverId);
                userGiftWall.setGiftId(msg.getGiftId());
                userGiftWall.setFirstUserId(msg.getSenderUserId());
                userGiftWall.setFirstQuantity(msg.getQuantity());
                userGiftWall.setFirstTime(curTime);
                userGiftWall.setQuantity(msg.getQuantity());
                userGiftWall.setUpdateTime(curTime);
                giftWallList.add(userGiftWall);
            }

            // 增加热爱榜-每天
            if (WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())
                    && giftChangeValue > 0) {
                scoreRankProvider.addHotLoveLeaderboardDay(receiverId, msg.getSenderUserId(), giftChangeValue);
            }
        }

        if (CollectionUtil.isNotEmpty(giftWallList)) {
            userGiftWallDAO.batchInsertOrUpdate(giftWallList);
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);

    }

    /**
     * 语音速配 礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleVoiceGiftPaid(QueueVoiceGiftPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.VOICE_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId;
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        log.info("语音速配 礼物支付消息 {}", JSON.toJSONString(msg));
        // 查询礼物
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 赠送人
        UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());
        UserCardVO receiver = userDAO.selectCardById(msg.getReceiveUserId());

        // 日期处理
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(curTime);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer dateNum = Convert.toInt(sdf.format(calendar.getTime()));
        List<UserGiftWallDO> giftWallList = new ArrayList<>();


        // 礼物变更价值
        long giftChangeValue = giftDO.getWealthValue() * msg.getQuantity();
        // 增加用户财富值
        scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.WEALTH, giftChangeValue, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiver.getUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiver.getUserId());

        // 增加目标用户星耀值
        scoreProvider.addScore(receiver.getUserId(), ScoreTypeEnum.STAR, giftDO.getStarValue() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiver.getUserId());

        long now = System.currentTimeMillis();
        if ((!msg.isFromBag()) && now >= startGrowUpTime && now <= endGrowUpTime) {
            // 增加活动娱乐勋章积分值
            scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.GIFT_SEND_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiver.getUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiver.getUserId());

            // 增加活动偶像勋章积分值
            scoreProvider.addScore(receiver.getUserId(), ScoreTypeEnum.GIFT_RECEIVE_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiver.getUserId());
        }

        // 礼物墙
        if (BoolIntVal.True == Optional.ofNullable(giftDO.getGiftWall()).orElse(BoolIntVal.False)) {
            UserGiftWallDO userGiftWall = new UserGiftWallDO();
            userGiftWall.setDateNum(dateNum);
            userGiftWall.setUserId(receiver.getUserId());
            userGiftWall.setGiftId(msg.getGiftId());
            userGiftWall.setFirstUserId(msg.getSenderUserId());
            userGiftWall.setFirstQuantity(msg.getQuantity());
            userGiftWall.setFirstTime(curTime);
            userGiftWall.setQuantity(msg.getQuantity());
            userGiftWall.setUpdateTime(curTime);
            giftWallList.add(userGiftWall);
        }

        // 增加热爱榜-每天
        if (WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType()) && giftChangeValue > 0) {
            scoreRankProvider.addHotLoveLeaderboardDay(receiver.getUserId(), msg.getSenderUserId(), giftChangeValue);
        }

        if (CollectionUtil.isNotEmpty(giftWallList)) {
            userGiftWallDAO.batchInsertOrUpdate(giftWallList);
        }


        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 处理语聊房麦下送礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleRtcGiftUnderMaiPaid(QueueRtcGiftUnderMaiPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.RTC_UNDER_MAI_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId;
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        try {
            log.info("语聊房麦下送礼物支付消息 {}", JSON.toJSONString(msg));
            // 查询礼物
            GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

            // 赠送人
            UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());

            // 日期处理
            Calendar calendar = Calendar.getInstance();
            calendar.setTimeInMillis(curTime);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            Integer dateNum = Convert.toInt(sdf.format(calendar.getTime()));
            List<UserGiftWallDO> giftWallList = new ArrayList<>();

            msg.getReceiverUsers().forEach(receiver -> {
                Long receiverId = receiver.getUserId();
                // 礼物变更价值
                long giftChangeValue = giftDO.getWealthValue() * msg.getQuantity();
                // 增加用户财富值
                scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.WEALTH, giftChangeValue, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

                // 增加目标用户星耀值
                scoreProvider.addScore(receiverId, ScoreTypeEnum.STAR, giftDO.getStarValue() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

                long now = System.currentTimeMillis();
                if ((!msg.isFromBag()) && now >= startGrowUpTime && now <= endGrowUpTime) {
                    // 增加活动娱乐勋章积分值
                    scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.GIFT_SEND_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

                    // 增加活动偶像勋章积分值
                    scoreProvider.addScore(receiverId, ScoreTypeEnum.GIFT_RECEIVE_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);
                }
                broadcastHisService.insert(msg.getSenderUserId(), receiverId, BroadcastTypeEnum.SEND,
                        giftDO.getStarValue() * msg.getQuantity(), giftDO.getGiftIcon(), msg.getQuantity(), msg.getRoomId(), 0L, "礼物赠送");

                // 礼物墙
                if (BoolIntVal.True == Optional.ofNullable(giftDO.getGiftWall()).orElse(BoolIntVal.False)) {
                    UserGiftWallDO userGiftWall = new UserGiftWallDO();
                    userGiftWall.setDateNum(dateNum);
                    userGiftWall.setUserId(receiverId);
                    userGiftWall.setGiftId(msg.getGiftId());
                    userGiftWall.setFirstUserId(msg.getSenderUserId());
                    userGiftWall.setFirstQuantity(msg.getQuantity());
                    userGiftWall.setFirstTime(curTime);
                    userGiftWall.setQuantity(msg.getQuantity());
                    userGiftWall.setUpdateTime(curTime);
                    giftWallList.add(userGiftWall);
                }

                // 增加热爱榜-每天
                if (WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())
                        && giftChangeValue > 0) {
                    scoreRankProvider.addHotLoveLeaderboardDay(receiverId, msg.getSenderUserId(), giftChangeValue);
                }
            });

            if (CollectionUtil.isNotEmpty(giftWallList)) {
                userGiftWallDAO.batchInsertOrUpdate(giftWallList);
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error(StrUtil.format("语聊房麦下送礼物支付消息处理失败 {}", JSON.toJSONString(msg)), e);
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 处理私聊送礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handlePrivateChatGiftPaid(QueueBaseGiftPaidMsg msg) {

        log.info("私聊送礼物支付消息 {}", JSON.toJSONString(msg));
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.PRIVATE_CHAT_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        try {
            // 查询礼物
            GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

            // 赠送人
            UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());
            // 接收人
            Long receiverId = msg.getReceiveUserId();
            Map<Long, UserCardVO> receiverMap = userDAO.selectNameAndAvatarByIds(Arrays.asList(receiverId))
                    .stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
            ;

            // 日期处理
            Calendar calendar = Calendar.getInstance();
            calendar.setTimeInMillis(curTime);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            Integer dateNum = Convert.toInt(sdf.format(calendar.getTime()));
            List<UserGiftWallDO> giftWallList = new ArrayList<>();

            UserCardVO receiver = receiverMap.get(receiverId);

            // 礼物变更价值
            long giftChangeValue = giftDO.getWealthValue() * msg.getQuantity();
            // 增加用户财富值
            scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.WEALTH, giftChangeValue, String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

            // 增加目标用户星耀值
            scoreProvider.addScore(receiverId, ScoreTypeEnum.STAR, giftDO.getStarValue() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

            long now = System.currentTimeMillis();
            if ((!msg.isFromBag()) && now >= startGrowUpTime && now <= endGrowUpTime) {
                // 增加活动娱乐勋章积分值
                scoreProvider.addScore(msg.getSenderUserId(), ScoreTypeEnum.GIFT_SEND_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("赠送给[%s](%d)%d个%s", receiver.getUserName(), receiverId, msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);

                // 增加活动偶像勋章积分值
                scoreProvider.addScore(receiverId, ScoreTypeEnum.GIFT_RECEIVE_21, giftDO.getGiftPrice() * msg.getQuantity(), String.format("收到[%s](%d)赠送的%d个%s", sender.getUserName(), msg.getSenderUserId(), msg.getQuantity(), giftDO.getGiftName()), msg.getOrderId(), receiverId);
            }


            // 礼物墙
            if (BoolIntVal.True == Optional.ofNullable(giftDO.getGiftWall()).orElse(BoolIntVal.False)) {
                UserGiftWallDO userGiftWall = new UserGiftWallDO();
                userGiftWall.setDateNum(dateNum);
                userGiftWall.setUserId(receiverId);
                userGiftWall.setGiftId(msg.getGiftId());
                userGiftWall.setFirstUserId(msg.getSenderUserId());
                userGiftWall.setFirstQuantity(msg.getQuantity());
                userGiftWall.setFirstTime(curTime);
                userGiftWall.setQuantity(msg.getQuantity());
                userGiftWall.setUpdateTime(curTime);
                giftWallList.add(userGiftWall);
            }

            // 增加热爱榜-每天
            if (WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())
                    && giftChangeValue > 0) {
                scoreRankProvider.addHotLoveLeaderboardDay(receiverId, msg.getSenderUserId(), giftChangeValue);
            }

            if (CollectionUtil.isNotEmpty(giftWallList)) {
                userGiftWallDAO.batchInsertOrUpdate(giftWallList);
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error(StrUtil.format("私聊送礼物支付消息处理失败 {}", JSON.toJSONString(msg)), e);
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 太空旅行，月光宝盒等抽奖活动抽奖完成消息
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleActivityLotteryGameComplete(QueueLotteryGameCompleteMsg msg) {
        log.info("抽奖活动完成:msg:{}", JSON.toJSONString(msg));
        // 读取抽奖配置
        ActivityLotteryDO activityLotteryDO = lotteryActivityService.getLottery(msg.getLotteryId());
        if (activityLotteryDO == null) {
            return;
        }

        long now = System.currentTimeMillis();
        if (now >= startGrowUpTime && now <= endGrowUpTime) {
            if (activityLotteryDO.getLotteryPayType() != null && activityLotteryDO.getLotteryPayType() == LotteryPayTypeEnum.COIN.getValue()) {
                Long amount = activityLotteryDO.getLotteryPrice() * msg.getTimes();

                // 增加活动娱乐勋章积分值
                scoreProvider.addScore(msg.getUserId(), ScoreTypeEnum.GIFT_SEND_21, amount, String.format("参与%s%d次消耗%d钻石", activityLotteryDO.getLotteryName(), msg.getTimes(), amount), msg.getHisId(), msg.getUserId());
            }
        }
    }

    /**
     * 用户创建消息
     *
     * @param msg
     */
    @Async
    public void handleUserCreate(QueueUserCreateMsg msg) {

        Long userId = msg.getUserId();
        UserDO userDO = userDAO.selectById(userId);
        if (userDO == null) {
            for (int i = 0; i < 5; i++) {
                try {
                    Thread.sleep(1000L);
                    userDO = userDAO.selectById(userId);
                    if (userDO != null) {
                        break;
                    }
                } catch (Exception e) {

                }
            }
        }

        if (userDO == null) {
            log.error("用户不存在 {}", msg.getUserId());
            throw new RuntimeException("未找到用户");
        }

        // 创建钱包
        walletService.createUserWallet(userId);

        // 创建积分账户
        scoreService.createScoreAccount(userId);

        // 同步 IM 信息
        ImUserProfile imUserProfile = new ImUserProfile();
        BeanUtils.copyProperties(userDO, imUserProfile);
        imService.syncUserProfile(imUserProfile);

        // 小号禁言
        imService.setUserNoSpeaking(userId, Integer.MAX_VALUE, Integer.MAX_VALUE);

        // 头像处理
        String imgUrl = msg.getHeadImgUrl();
        if (!StringUtil.isNullOrEmpty(imgUrl)) {
            // 把头像转存到七牛
            userDO.setUserAvatar(fileService.saveNetworkImageToAvatar(imgUrl, userId));
            userDAO.updateUserAvatar(userDO.getUserId(), userDO.getUserAvatar());
        }

        // 写入es
        userSearchService.insertOrUpdate(userDO);

        // 事件埋点
        UserRegEventParam userRegEventParam = new UserRegEventParam();
        userRegEventParam.setUserId(userId);
        userRegEventParam.setChannelId(msg.getChannelId());
        triggerService.emitEvent(TriggerEventEnum.USER_REG, userRegEventParam);

        // 统计新增注册用户
        statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS, BigDecimal.ONE);
        ClientType clientType = ClientType.toEnum(msg.getClientType());
        switch (clientType) {
            case iOS:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS_IOS, BigDecimal.ONE);
                break;
            case Android:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS_ANDROID, BigDecimal.ONE);
                break;
            default:
                break;
        }

        // 统计邀请人数
        Long inviterUserId = msg.getInviterUserId();
        if (inviterUserId != null && inviterUserId > 0) {
            statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.INVITE_NUM, BigDecimal.ONE);
            statUserService.addUserStat(inviterUserId, StatUserTypeEnum.INVITE_NUM.getValue(), BigDecimal.ONE);
        }

        // 判断是否安装当日
        DeviceDO deviceDO = deviceDAO.selectById(msg.getDeviceId());
        if (deviceDO != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            if (sdf.format(new Date(deviceDO.getDeviceCreateTime())).equals(sdf.format(msg.getTimestamp()))) {
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS_INSTALL_DAILY, BigDecimal.ONE);
                switch (clientType) {
                    case iOS:
                        statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS_INSTALL_DAILY_IOS, BigDecimal.ONE);
                        break;
                    case Android:
                        statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_USERS_INSTALL_DAILY_ANDROID, BigDecimal.ONE);
                        break;
                    default:
                        break;
                }
            }
        }

        // 统计新增注册方式
        RegisterTypeEnum registerType = RegisterTypeEnum.toEnum(msg.getRegisterType());
        switch (registerType) {
            case SMS:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_SMS, BigDecimal.ONE);
                break;
            case WX:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_WX, BigDecimal.ONE);
                break;
            case QQ:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_QQ, BigDecimal.ONE);
                break;
            case WB:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_WB, BigDecimal.ONE);
                break;
            case HUAWEI:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_HUAWEI, BigDecimal.ONE);
                break;
            case ONE_KEY:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_ONE_KEY, BigDecimal.ONE);
                break;
            case FACEBOOK:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_FACEBOOK, BigDecimal.ONE);
                break;
            case GOOGLE:
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.REGISTER_BY_GOOGLE, BigDecimal.ONE);
                break;
            default:
                break;
        }

        // 性别
        Integer sex = msg.getSex();
        if (sex != null) {
            SexEnum sexEnum = SexEnum.toEnum(sex);
            switch (sexEnum) {
                case FEMALE:
                    statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.USER_SEX_FEMALE, BigDecimal.ONE);
                    break;
                case MALE:
                    statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.USER_SEX_MALE, BigDecimal.ONE);
                    break;
                case UNKNOWN:
                    statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.USER_SEX_UNKNOWN, BigDecimal.ONE);
                    break;
                default:
                    break;
            }
        }

        // 打招呼
        Integer greetEnable = Optional.ofNullable(channelDAO.selectGreetEnable(msg.getChannelId())).orElse(0);
        if (greetEnable == 1) {
            sendLiveGreet(userId);
        } else if (greetEnable == 2) {
            sendRtcGreet(userId);
        } else if (greetEnable == 3) {
            sendLiveGreet(userId);
            sendRtcGreet(userId);
        }

        // 客服招呼  跟渠道开关 没关系
        sendCustomerGreet(userId, msg.getChannelId(),
                "亲爱的用户，如果您在使用过程中遇到疑惑或问题，以及任何的想法和建议，欢迎您在这里向我述说，我必然会知无不言，服务至全！");

        try {
            if (deviceDO != null) {
                Long channelId = deviceDO.getChannelId();
                String baiduAKey = channelDAO.selectBaiduAKey(channelId);
                log.info("channelId:{},baiduAKey：{}", channelId, baiduAKey);
                String idfa = msg.getIdfa();
                String imeiMd5 = msg.getImeiMd5();
                log.info("idfa:{},imeiMd5:{}", idfa, imeiMd5);
                if (StringUtils.isNotEmpty(baiduAKey) && (StringUtils.isNotEmpty(idfa) || StringUtils.isNotEmpty(imeiMd5))) {
                    BaiduNoticeDTO baiduNotice = dataService.getBaiduNotice(idfa, imeiMd5);
                    if (baiduNotice != null) {
                        Long noticeId = baiduNotice.getNoticeId();
                        String callbackUrl = baiduNotice.getCallbackUrl();
                        log.info("noticeId:{},callbackUrl：{}", noticeId, callbackUrl);
                        if (StringUtils.isNotEmpty(callbackUrl)) {
                            boolean result = dataService.baiduCallback(callbackUrl, baiduAKey, BaiduATypeEnum.REGISTER, "0");
                            log.info("转化指标:{},上报结果:{}", BaiduATypeEnum.REGISTER, result);
                            if (result) {
                                dataService.updateBaiduNotice(noticeId, BaiduATypeEnum.REGISTER.getAType(), "0");
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.info("出现异常", e);
        }

        try {
            IpLocation ipLocation = null;
            // 如果是国内
            String areaCode = userService.getAreaCode(userId);
            if ("86".equals(areaCode)) {
                ipLocation = areaService.getIpLocation(msg.getClientIp());
            }
            if (ipLocation != null && ipLocation.getAreaId() != null) {
                userService.setAreaId(userId, ipLocation.getAreaId());
            }
        } catch (Exception e) {
            log.error("默认区域设置失败", e);
        }

        // 星芽注册用户通知第三方
        NoticeUserRegistered userRegistered = new NoticeUserRegistered();
        BeanUtils.copyProperties(userDO, userRegistered);
        userRegistered.setInviteUserId(msg.getInviterUserId());
        userRegistered.setChannelId(msg.getChannelId());
        thirdPartyService.sendNoticeUserRegistered(userRegistered);
    }

    /**
     * 用户编辑资料
     *
     * @param msg
     */
    @Async
    public void handleUserEdit(QueueUserEditMsg msg) {
        log.info("用户编辑资料 {}", JSON.toJSONString(msg));

        UserDO userDO = msg.getUserDO();

        // 同步 ES
        userSearchService.insertOrUpdate(userDO);

        UserDO dbUser = userDAO.selectById(userDO.getUserId());
        if (dbUser != null) {
            // 同步 IM
            ImUserProfile imUserProfile = new ImUserProfile();
            if (StrUtil.isNotEmpty(userDO.getUserName())) {
                imUserProfile.setUserId(userDO.getUserId());
                imUserProfile.setUserName(userDO.getUserName());
            }
            if (StrUtil.isNotEmpty(userDO.getUserAvatar())) {
                imUserProfile.setUserId(userDO.getUserId());
                imUserProfile.setUserAvatar(userDO.getUserAvatar());
            }
            if (userDO.getUserSex() != null) {
                imUserProfile.setUserId(userDO.getUserId());
                imUserProfile.setUserSex(userDO.getUserSex());
            }
            if (imUserProfile.getUserId() != null) {
                imService.syncUserProfile(imUserProfile);
            }
        }
    }

    /**
     * 积分等级提升
     *
     * @param msg
     */
    @Async
    public void handleScoreLevelUp(QueueScoreLevelUpMsg msg) {
        log.info("处理积分升级{}", JSON.toJSONString(msg));
        try {
            if (ScoreTypeEnum.WEALTH == ScoreTypeEnum.toEnum(msg.getScoreType())) {
                // 触发器
                ScoreLevelUpEventParam eventParam = new ScoreLevelUpEventParam();
                eventParam.setUserId(msg.getUserId());
                eventParam.setScoreType(msg.getScoreType());
                // 升级后的等级
                eventParam.setScoreLevel(msg.getNewLevel());
                eventParam.setScoreValue(msg.getScoreValue());
                triggerService.emitEvent(TriggerEventEnum.SCORE_LEVEL_UP, eventParam);

                // 解除禁言
                if (msg.getScoreLevel() < 2 && msg.getNewLevel() > 1) {
                    imService.setUserNoSpeaking(msg.getUserId(), 0, 0);
                }

                // 延迟执行 防止数据库未同步
                Thread.sleep(1000L);
                // 更新用户直播间缓存
                userService.refreshLiveUserProfile(msg.getUserId());
                rtcUserService.refreshRoomProfile(msg.getUserId());
            } else if (ScoreTypeEnum.STAR == ScoreTypeEnum.toEnum(msg.getScoreType())) {
                // 触发器
                ScoreLevelUpEventParam eventParam = new ScoreLevelUpEventParam();
                eventParam.setUserId(msg.getUserId());
                eventParam.setScoreType(msg.getScoreType());
                eventParam.setScoreLevel(msg.getScoreLevel());
                eventParam.setScoreValue(msg.getScoreValue());
                triggerService.emitEvent(TriggerEventEnum.SCORE_LEVEL_UP, eventParam);
                // 更新用户房间缓存
                Thread.sleep(1000L);
                rtcUserService.refreshRoomProfile(msg.getUserId());
            }

            if (ScoreTypeEnum.GIFT_SEND_21.equals(ScoreTypeEnum.toEnum(msg.getScoreType())) || ScoreTypeEnum.GIFT_RECEIVE_21.equals(ScoreTypeEnum.toEnum(msg.getScoreType()))) {
                ActivityMedalLevelUpLevelMsg medalLevelUpLevelMsg = new ActivityMedalLevelUpLevelMsg();
                DataLevelScoreDO nextLevelScore = dataService.getCurrentLevelScore(msg.getNewLevel(), ScoreTypeEnum.toEnum(msg.getScoreType()));
                BeanUtils.copyProperties(nextLevelScore, medalLevelUpLevelMsg);
                medalLevelUpLevelMsg.setUserId(msg.getUserId());

                String type = "";
                String desc = "";
                Integer styleType = 0;
                if (ScoreTypeEnum.GIFT_SEND_21.equals(ScoreTypeEnum.toEnum(msg.getScoreType()))) {
                    type = "娱乐勋章";
                    desc = "注：全站消耗钻石可升级勋章";
                    styleType = 1;
                    DataLevelScoreDO lastLevelScore = dataService.getLastLevelScore(ScoreTypeEnum.toEnum(msg.getScoreType()));
                    if (lastLevelScore.getScoreLevel().equals(nextLevelScore.getScoreLevel())) {
                        desc = "注：保持娱乐排行榜前3名才可拥有该勋章";
                        styleType = 3;
                    }
                } else if (ScoreTypeEnum.GIFT_RECEIVE_21.equals(ScoreTypeEnum.toEnum(msg.getScoreType()))) {
                    type = "偶像勋章";
                    desc = "注：收到面板礼物可升级勋章";
                    styleType = 2;
                }

                String body = String.format("恭喜「%s」%s升级为\"%s\"", userService.getUserName(msg.getUserId()), type, nextLevelScore.getLevelName());
                medalLevelUpLevelMsg.setDesc(desc);
                medalLevelUpLevelMsg.setStyleType(styleType);
                medalLevelUpLevelMsg.setMsg(body);
                medalLevelUpLevelMsg.setRoomId(rtcUserService.getUserStayRoomId(msg.getUserId()));
                medalLevelUpLevelMsg.setActivitiesMedalIdArr(scoreService.getScoreLevelIds(msg.getUserId()));
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, msg.getUserId(), 0L, medalLevelUpLevelMsg);

                // 系统消息
                SystemMsg systemMsg = new SystemMsg();
                systemMsg.setContentType(com.starbuds.server.common.constant.message.im.ContentType.TEXT);
                systemMsg.setTitle("年度人物");
                systemMsg.setBody(body);
                systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
                imService.sendCustomImMsg(null, msg.getUserId(), systemMsg, null);

                // 更新用户房间缓存
                Thread.sleep(1000L);
                rtcUserService.refreshRoomProfile(msg.getUserId());
            } else {
                // 升级消息
                ScoreLevelUpMsg levelUpMsg = new ScoreLevelUpMsg();
                levelUpMsg.setScoreLevel(msg.getNewLevel());
                levelUpMsg.setScoreType(msg.getScoreType());
                levelUpMsg.setScoreValue(msg.getScoreValue());
                levelUpMsg.setUserId(msg.getUserId());
                imService.sendCustomImMsg(null, msg.getUserId(), levelUpMsg, null);
            }

        } catch (Exception e) {
            log.error("积分等级提升失败", e);
        }
    }

    /**
     * 开通贵族
     *
     * @param msg
     */
    public void handleNobleNew(QueueNobleNewMsg msg) {
        UserCardVO userCard = userDAO.selectCardById(msg.getUserId());

        // 广播直播间
        LiveNobleNoticeMsg liveNobleNoticeMsg = new LiveNobleNoticeMsg();
        liveNobleNoticeMsg.setNobleLevel(msg.getNobleLevel());
        liveNobleNoticeMsg.setNoticeContent(String.format("天空一声巨响，原来是%s开通了%s贵族，恭喜恭喜！", userCard.getUserName(), NobleLevelEnum.toEnum(msg.getNobleLevel()).getDesc()));
        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, liveNobleNoticeMsg);
        imService.sendLiveRoomMsg(ImTargetType.ALL, null, null, liveNobleNoticeMsg);

        // 触发器
        NobleNewEventParam nobleNewEventParam = new NobleNewEventParam();
        nobleNewEventParam.setUserId(msg.getUserId());
        nobleNewEventParam.setNobleLevel(msg.getNobleLevel());
        nobleNewEventParam.setIsFromProps(msg.getIsFromProps());
        nobleNewEventParam.setDuration(msg.getDuration());
        nobleNewEventParam.setPropsId(msg.getPropsId());
        triggerService.emitEvent(TriggerEventEnum.NOBLE_NEW, nobleNewEventParam);

        // 刷新直播间缓存
        liveUserProvider.refreshLiveUserProfile(msg.getUserId());
    }

    /**
     * 续费贵族
     *
     * @param msg
     */
    public void handleNobleRenew(QueueNobleRenewalMsg msg) {
        UserCardVO userCard = userDAO.selectCardById(msg.getUserId());
        String sex = Optional.ofNullable(userCard.getUserSex()).orElse(1).equals(2) ? "姐" : "哥";

        // 广播直播间
        LiveNobleNoticeMsg liveNobleNoticeMsg = new LiveNobleNoticeMsg();
        liveNobleNoticeMsg.setNobleLevel(msg.getNobleLevel());
        liveNobleNoticeMsg.setNoticeContent(String.format("%s不在江湖，江湖却有%s的传说，恭喜%s成功续费%s贵族%d次！", sex, sex, userCard.getUserName(), NobleLevelEnum.toEnum(msg.getNobleLevel()).getDesc(), msg.getTimes()));
        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, liveNobleNoticeMsg);
        imService.sendLiveRoomMsg(ImTargetType.ALL, null, null, liveNobleNoticeMsg);

        // 触发器
        NobleRenewEventParam eventParam = new NobleRenewEventParam();
        eventParam.setUserId(msg.getUserId());
        eventParam.setNobleLevel(msg.getNobleLevel());
        eventParam.setDuration(msg.getDuration());
        eventParam.setTimes(msg.getTimes());
        triggerService.emitEvent(TriggerEventEnum.NOBLE_RENEW, eventParam);

        // 刷新直播间缓存
        liveUserProvider.refreshLiveUserProfile(msg.getUserId());
    }

    /**
     * 升级贵族
     *
     * @param msg
     */
    public void handleNobleUpgrade(QueueNobleUpgradeMsg msg) {
        UserCardVO userCard = userDAO.selectCardById(msg.getUserId());

        // 广播直播间
        LiveNobleNoticeMsg liveNobleNoticeMsg = new LiveNobleNoticeMsg();
        liveNobleNoticeMsg.setNobleLevel(msg.getNewNobleLevel());
        liveNobleNoticeMsg.setNoticeContent(String.format("祥云环绕，财气升腾，恭喜%s升级成为%s贵族！", userCard.getUserName(), NobleLevelEnum.toEnum(msg.getNewNobleLevel()).getDesc()));
        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, liveNobleNoticeMsg);
        imService.sendLiveRoomMsg(ImTargetType.ALL, null, null, liveNobleNoticeMsg);

        NobleUpgradeEventParam eventParam = new NobleUpgradeEventParam();
        eventParam.setUserId(msg.getUserId());
        eventParam.setOldNobleLevel(msg.getOldNobleLevel());
        eventParam.setNewNobleLevel(msg.getNewNobleLevel());
        triggerService.emitEvent(TriggerEventEnum.NOBLE_UPGRADE, eventParam);

        // 刷新直播间缓存
        liveUserProvider.refreshLiveUserProfile(msg.getUserId());
    }

    /**
     * 处理设备接入
     *
     * @param deviceAddMsg
     */
    public void handleDeviceAdd(QueueDeviceAddMsg deviceAddMsg) {
        Long channelId = deviceAddMsg.getChannelId();
        String idfa = deviceAddMsg.getIdfa();
        String imeiMd5 = deviceAddMsg.getImeiMd5();

        String baiduAKey = channelDAO.selectBaiduAKey(channelId);
        if (StringUtils.isEmpty(baiduAKey)) {
            return;
        }

        if (StringUtils.isNotEmpty(idfa) || StringUtils.isNotEmpty(imeiMd5)) {
            BaiduNoticeDTO noticeDTO = dataService.getBaiduNotice(idfa, imeiMd5);
            if (noticeDTO != null) {
                Long noticeId = noticeDTO.getNoticeId();
                String callbackUrl = noticeDTO.getCallbackUrl();
                if (StringUtils.isNotEmpty(callbackUrl)) {
                    boolean result = dataService.baiduCallback(callbackUrl, baiduAKey, BaiduATypeEnum.ACTIVATE, "0");
                    log.info("转化指标:{},上报结果:{}", BaiduATypeEnum.ACTIVATE, result);
                    if (result) {
                        dataService.updateBaiduNotice(noticeId, BaiduATypeEnum.ACTIVATE.getAType(), "0");
                    }
                }
            }
        }
    }

    /**
     * 处理添加身份证
     *
     * @param msg
     */
    public void handleIdCardAdd(QueueIdCardAddMsg msg) {
        log.info("收到普通队列消息-handleIdCardAdd");
        Long userId = msg.getUserId();
        String faceImg = msg.getFaceImg();
        // ocr识别
        OcrResultVO rest = ocrService.doOcr(faceImg);
        if (rest == null) {
            return;
        }
        Boolean isSuccess = rest.getSuccess();
        if (isSuccess != null && isSuccess) {
            log.info("orc识别结果:{}", rest);
            String idCard = rest.getNum();
            String realName = rest.getName();
            if (StringUtils.isEmpty(idCard) || StringUtils.isEmpty(realName)) {
                userIdCardDAO.updateOcrResult(userId, rest.getResult(), rest.getErrMsg(), CommonStatus.Deleted);
                return;
            }
            if (cloudAccountService.verifyId(idCard, realName)) {
                log.info("身份证实名验证成功");
                int userAge = UserUtil.getAgeCertID(idCard);
                if (userAge < 18) {
                    userIdCardDAO.updateOcrResult(userId, rest.getResult(), "账号认证未满18周岁，认证失败", CommonStatus.Disabled);
                    return;
                }
                // 这里做一个检查, 是否超过五个
                Integer cardNoCount = userAuthDAO.countByIdCardNo(idCard);
                if (cardNoCount > 4) {
                    userIdCardDAO.updateOcrResult(userId, rest.getResult(), "同一身份证只能认证5个账号", CommonStatus.Disabled);
                    return;
                }

                UserAuthDO userAuthDO = new UserAuthDO();
                userAuthDO.setUserId(userId);
                userAuthDO.setRealName(realName);
                userAuthDO.setIdCardNo(idCard);
                userAuthDAO.insertOrUpdate(userAuthDO);
                // 更新状态
                userIdCardDAO.updateOcrResult(userId, rest.getResult(), "实名认证通过", CommonStatus.Normal);
                userSearchService.syncUser(Arrays.asList(userId));
                return;
            }
            // 更新状态
            userIdCardDAO.updateOcrResult(userId, rest.getResult(), "实名认证失败", CommonStatus.Deleted);
            log.info("身份证实名验证失败");
            return;
        }
    }

    /**
     * 处理竞拍结束
     *
     * @param msg
     */
    public void handleAuctionEnd(QueueRtcAuctionEndMsg msg) {
        if (msg.getStatus() != null && msg.getStatus() == RtcAuctionStatus.WIN) {
            userWatchRelationService.newWatchRelationByAuction(msg.getBidDuration(), msg.getRtcRoomAuctionId(), msg.getWinUserId(), msg.getBidUserId(), msg.getBidItemId(), msg.getHighestBidPrice());
        }
    }

    /**
     * 处理竞拍赠送礼物
     *
     * @param msg
     */
    public void handleAuctionBid(QueueRtcAuctionBidMsg msg) {
        Long price = msg.getGiftTotalPrice();
        if (price == null) {
            price = 0L;
        }
        userWatchRelationService.increaseIntimacy(UserIntimacyCreateReasonEnum.AUCTION_GIFT,
                new BigDecimal(price).divide(new BigDecimal(100)), msg.getSendUserId(), msg.getReceiveUserId(), joinTwoObj(msg.getOrderId(), msg.getGiftId()));
        userWatchRelationService.increaseWatchValue(UserIntimacyCreateReasonEnum.AUCTION_GIFT, price, msg.getSendUserId(), msg.getReceiveUserId(), joinTwoObj(msg.getOrderId(), msg.getGiftId()), msg.getBidItemId(), msg.getRoomId(), msg.getRtcRoomAuctionId());
    }

    private String joinTwoObj(Object one, Object two) {
        if (one != null && two != null) {
            return one.toString() + two.toString();
        }
        if (one != null) {
            return one.toString();
        } else {
            return two.toString();
        }
    }

    /**
     * 用户在线状态改变
     *
     * @param msg
     */
    public void handleUserOnlineChange(QueueUserOnlineChangeMsg msg) {
        if (BoolIntVal.True == msg.getOnlineStatus()) {
            // 同步用户 IM 信息
            UserDO user = userDAO.selectById(msg.getUserId());
            if (user != null) {
                ImUserProfile imUserProfile = new ImUserProfile();
                BeanUtils.copyProperties(user, imUserProfile);
                imService.syncUserProfile(imUserProfile);
            }
        }
    }

    /**
     * 发送直播招呼
     *
     * @param userId
     */
    private void sendLiveGreet(Long userId) {
        List<Long> anchorUserIds = anchorDAO.selectRandomAnchorUserId();
        if (CollectionUtil.isNotEmpty(anchorUserIds)) {
            for (Long anchorUserId : anchorUserIds) {
                log.info("LIVE,打招呼的主播id：{}", anchorUserId);
                if (anchorUserId != null) {
                    String greetContent = dataService.getGreetContent(GreetTypeEnum.LIVE.getCode());
                    log.info("招呼内容：{}", greetContent);
                    if (StringUtils.isNotEmpty(greetContent)) {
                        imService.sendTextMsg(anchorUserId, userId, greetContent);
                    }
                }
            }
        }
    }

    /**
     * 发送语聊招呼
     *
     * @param userId
     */
    private void sendRtcGreet(Long userId) {
        Long sendUserId = userGrantAuthDAO.selectRandomUserId(GrantAuthTypeEnum.RTC_GREET.getCode());
        log.info("RTC,打招呼的用户id：{}", sendUserId);
        if (sendUserId != null) {
            String greetContent = dataService.getGreetContent(GreetTypeEnum.RTC.getCode());
            log.info("招呼内容：{}", greetContent);
            if (StringUtils.isNotEmpty(greetContent)) {
                imService.sendTextMsg(sendUserId, userId, greetContent);
            }
        }
    }

    /**
     * 发送客服招呼
     *
     * @param userId
     */
    private void sendCustomerGreet(Long userId, Long channelId, String greetContent) {
        // 客服id
        Long customerUserId = Optional.ofNullable(channelDAO.selectCustomerUserId(channelId)).orElse(channelDAO.selectCustomerUserId(0L));
        log.info("客服招呼，打招呼的客服id：{}", customerUserId);
        if (customerUserId != null) {
            imService.sendTextMsg(customerUserId, userId, greetContent);
        }
    }


    /**
     * 广告引流用户注册后处理动作
     * zz @param msg
     */
    @Async
    public void handleUserCreateToAd(QueueUserCreateMsg msg) {
        log.info("adCreateHandle::" + msg.getClientType() + ",idfa:" + msg.getIdfa() + ",Imei:" + msg.getImeiMd5());

        //1, "Web"  2, "iOS"  3, "Android"
        int os = msg.getClientType();
        if (os == 2) {
            String idfa = msg.getIdfa();
            if (StringUtils.isNotEmpty(idfa) && (!idfa.equals("00000000-0000-0000-0000-000000000000"))) {
                UserAdHisDo userAdHisDo = userAdHisDAO.getByIdfa(idfa);
                if (userAdHisDo != null && userAdHisDo.getAdId() > 0) {
                    String url = userAdHisDo.getCallbackUrl() + "&event_type=1";
                    String res = HttpUtil.get(url);
                    log.info("adUser3---------idfa::::" + res);
                    userAdHisDAO.setAdState(userAdHisDo.getAdId(), msg.getUserId());
                    return;
                }
            }

            String ip = msg.getClientIp();
            if (StringUtils.isNotEmpty(ip)) {
                UserAdHisDo userAdHisDo = userAdHisDAO.getByIp(ip);
                if (userAdHisDo != null && userAdHisDo.getAdId() > 0) {
                    String url = userAdHisDo.getCallbackUrl() + "&event_type=1";
                    String res = HttpUtil.get(url);
                    log.info("adUser3---------ip::::" + res);
                    userAdHisDAO.setAdState(userAdHisDo.getAdId(), msg.getUserId());
                    return;
                }
            }
        }

        //android
        if (os == 3) {
            String imei = msg.getImeiMd5();
            if (StringUtils.isNotEmpty(imei)) {
                UserAdHisDo userAdHisDo = userAdHisDAO.getByImei(imei);
                if (userAdHisDo != null && userAdHisDo.getAdId() > 0) {
                    String url = userAdHisDo.getCallbackUrl() + "&event_type=1";
                    String res = HttpUtil.get(url);
                    log.info("adUser3---------imei::::" + res);
                    userAdHisDAO.setAdState(userAdHisDo.getAdId(), msg.getUserId());
                    return;
                }
            }

            String ip = msg.getClientIp();
            if (StringUtils.isNotEmpty(ip)) {
                UserAdHisDo userAdHisDo = userAdHisDAO.getByIp(ip);
                if (userAdHisDo != null && userAdHisDo.getAdId() > 0) {
                    String url = userAdHisDo.getCallbackUrl() + "&event_type=1";
                    String res = HttpUtil.get(url);
                    log.info("adUser3---------ip::::" + res);
                    userAdHisDAO.setAdState(userAdHisDo.getAdId(), msg.getUserId());
                    return;
                }
            }
        }

    }

    /**
     * 用户首冲
     */
    @Async
    public void handleUserFristRecharge(QueueUserFirstRechargeMsg msg) {
        //调用巨量
        String config = releaseConfigService.getReleaseConfig("is_ad_create_user_open");
        if (StringUtils.isNotEmpty(config) && config.equals("open")) {
            Long userId = msg.getUserId();
            if (userId != null && userId > 0L) {
                UserAdHisDo userAdHisDo = userAdHisDAO.getByUserId(userId);
                if (userAdHisDo != null && userAdHisDo.getAdId() > 0) {
                    String url = userAdHisDo.getCallbackUrl() + "&event_type=2";
                    String res = HttpUtil.get(url);
                    log.info("adUserFirstRecharge---------::::" + res);
                    userAdHisDAO.setIsRecharge(userAdHisDo.getAdId());
                    return;
                }
            }
        }
    }

}
