package com.starbuds.server.service.user.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.QueueNiceNoExpireMsg;
import com.starbuds.server.common.pojo.mq.QueueNobleExpireMsg;
import com.starbuds.server.service.user.rocket.handler.DelayMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 延时消息处理
 *
 * @author Lay
 * @date 2020/3/2
 */
@Component
@Slf4j
public class DelayMsgListener implements MessageListener {
    @Resource
    DelayMsgHandler delayMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            switch (message.getTag()) {
                case RocketTag.NICE_NO_EXPIRE:
                    // 靓号过期消息
                    delayMsgHandler.handleNiceNoExpire(JSON.parseObject(message.getBody(), QueueNiceNoExpireMsg.class));
                    break;
                case RocketTag.NOBLE_EXPIRE:
                    delayMsgHandler.handleNobleExpire(JSON.parseObject(message.getBody(), QueueNobleExpireMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}
