package com.starbuds.server.service.user.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.user.provider.UserMedalStatusProvider;
import com.starbuds.server.service.user.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 普通队列消息监听
 *
 * @author Lay
 * @date 2020/1/3
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Resource
    private UserMedalStatusProvider userMedalStatusProvider;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        log.info("收到普通队列消息 {} {} {}", message.getTopic(), message.getTag(),message.getBody());
        try {
            switch (message.getTag()) {
                case RocketTag.GIFT_PAID:
                    // 礼物支付消息
                    normalMsgHandler.handleGiftPaid(JSON.parseObject(message.getBody(), QueueGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_GIFT_PAID:
                    // RTC 礼物支付消息
                    normalMsgHandler.handleRtcGiftPaid(JSON.parseObject(message.getBody(), QueueRtcGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_UNDER_MAI_GIFT_PAID:
                    // 语聊房麦下送礼物支付消息
                    normalMsgHandler.handleRtcGiftUnderMaiPaid(JSON.parseObject(message.getBody(), QueueRtcGiftUnderMaiPaidMsg.class));
                    break;
                case RocketTag.VOICE_GIFT_PAID:
                    // 语音速配 礼物支付消息
                    normalMsgHandler.handleVoiceGiftPaid(JSON.parseObject(message.getBody(), QueueVoiceGiftPaidMsg.class));
                    break;
                case RocketTag.PRIVATE_CHAT_GIFT_PAID:
                    // 私聊送礼物支付
                    normalMsgHandler.handlePrivateChatGiftPaid(JSON.parseObject(message.getBody(), QueueBaseGiftPaidMsg.class));
                    break;
                case RocketTag.USER_CREATE:
                    // 用户创建消息
                    normalMsgHandler.handleUserCreate(JSON.parseObject(message.getBody(), QueueUserCreateMsg.class));
                    break;
                case RocketTag.USER_CREATE_TO_AD:
                    // 广告引流创建用户完成
                    normalMsgHandler.handleUserCreateToAd(JSON.parseObject(message.getBody(), QueueUserCreateMsg.class));
                    break;
                case RocketTag.USER_FIRST_RECHARGE:
                    // 广告引流创建用户完成
                    normalMsgHandler.handleUserFristRecharge(JSON.parseObject(message.getBody(), QueueUserFirstRechargeMsg.class));
                    break;
                case RocketTag.USER_EDIT:
                    // 用户创建消息
                    normalMsgHandler.handleUserEdit(JSON.parseObject(message.getBody(), QueueUserEditMsg.class));
                    break;
                case RocketTag.SCORE_LEVEL_UP:
                    // 积分等级提升消息
                    normalMsgHandler.handleScoreLevelUp(JSON.parseObject(message.getBody(), QueueScoreLevelUpMsg.class));
                    break;
                case RocketTag.NOBLE_NEW:
                    // 开通贵族消息
                    normalMsgHandler.handleNobleNew(JSON.parseObject(message.getBody(), QueueNobleNewMsg.class));
                    break;
                case RocketTag.NOBLE_RENEW:
                    // 续费贵族消息
                    normalMsgHandler.handleNobleRenew(JSON.parseObject(message.getBody(), QueueNobleRenewalMsg.class));
                    break;
                case RocketTag.NOBLE_UPGRADE:
                    // 升级贵族消息
                    normalMsgHandler.handleNobleUpgrade(JSON.parseObject(message.getBody(), QueueNobleUpgradeMsg.class));
                    break;
                case RocketTag.DEVICE_CONNECT:
                    // 设备接入
                    normalMsgHandler.handleDeviceAdd(JSON.parseObject(message.getBody(), QueueDeviceAddMsg.class));
                    break;
                case RocketTag.ID_CARD_ADD:
                    // 添加身份证
                    normalMsgHandler.handleIdCardAdd(JSON.parseObject(message.getBody(), QueueIdCardAddMsg.class));
                    break;
                case RocketTag.USER_ONLINE_CHANGE:
                    normalMsgHandler.handleUserOnlineChange(JSON.parseObject(message.getBody(), QueueUserOnlineChangeMsg.class));
                    break;
                // RTC 拍卖结束
                case RocketTag.RTC_AUCTION_END:
                    normalMsgHandler.handleAuctionEnd(JSON.parseObject(message.getBody(), QueueRtcAuctionEndMsg.class));
                    break;
                // RTC 赠送礼物
                case RocketTag.RTC_AUCTION_BID:
                    normalMsgHandler.handleAuctionBid(JSON.parseObject(message.getBody(), QueueRtcAuctionBidMsg.class));
                    break;
                case RocketTag.LOTTERY_GAME_COMPLETE:
                    // 太空旅行，月光宝盒等抽奖活动抽奖完成消息
                    normalMsgHandler.handleActivityLotteryGameComplete(JSON.parseObject(message.getBody(), QueueLotteryGameCompleteMsg.class));
                    break;
                case RocketTag.SEND_GIFT_SNAPSHOT:
                    userMedalStatusProvider.updateMedalStatusToShiny(JSON.parseObject(message.getBody(), QueueSendGiftSnapShotMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error(String.format("消息处理失败-%s-%s", message.getTag(), JSON.parseObject(message.getBody(), JSONObject.class)), e);
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}
