package com.starbuds.server.service.user.util;

import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

/**
 * 用户签名工具
 *
 * @author Lay
 * @date 2020/3/13
 */
@Slf4j
public class UserSignUtil {

    public static final String SIGN = "sign";

    /**
     * 构建签名原⽂ 按字⺟顺序
     *
     * @param params 参数与值
     * @return
     */
    private static String getToSign(Map<String, Object> params) {
        if (params == null || params.size() == 0) {
            return "";
        }

        StringBuilder sbToSign = new StringBuilder();
        Map<String, Object> toSign = new TreeMap<>(params);
        for (Map.Entry<String, Object> entry : toSign.entrySet()) {
            // 忽略签名字段
            if (SIGN.equals(entry.getKey())) {
                continue;
            }
            sbToSign.append("&");
            sbToSign.append(entry.getKey());
            sbToSign.append("=");
            sbToSign.append(Convert.toStr(entry.getValue(), ""));
        }

        return sbToSign.substring(1);
    }

    /**
     * MD5签名
     *
     * @param params
     * @param key
     * @return
     */
    public static String md5Sign(Map<String, Object> params, String key) {

        String toSign = getToSign(params);
        if (StringUtils.isNotBlank(toSign)) {
            toSign += "&";
        }
        String sign = String.format("%skey=%s", toSign, key);
        String result = SecureUtil.md5(sign).toUpperCase();
        log.info("sign={} result={}", sign, result);
        return result;
    }

    /**
     * 验签
     *
     * @param params
     * @param key
     * @param sign
     * @return
     */
    public static boolean checkSign(Map<String, Object> params, String key, String sign) {
        String signStr = md5Sign(params, key);
        return signStr.equals(sign);
    }

    /**
     * 验签
     *
     * @param jsonObject
     * @param key
     * @param sign
     * @return
     */
    public static boolean checkSign(JSONObject jsonObject, String key, String sign) {
        Map<String, Object> params = new HashMap<>();
        for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
            params.put(entry.getKey(), entry.getValue());
        }
        return checkSign(params, key, sign);
    }
}
